/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Host
 * Copyright (c) 2004-2013 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    usbh_config.h
 * Purpose: USB Host Library Configuration
 * Rev.:    V5.04
 *----------------------------------------------------------------------------*/

#include <stdint.h>
#include <stdbool.h>
#include "cmsis_os.h"
#include "usb_def.h"
#include "usb_adc.h"
#include "usb_cdc.h"
#include "usb_hid.h"
#include "usb_msc.h"
#include "rl_usbh.h"
#include "rl_usbd.h"

#pragma thumb
#pragma O3


/*------------------------------------------------------------------------------
 *      USB Host Global Macros
 *----------------------------------------------------------------------------*/

#define USBHn_DRIVER_(n)                Driver_USBH##n
#define USBHn_DRIVER(n)                 USBHn_DRIVER_(n)

#if    (defined(USBH3_HC_NUM))
#define LAST_DEVICE_COMPILE             1
#elif  (defined(USBH2_HC_NUM) && !defined(RTE_USB_Host_3))
#define LAST_DEVICE_COMPILE             1
#elif  (defined(USBH1_HC_NUM) && !defined(RTE_USB_Host_3) && !defined(RTE_USB_Host_2))
#define LAST_DEVICE_COMPILE             1
#elif  (defined(USBH0_HC_NUM) && !defined(RTE_USB_Host_3) && !defined(RTE_USB_Host_2) && !defined(RTE_USB_Host_1))
#define LAST_DEVICE_COMPILE             1
#endif


/*------------------------------------------------------------------------------
 *      USB Host 0 Macros, Calculations and Constants for Library
 *----------------------------------------------------------------------------*/

#if    (defined(USBH0_HC_NUM) && defined(RTE_USB_Host_0))

extern  ARM_DRIVER_USBH      USBHn_DRIVER(USBH0_HC_NUM);
const   ARM_DRIVER_USBH     *usbh0_hcd_ptr = &USBHn_DRIVER(USBH0_HC_NUM);

#if    (USBH0_HC_MEM_POOL_RELOC == 1)
#define __AT_MEM_POOL0   __at(USBH0_HC_MEM_POOL_ADDR)
#else
#define __AT_MEM_POOL0
#endif

        uint32_t         usbh0_mem_pool[(USBH0_HC_MEM_POOL_SIZE+3)/4] __AT_MEM_POOL0;
const   uint32_t         usbh0_mem_pool_size = ((USBH0_HC_MEM_POOL_SIZE+3)/4)*4;
#endif


/*------------------------------------------------------------------------------
 *      USB Host 1 Macros, Calculations and Constants for Library
 *----------------------------------------------------------------------------*/

#if    (defined(USBH1_HC_NUM) && defined(RTE_USB_Host_1))

extern  ARM_DRIVER_USBH      USBHn_DRIVER(USBH1_HC_NUM);
const   ARM_DRIVER_USBH     *usbh1_hcd_ptr = &USBHn_DRIVER(USBH1_HC_NUM);

#if    (USBH1_HC_MEM_POOL_RELOC == 1)
#define __AT_MEM_POOL1   __at(USBH1_HC_MEM_POOL_ADDR)
#else
#define __AT_MEM_POOL1
#endif

        uint32_t         usbh1_mem_pool[(USBH1_HC_MEM_POOL_SIZE+3)/4] __AT_MEM_POOL1;
const   uint32_t         usbh1_mem_pool_size = ((USBH1_HC_MEM_POOL_SIZE+3)/4)*4;
#endif


/*------------------------------------------------------------------------------
 *      USB Host 2 Macros, Calculations and Constants for Library
 *----------------------------------------------------------------------------*/

#if    (defined(USBH2_HC_NUM) && defined(RTE_USB_Host_2))

extern  ARM_DRIVER_USBH      USBHn_DRIVER(USBH2_HC_NUM);
const   ARM_DRIVER_USBH     *usbh2_hcd_ptr = &USBHn_DRIVER(USBH2_HC_NUM);

#if    (USBH2_HC_MEM_POOL_RELOC == 1)
#define __AT_MEM_POOL2   __at(USBH2_HC_MEM_POOL_ADDR)
#else
#define __AT_MEM_POOL2
#endif

        uint32_t         usbh2_mem_pool[(USBH2_HC_MEM_POOL_SIZE+3)/4] __AT_MEM_POOL2;
const   uint32_t         usbh2_mem_pool_size = ((USBH2_HC_MEM_POOL_SIZE+3)/4)*4;
#endif


/*------------------------------------------------------------------------------
 *      USB Host 3 Macros, Calculations and Constants for Library
 *----------------------------------------------------------------------------*/

#if    (defined(USBH3_HC_NUM) && defined(RTE_USB_Host_3))

extern  ARM_DRIVER_USBH      USBHn_DRIVER(USBH3_HC_NUM);
const   ARM_DRIVER_USBH     *usbh3_hcd_ptr = &USBHn_DRIVER(USBH3_HC_NUM);

#if    (USBH3_HC_MEM_POOL_RELOC == 1)
#define __AT_MEM_POOL3   __at(USBH3_HC_MEM_POOL_ADDR)
#else
#define __AT_MEM_POOL3
#endif

        uint32_t         usbh3_mem_pool[(USBH3_HC_MEM_POOL_SIZE+3)/4] __AT_MEM_POOL3;
const   uint32_t         usbh3_mem_pool_size = ((USBH3_HC_MEM_POOL_SIZE+3)/4)*4;
#endif


/*------------------------------------------------------------------------------
 *      USB Host 0..3 Configurations
 *----------------------------------------------------------------------------*/

#if    (defined(LAST_DEVICE_COMPILE))

#ifndef USBH_MSC_NUM
#define USBH_MSC_NUM       0
#endif

#ifndef USBH_HID_NUM
#define USBH_HID_NUM       0
#endif

#ifndef USBH_CLS_NUM
#define USBH_CLS_NUM       0
#endif

#if     defined(RTE_USB_Host_0)
#define USBH0_HC           1
#else
#define USBH0_HC           0
#endif

#if     defined(RTE_USB_Host_1)
#define USBH1_HC           1
#else
#define USBH1_HC           0
#endif

#if     defined(RTE_USB_Host_2)
#define USBH2_HC           1
#else
#define USBH2_HC           0
#endif

#if     defined(RTE_USB_Host_3)
#define USBH3_HC           1
#else
#define USBH3_HC           0
#endif

#define USBH_HC_NUM       (USBH0_HC + USBH1_HC + USBH2_HC + USBH3_HC)
#define USBH_HC_MSK       (0x0F >> (4 - USBH_HC_NUM))
#define USBH_OHCI_NUM     (USBH_HC_NUM)
#define USBH_EHCI_NXP_NUM (USBH_HC_NUM)
#define USBH_DCD_NUM      ((USBH_MSC_NUM > 0) + (USBH_HID_NUM > 0) + (USBH_CLS_NUM > 0))
#define USBH_DEV_NUM      ( USBH_MSC_NUM      +  USBH_HID_NUM      +  USBH_CLS_NUM     )

                                        /* Define constants                   */
        const uint8_t usbh_hc_num  = USBH_HC_NUM;
        const uint8_t usbh_hc_msk  = USBH_HC_MSK;
        const uint8_t usbh_dcd_num = USBH_DCD_NUM;
        const uint8_t usbh_dev_num = USBH_DEV_NUM;
        const uint8_t usbh_msc_num = USBH_MSC_NUM;
        const uint8_t usbh_hid_num = USBH_HID_NUM;
        const uint8_t usbh_cls_num = USBH_CLS_NUM;

extern  const ARM_DRIVER_USBH *usbh0_hcd_ptr;
extern  const ARM_DRIVER_USBH *usbh1_hcd_ptr;
extern  const ARM_DRIVER_USBH *usbh2_hcd_ptr;
extern  const ARM_DRIVER_USBH *usbh3_hcd_ptr;

                                        /* Register Host Controller Drivers   */
const   ARM_DRIVER_USBH ** const usbh_hcd_ptr[USBH_HC_NUM] = { 
#if     defined(RTE_USB_Host_0)
                     &usbh0_hcd_ptr
#endif
#if     defined(RTE_USB_Host_1)
                   , &usbh1_hcd_ptr
#endif
#if     defined(RTE_USB_Host_2)
                   , &usbh2_hcd_ptr
#endif
#if     defined(RTE_USB_Host_3)
                   , &usbh3_hcd_ptr
#endif
};

#if 0
                                        /* Register OHCI Hw Driver              */
const   USBH_HWD_OHCI * const usbh_hwd_ohci_ptr[USBH_OHCI_NUM] = { 
#if     defined(RTE_USB_Host_0)
                     &usbh0_hwd_ohci_ptr
#endif
#if     defined(RTE_USB_Host_1)
                   , &usbh1_hwd_ohci_ptr
#endif
#if     defined(RTE_USB_Host_2)
                   , &usbh2_hwd_ohci_ptr
#endif
#if     defined(RTE_USB_Host_3)
                   , &usbh3_hwd_ohci_ptr
#endif
};

                                        /* Register NXP EHCI Hw Driver        */
const   USBH_HWD_EHCI * const usbh_hwd_ehci_NXP_ptr[USBH_EHCI_NXP_NUM] = { 
#if     defined(RTE_USB_Host_0)
                     &usbh0_hwd_ehci_NXP_ptr
#endif
#if     defined(RTE_USB_Host_1)
                   , &usbh1_hwd_ehci_NXP_ptr
#endif
#if     defined(RTE_USB_Host_2)
                   , &usbh2_hwd_ehci_NXP_ptr
#endif
#if     defined(RTE_USB_Host_3)
                   , &usbh3_hwd_ehci_NXP_ptr
#endif
};
#endif

#if     defined(RTE_USB_Host_0)
extern  uint32_t         usbh0_mem_pool[];
extern  const uint32_t   usbh0_mem_pool_size;
#endif
#if     defined(RTE_USB_Host_1)
extern  uint32_t         usbh1_mem_pool[];
extern  const uint32_t   usbh1_mem_pool_size;
#endif
#if     defined(RTE_USB_Host_2)
extern  uint32_t         usbh2_mem_pool[];
extern  const uint32_t   usbh2_mem_pool_size;
#endif
#if     defined(RTE_USB_Host_3)
extern  uint32_t         usbh3_mem_pool[];
extern  const uint32_t   usbh3_mem_pool_size;
#endif

        uint32_t * const usbh_mem_pool_ptr [USBH_HC_NUM] = {
#if     defined(RTE_USB_Host_0)
                     usbh0_mem_pool
#endif
#if     defined(RTE_USB_Host_1)
                   , usbh1_mem_pool
#endif
#if     defined(RTE_USB_Host_2)
                   , usbh2_mem_pool
#endif
#if     defined(RTE_USB_Host_3)
                   , usbh3_mem_pool
#endif
};
const   uint32_t * const usbh_mem_pool_size_ptr [USBH_HC_NUM] = {
#if     defined(RTE_USB_Host_0)
                     &usbh0_mem_pool_size
#endif
#if     defined(RTE_USB_Host_1)
                   , &usbh0_mem_pool_size
#endif
#if     defined(RTE_USB_Host_2)
                   , &usbh0_mem_pool_size
#endif
#if     defined(RTE_USB_Host_3)
                   , &usbh0_mem_pool_size
#endif
};

#if    (USBH_MSC_NUM)
extern  USBH_DCD      usbh_dcd_msc;
#endif
#if    (USBH_HID_NUM)
extern  USBH_DCD      usbh_dcd_hid;
#endif
#if    (USBH_CLS_NUM)
extern  USBH_DCD      usbh_dcd_cls;
#endif

                                        /* Register Class Drivers             */
const   USBH_DCD * const usbh_dcd_ptr[USBH_DCD_NUM] = { 
#if    (USBH_MSC_NUM)
                     &usbh_dcd_msc,
#endif
#if    (USBH_HID_NUM)
                     &usbh_dcd_hid,
#endif
#if    (USBH_CLS_NUM)
                     &usbh_dcd_cls
#endif
};

extern  void USBH_CoreThread (void const *arg);
osThreadDef (USBH_CoreThread, osPriorityNormal, USBH_HC_NUM, NULL);
const   osThreadDef_t * const usbh_core_thread_def_ptr[USBH_HC_NUM] = {
#if     defined(RTE_USB_Host_0)
                      osThread(USBH_CoreThread)
#endif
#if     defined(RTE_USB_Host_1)
                    , osThread(USBH_CoreThread)
#endif
#if     defined(RTE_USB_Host_2)
                    , osThread(USBH_CoreThread)
#endif
#if     defined(RTE_USB_Host_3)
                    , osThread(USBH_CoreThread)
#endif
};
osThreadId usbh_core_thread_id[USBH_HC_NUM];

#define USBH_DEBOUNCE_IN_MS 100

const   uint16_t usbh_debounce_in_ms[USBH_HC_NUM] = {
#if     defined(RTE_USB_Host_0)
                      USBH_DEBOUNCE_IN_MS
#endif
#if     defined(RTE_USB_Host_1)
                    , USBH_DEBOUNCE_IN_MS
#endif
#if     defined(RTE_USB_Host_2)
                    , USBH_DEBOUNCE_IN_MS
#endif
#if     defined(RTE_USB_Host_3)
                    , USBH_DEBOUNCE_IN_MS
#endif
};

extern  void USBH_ConDisconDebounce (void const * arg);
#if     defined(RTE_USB_Host_0)
osTimerDef (USBH_DebounceTimer0, USBH_ConDisconDebounce);
#endif
#if     defined(RTE_USB_Host_1)
osTimerDef (USBH_DebounceTimer1, USBH_ConDisconDebounce);
#endif
#if     defined(RTE_USB_Host_2)
osTimerDef (USBH_DebounceTimer2, USBH_ConDisconDebounce);
#endif
#if     defined(RTE_USB_Host_3)
osTimerDef (USBH_DebounceTimer3, USBH_ConDisconDebounce);
#endif
const   osTimerDef_t * const usbh_debounce_timer_def_ptr[USBH_HC_NUM] = {
#if     defined(RTE_USB_Host_0)
                      osTimer(USBH_DebounceTimer0)
#endif
#if     defined(RTE_USB_Host_1)
                    , osTimer(USBH_DebounceTimer1)
#endif
#if     defined(RTE_USB_Host_2)
                    , osTimer(USBH_DebounceTimer2)
#endif
#if     defined(RTE_USB_Host_3)
                    , osTimer(USBH_DebounceTimer3)
#endif
};
osTimerId usbh_debounce_timer_id[USBH_HC_NUM];

extern  void USBH_SignalPortEvent     (uint8_t ctrl, uint8_t port, ARM_USBH_EVENT event);
extern  void USBH_SignalEndpointEvent (uint8_t ctrl, ARM_USBH_EP_HANDLE ep_hndl, ARM_USBH_URB *ptr_urb);
#if     defined(RTE_USB_Host_0)
        void USBH0_SignalPortEvent    (uint8_t port, ARM_USBH_EVENT event)                { USBH_SignalPortEvent    (0, port, event);      }
        void USBH0_SignalEndpointEvent(ARM_USBH_EP_HANDLE ep_hndl, ARM_USBH_URB *ptr_urb) { USBH_SignalEndpointEvent(0, ep_hndl, ptr_urb); }
#endif

#if     defined(RTE_USB_Host_1)
        void USBH1_SignalPortEvent    (uint8_t port, ARM_USBH_EVENT event)                { USBH_SignalPortEvent    (1, port, event);      }
        void USBH1_SignalEndpointEvent(ARM_USBH_EP_HANDLE ep_hndl, ARM_USBH_URB *ptr_urb) { USBH_SignalEndpointEvent(1, ep_hndl, ptr_urb); }
#endif

#if     defined(RTE_USB_Host_2)
        void USBH2_SignalPortEvent    (uint8_t port, ARM_USBH_EVENT event)                { USBH_SignalPortEvent    (2, port, event);      }
        void USBH2_SignalEndpointEvent(ARM_USBH_EP_HANDLE ep_hndl, ARM_USBH_URB *ptr_urb) { USBH_SignalEndpointEvent(2, ep_hndl, ptr_urb); }
#endif

#if     defined(RTE_USB_Host_3)
        void USBH3_SignalPortEvent    (uint8_t port, ARM_USBH_EVENT event)                { USBH_SignalPortEvent    (3, port, event);      }
        void USBH3_SignalEndpointEvent(ARM_USBH_EP_HANDLE ep_hndl, ARM_USBH_URB *ptr_urb) { USBH_SignalEndpointEvent(3, ep_hndl, ptr_urb); }
#endif

#if    (USBH_HC_NUM > 0)
        ARM_USBH_SignalPortEvent_t const usbh_cb_port_event_ptr[USBH_HC_NUM] = {
#if     defined(RTE_USB_Host_0)
                      USBH0_SignalPortEvent
#endif
#if     defined(RTE_USB_Host_1)
                    , USBH1_SignalPortEvent
#endif
#if     defined(RTE_USB_Host_2)
                    , USBH2_SignalPortEvent
#endif
#if     defined(RTE_USB_Host_3)
                    , USBH3_SignalPortEvent 
#endif
};

        ARM_USBH_SignalEndpointEvent_t const usbh_cb_endpoint_event_ptr[USBH_HC_NUM] = {
#if     defined(RTE_USB_Host_0)
                      USBH0_SignalEndpointEvent
#endif
#if     defined(RTE_USB_Host_1)
                    , USBH1_SignalEndpointEvent
#endif
#if     defined(RTE_USB_Host_2)
                    , USBH2_SignalEndpointEvent
#endif
#if     defined(RTE_USB_Host_3)
                    , USBH3_SignalEndpointEvent
#endif
};

ARM_DRV_VERSION         usbh_drv_version [USBH_HC_NUM] = { 0 };
ARM_USBH_CAPABILITIES   usbh_capabilities[USBH_HC_NUM] = { 0 };

USBH_HCI  usbh_hci [USBH_HC_NUM]  = { 0 };
#endif

USBH_DEV  usbh_dev [USBH_DEV_NUM] = { 0 };

#if      (USBH_MSC_NUM)
USBH_MSC  usbh_msc [USBH_MSC_NUM] = { 0 };
#endif
#if      (USBH_HID_NUM)
USBH_HID  usbh_hid [USBH_HID_NUM] = { 0 };
extern    void USBH_HID_Thread (void const *arg);
osThreadId usbh_hid_thread_id[USBH_HID_NUM];
osThreadDef(USBH_HID_Thread, osPriorityAboveNormal, USBH_HID_NUM, NULL);
const     osThreadDef_t * const usbh_hid_thread_def_ptr[USBH_HID_NUM] = {
#if      (USBH_HID_NUM > 0)
                      osThread(USBH_HID_Thread)
#endif
#if      (USBH_HID_NUM > 1)
                    , osThread(USBH_HID_Thread)
#endif
#if      (USBH_HID_NUM > 2)
                    , osThread(USBH_HID_Thread)
#endif
#if      (USBH_HID_NUM > 3)
                    , osThread(USBH_HID_Thread) 
#endif
};
#endif
#if      (USBH_CLS_NUM)
USBH_CLS  usbh_cls [USBH_CLS_NUM] = { 0 };
#endif

#endif
