/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2013 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    usbd_config.h
 * Purpose: USB Device Library Configuration
 * Rev.:    V5.04
 *----------------------------------------------------------------------------*/

#include <stdint.h>
#include <stdbool.h>
#include "cmsis_os.h"
#include "usb_def.h"
#include "usb_adc.h"
#include "usb_cdc.h"
#include "usb_hid.h"
#include "usb_msc.h"
#include "rl_usbh.h"
#include "rl_usbd.h"

#pragma thumb
#pragma O3


/*------------------------------------------------------------------------------
 *      USB Device Global Macros
 *----------------------------------------------------------------------------*/

#define MAX_OF_2(a,b)                   ((a)>(b)?(a):(b))
#define MAX_OF_3(a,b,c)                 MAX_OF_2(MAX_OF_2(a,b),c)
#define MAX_OF_4(a,b,c,d)               MAX_OF_2(MAX_OF_3(a,b,c),d)
#define MAX_OF_5(a,b,c,d,e)             MAX_OF_2(MAX_OF_4(a,b,c,d),e)
#define MAX_OF_6(a,b,c,d,e,f)           MAX_OF_2(MAX_OF_5(a,b,c,d,e),f)
#define MAX_OF_7(a,b,c,d,e,f,g)         MAX_OF_2(MAX_OF_6(a,b,c,d,e,f),g)
#define MAX_OF_8(a,b,c,d,e,f,g,h)       MAX_OF_2(MAX_OF_7(a,b,c,d,e,f,g),h)
#define MAX_OF_9(a,b,c,d,e,f,g,h,i)     MAX_OF_2(MAX_OF_8(a,b,c,d,e,f,g,h),i)
#define MAX_OF_10(a,b,c,d,e,f,g,h,i,j)  MAX_OF_2(MAX_OF_9(a,b,c,d,e,f,g,h,i),j)
#define MAX_OF_11(a,b,c,d,e,f,g,h,i,j,k)  MAX_OF_2(MAX_OF_10(a,b,c,d,e,f,g,h,i,j),k)
#define MAX_OF_12(a,b,c,d,e,f,g,h,i,j,k,l) MAX_OF_2(MAX_OF_11(a,b,c,d,e,f,g,h,i,j,k),l)

#define USBDn_c_NUM(n,c)               (USBD##n##_##c##0 + USBD##n##_##c##1 + USBD##n##_##c##2 + USBD##n##_##c##3)
#define USBDn_DRIVER_(n)                Driver_USBD##n
#define USBDn_DRIVER(n)                 USBDn_DRIVER_(n)


/*------------------------------------------------------------------------------
 *      USB Device 0 Macros, Calculations and Constants for Library
 *----------------------------------------------------------------------------*/

#ifndef USBD0_MACROS
#define USBD0_MACROS                    1

#if    (defined(RTE_USB_Device_HID_0) && (USBD_HID0_DEV == 0))
#define USBD0_HID0                      1
#define USBD0_HID0_EP_INT_IN            USBD_HID0_EP_INT_IN
#define USBD0_HID0_EP_INT_OUT           USBD_HID0_EP_INT_OUT
#define USBD0_HID0_IF_NUM               1
#if   ((USBD_HID0_EP_INT_IN ==  1) || (USBD_HID0_EP_INT_OUT ==  1))
#ifdef  USBD0_EP1 
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  2) || (USBD_HID0_EP_INT_OUT ==  2))
#ifdef  USBD0_EP2 
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  3) || (USBD_HID0_EP_INT_OUT ==  3))
#ifdef  USBD0_EP3 
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  4) || (USBD_HID0_EP_INT_OUT ==  4))
#ifdef  USBD0_EP4 
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  5) || (USBD_HID0_EP_INT_OUT ==  5))
#ifdef  USBD0_EP5 
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  6) || (USBD_HID0_EP_INT_OUT ==  6))
#ifdef  USBD0_EP6 
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  7) || (USBD_HID0_EP_INT_OUT ==  7))
#ifdef  USBD0_EP7 
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  8) || (USBD_HID0_EP_INT_OUT ==  8))
#ifdef  USBD0_EP8 
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  9) || (USBD_HID0_EP_INT_OUT ==  9))
#ifdef  USBD0_EP9 
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 10) || (USBD_HID0_EP_INT_OUT == 10))
#ifdef  USBD0_EP10 
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 11) || (USBD_HID0_EP_INT_OUT == 11))
#ifdef  USBD0_EP11 
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 12) || (USBD_HID0_EP_INT_OUT == 12))
#ifdef  USBD0_EP12 
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 13) || (USBD_HID0_EP_INT_OUT == 13))
#ifdef  USBD0_EP13 
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 14) || (USBD_HID0_EP_INT_OUT == 14))
#ifdef  USBD0_EP14 
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 15) || (USBD_HID0_EP_INT_OUT == 15))
#ifdef  USBD0_EP15 
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_hid_thread_id[0]
#endif 
#else
#define USBD0_HID0                      0
#define USBD0_HID0_EP_INT_IN            0
#define USBD0_HID0_EP_INT_OUT           0
#define USBD0_HID0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_1) && (USBD_HID1_DEV == 0))
#define USBD0_HID1                      1
#define USBD0_HID1_EP_INT_IN            USBD_HID1_EP_INT_IN
#define USBD0_HID1_EP_INT_OUT           USBD_HID1_EP_INT_OUT
#define USBD0_HID1_IF_NUM               1
#if   ((USBD_HID1_EP_INT_IN ==  1) || (USBD_HID1_EP_INT_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  2) || (USBD_HID1_EP_INT_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  3) || (USBD_HID1_EP_INT_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  4) || (USBD_HID1_EP_INT_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  5) || (USBD_HID1_EP_INT_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  6) || (USBD_HID1_EP_INT_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  7) || (USBD_HID1_EP_INT_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  8) || (USBD_HID1_EP_INT_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  9) || (USBD_HID1_EP_INT_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 10) || (USBD_HID1_EP_INT_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 11) || (USBD_HID1_EP_INT_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 12) || (USBD_HID1_EP_INT_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 13) || (USBD_HID1_EP_INT_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 14) || (USBD_HID1_EP_INT_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 15) || (USBD_HID1_EP_INT_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_hid_thread_id[1]
#endif 
#else
#define USBD0_HID1                      0
#define USBD0_HID1_EP_INT_IN            0
#define USBD0_HID1_EP_INT_OUT           0
#define USBD0_HID1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_2) && (USBD_HID2_DEV == 0))
#define USBD0_HID2                      1
#define USBD0_HID2_EP_INT_IN            USBD_HID2_EP_INT_IN
#define USBD0_HID2_EP_INT_OUT           USBD_HID2_EP_INT_OUT
#define USBD0_HID2_IF_NUM               1
#if   ((USBD_HID2_EP_INT_IN ==  1) || (USBD_HID2_EP_INT_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  2) || (USBD_HID2_EP_INT_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  3) || (USBD_HID2_EP_INT_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  4) || (USBD_HID2_EP_INT_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  5) || (USBD_HID2_EP_INT_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  6) || (USBD_HID2_EP_INT_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  7) || (USBD_HID2_EP_INT_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  8) || (USBD_HID2_EP_INT_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  9) || (USBD_HID2_EP_INT_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 10) || (USBD_HID2_EP_INT_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 11) || (USBD_HID2_EP_INT_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 12) || (USBD_HID2_EP_INT_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 13) || (USBD_HID2_EP_INT_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 14) || (USBD_HID2_EP_INT_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 15) || (USBD_HID2_EP_INT_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_hid_thread_id[2]
#endif 
#else
#define USBD0_HID2                      0
#define USBD0_HID2_EP_INT_IN            0
#define USBD0_HID2_EP_INT_OUT           0
#define USBD0_HID2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_3) && (USBD_HID3_DEV == 0))
#define USBD0_HID3                      1
#define USBD0_HID3_EP_INT_IN            USBD_HID3_EP_INT_IN
#define USBD0_HID3_EP_INT_OUT           USBD_HID3_EP_INT_OUT
#define USBD0_HID3_IF_NUM               1
#if   ((USBD_HID3_EP_INT_IN ==  1) || (USBD_HID3_EP_INT_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  2) || (USBD_HID3_EP_INT_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  3) || (USBD_HID3_EP_INT_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  4) || (USBD_HID3_EP_INT_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  5) || (USBD_HID3_EP_INT_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  6) || (USBD_HID3_EP_INT_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  7) || (USBD_HID3_EP_INT_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  8) || (USBD_HID3_EP_INT_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  9) || (USBD_HID3_EP_INT_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 10) || (USBD_HID3_EP_INT_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 11) || (USBD_HID3_EP_INT_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 12) || (USBD_HID3_EP_INT_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 13) || (USBD_HID3_EP_INT_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 14) || (USBD_HID3_EP_INT_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 15) || (USBD_HID3_EP_INT_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_hid_thread_id[3]
#endif 
#else
#define USBD0_HID3                      0
#define USBD0_HID3_EP_INT_IN            0
#define USBD0_HID3_EP_INT_OUT           0
#define USBD0_HID3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_0) && (USBD_MSC0_DEV == 0))
#define USBD0_MSC0                      1
#define USBD0_MSC0_EP_BULK_IN           USBD_MSC0_EP_BULK_IN
#define USBD0_MSC0_EP_BULK_OUT          USBD_MSC0_EP_BULK_OUT
#define USBD0_MSC0_IF_NUM               1
#if   ((USBD_MSC0_EP_BULK_IN ==  1) || (USBD_MSC0_EP_BULK_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  2) || (USBD_MSC0_EP_BULK_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif            
#define USBD0_EP2                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  3) || (USBD_MSC0_EP_BULK_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif           
#define USBD0_EP3                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  4) || (USBD_MSC0_EP_BULK_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  5) || (USBD_MSC0_EP_BULK_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  6) || (USBD_MSC0_EP_BULK_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  7) || (USBD_MSC0_EP_BULK_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  8) || (USBD_MSC0_EP_BULK_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  9) || (USBD_MSC0_EP_BULK_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 10) || (USBD_MSC0_EP_BULK_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 11) || (USBD_MSC0_EP_BULK_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 12) || (USBD_MSC0_EP_BULK_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 13) || (USBD_MSC0_EP_BULK_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 14) || (USBD_MSC0_EP_BULK_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 15) || (USBD_MSC0_EP_BULK_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_msc_thread_id[0]
#endif 
#else
#define USBD0_MSC0                      0
#define USBD0_MSC0_EP_BULK_IN           0
#define USBD0_MSC0_EP_BULK_OUT          0
#define USBD0_MSC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_1) && (USBD_MSC1_DEV == 0))
#define USBD0_MSC1                      1
#define USBD0_MSC1_EP_BULK_IN           USBD_MSC1_EP_BULK_IN
#define USBD0_MSC1_EP_BULK_OUT          USBD_MSC1_EP_BULK_OUT
#define USBD0_MSC1_IF_NUM               1
#if   ((USBD_MSC1_EP_BULK_IN ==  1) || (USBD_MSC1_EP_BULK_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  2) || (USBD_MSC1_EP_BULK_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  3) || (USBD_MSC1_EP_BULK_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  4) || (USBD_MSC1_EP_BULK_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  5) || (USBD_MSC1_EP_BULK_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  6) || (USBD_MSC1_EP_BULK_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  7) || (USBD_MSC1_EP_BULK_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  8) || (USBD_MSC1_EP_BULK_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  9) || (USBD_MSC1_EP_BULK_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 10) || (USBD_MSC1_EP_BULK_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 11) || (USBD_MSC1_EP_BULK_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 12) || (USBD_MSC1_EP_BULK_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 13) || (USBD_MSC1_EP_BULK_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 14) || (USBD_MSC1_EP_BULK_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 15) || (USBD_MSC1_EP_BULK_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_msc_thread_id[1]
#endif 
#else
#define USBD0_MSC1                      0
#define USBD0_MSC1_EP_BULK_IN           0
#define USBD0_MSC1_EP_BULK_OUT          0
#define USBD0_MSC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_2) && (USBD_MSC2_DEV == 0))
#define USBD0_MSC2                      1
#define USBD0_MSC2_EP_BULK_IN           USBD_MSC2_EP_BULK_IN
#define USBD0_MSC2_EP_BULK_OUT          USBD_MSC2_EP_BULK_OUT
#define USBD0_MSC2_IF_NUM               1
#if   ((USBD_MSC2_EP_BULK_IN ==  1) || (USBD_MSC2_EP_BULK_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  2) || (USBD_MSC2_EP_BULK_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  3) || (USBD_MSC2_EP_BULK_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  4) || (USBD_MSC2_EP_BULK_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  5) || (USBD_MSC2_EP_BULK_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  6) || (USBD_MSC2_EP_BULK_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  7) || (USBD_MSC2_EP_BULK_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  8) || (USBD_MSC2_EP_BULK_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  9) || (USBD_MSC2_EP_BULK_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 10) || (USBD_MSC2_EP_BULK_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 11) || (USBD_MSC2_EP_BULK_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 12) || (USBD_MSC2_EP_BULK_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 13) || (USBD_MSC2_EP_BULK_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 14) || (USBD_MSC2_EP_BULK_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 15) || (USBD_MSC2_EP_BULK_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_msc_thread_id[2]
#endif 
#else
#define USBD0_MSC2                      0
#define USBD0_MSC2_EP_BULK_IN           0
#define USBD0_MSC2_EP_BULK_OUT          0
#define USBD0_MSC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_3) && (USBD_MSC3_DEV == 0))
#define USBD0_MSC3                      1
#define USBD0_MSC3_EP_BULK_IN           USBD_MSC3_EP_BULK_IN
#define USBD0_MSC3_EP_BULK_OUT          USBD_MSC3_EP_BULK_OUT
#define USBD0_MSC3_IF_NUM               1
#if   ((USBD_MSC3_EP_BULK_IN ==  1) || (USBD_MSC3_EP_BULK_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  2) || (USBD_MSC3_EP_BULK_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  3) || (USBD_MSC3_EP_BULK_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  4) || (USBD_MSC3_EP_BULK_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  5) || (USBD_MSC3_EP_BULK_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  6) || (USBD_MSC3_EP_BULK_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  7) || (USBD_MSC3_EP_BULK_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  8) || (USBD_MSC3_EP_BULK_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  9) || (USBD_MSC3_EP_BULK_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 10) || (USBD_MSC3_EP_BULK_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 11) || (USBD_MSC3_EP_BULK_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 12) || (USBD_MSC3_EP_BULK_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 13) || (USBD_MSC3_EP_BULK_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 14) || (USBD_MSC3_EP_BULK_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 15) || (USBD_MSC3_EP_BULK_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_msc_thread_id[3]
#endif 
#else
#define USBD0_MSC3                      0
#define USBD0_MSC3_EP_BULK_IN           0
#define USBD0_MSC3_EP_BULK_OUT          0
#define USBD0_MSC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_0) && (USBD_ADC0_DEV == 0))
#define USBD0_ADC0                      1
#define USBD0_ADC0_EP_ISO_OUT           USBD_ADC0_EP_ISO_OUT
#define USBD0_ADC0_IF_NUM               2
#if   ((USBD_ADC0_EP_ISO_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_adc_thread_id[0]
#endif 
#else
#define USBD0_ADC0                      0
#define USBD0_ADC0_EP_ISO_OUT           0
#define USBD0_ADC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_1) && (USBD_ADC1_DEV == 0))
#define USBD0_ADC1                      1
#define USBD0_ADC1_EP_ISO_OUT           USBD_ADC1_EP_ISO_OUT
#define USBD0_ADC1_IF_NUM               2
#if   ((USBD_ADC1_EP_ISO_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_adc_thread_id[1]
#endif 
#else
#define USBD0_ADC1                      0
#define USBD0_ADC1_EP_ISO_OUT           0
#define USBD0_ADC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_2) && (USBD_ADC2_DEV == 0))
#define USBD0_ADC2                      1
#define USBD0_ADC2_EP_ISO_OUT           USBD_ADC2_EP_ISO_OUT
#define USBD0_ADC2_IF_NUM               2
#if   ((USBD_ADC2_EP_ISO_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_adc_thread_id[2]
#endif 
#else
#define USBD0_ADC2                      0
#define USBD0_ADC2_EP_ISO_OUT           0
#define USBD0_ADC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_3) && (USBD_ADC3_DEV == 0))
#define USBD0_ADC3                      1
#define USBD0_ADC3_EP_ISO_OUT           USBD_ADC3_EP_ISO_OUT
#define USBD0_ADC3_IF_NUM               2
#if   ((USBD_ADC3_EP_ISO_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_adc_thread_id[3]
#endif 
#else
#define USBD0_ADC3                      0
#define USBD0_ADC3_EP_ISO_OUT           0
#define USBD0_ADC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_0) && (USBD_CDC0_DEV == 0))
#define USBD0_CDC0                      1
#define USBD0_CDC0_EP_INT_IN            USBD_CDC0_EP_INT_IN
#define USBD0_CDC0_EP_BULK_IN           USBD_CDC0_EP_BULK_IN
#define USBD0_CDC0_EP_BULK_OUT          USBD_CDC0_EP_BULK_OUT
#define USBD0_CDC0_IF_NUM               2
#if   ((USBD_CDC0_EP_INT_IN ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_cdc_thread_id[0]
#endif 
#if   ((USBD_CDC0_EP_BULK_IN ==  1) || (USBD_CDC0_EP_BULK_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  2) || (USBD_CDC0_EP_BULK_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  3) || (USBD_CDC0_EP_BULK_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  4) || (USBD_CDC0_EP_BULK_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  5) || (USBD_CDC0_EP_BULK_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  6) || (USBD_CDC0_EP_BULK_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  7) || (USBD_CDC0_EP_BULK_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  8) || (USBD_CDC0_EP_BULK_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  9) || (USBD_CDC0_EP_BULK_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 10) || (USBD_CDC0_EP_BULK_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 11) || (USBD_CDC0_EP_BULK_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 12) || (USBD_CDC0_EP_BULK_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 13) || (USBD_CDC0_EP_BULK_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 14) || (USBD_CDC0_EP_BULK_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 15) || (USBD_CDC0_EP_BULK_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_cdc_thread_id[1]
#endif 
#else
#define USBD0_CDC0                      0
#define USBD0_CDC0_EP_INT_IN            0
#define USBD0_CDC0_EP_BULK_IN           0
#define USBD0_CDC0_EP_BULK_OUT          0
#define USBD0_CDC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_1) && (USBD_CDC1_DEV == 0))
#define USBD0_CDC1                      1
#define USBD0_CDC1_EP_INT_IN            USBD_CDC1_EP_INT_IN
#define USBD0_CDC1_EP_BULK_IN           USBD_CDC1_EP_BULK_IN
#define USBD0_CDC1_EP_BULK_OUT          USBD_CDC1_EP_BULK_OUT
#define USBD0_CDC1_IF_NUM               2
#if   ((USBD_CDC1_EP_INT_IN ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_cdc_thread_id[2]
#endif 
#if   ((USBD_CDC1_EP_BULK_IN ==  1) || (USBD_CDC1_EP_BULK_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  2) || (USBD_CDC1_EP_BULK_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  3) || (USBD_CDC1_EP_BULK_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  4) || (USBD_CDC1_EP_BULK_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  5) || (USBD_CDC1_EP_BULK_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  6) || (USBD_CDC1_EP_BULK_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  7) || (USBD_CDC1_EP_BULK_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  8) || (USBD_CDC1_EP_BULK_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  9) || (USBD_CDC1_EP_BULK_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 10) || (USBD_CDC1_EP_BULK_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 11) || (USBD_CDC1_EP_BULK_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 12) || (USBD_CDC1_EP_BULK_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 13) || (USBD_CDC1_EP_BULK_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 14) || (USBD_CDC1_EP_BULK_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 15) || (USBD_CDC1_EP_BULK_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_cdc_thread_id[3]
#endif 
#else
#define USBD0_CDC1                      0
#define USBD0_CDC1_EP_INT_IN            0
#define USBD0_CDC1_EP_BULK_IN           0
#define USBD0_CDC1_EP_BULK_OUT          0
#define USBD0_CDC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_2) && (USBD_CDC2_DEV == 0))
#define USBD0_CDC2                      1
#define USBD0_CDC2_EP_INT_IN            USBD_CDC2_EP_INT_IN
#define USBD0_CDC2_EP_BULK_IN           USBD_CDC2_EP_BULK_IN
#define USBD0_CDC2_EP_BULK_OUT          USBD_CDC2_EP_BULK_OUT
#define USBD0_CDC2_IF_NUM               2
#if   ((USBD_CDC2_EP_INT_IN ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_cdc_thread_id[4]
#endif 
#if   ((USBD_CDC2_EP_BULK_IN ==  1) || (USBD_CDC2_EP_BULK_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  2) || (USBD_CDC2_EP_BULK_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  3) || (USBD_CDC2_EP_BULK_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  4) || (USBD_CDC2_EP_BULK_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  5) || (USBD_CDC2_EP_BULK_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  6) || (USBD_CDC2_EP_BULK_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  7) || (USBD_CDC2_EP_BULK_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  8) || (USBD_CDC2_EP_BULK_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  9) || (USBD_CDC2_EP_BULK_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 10) || (USBD_CDC2_EP_BULK_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 11) || (USBD_CDC2_EP_BULK_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 12) || (USBD_CDC2_EP_BULK_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 13) || (USBD_CDC2_EP_BULK_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 14) || (USBD_CDC2_EP_BULK_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 15) || (USBD_CDC2_EP_BULK_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_cdc_thread_id[5]
#endif 
#else
#define USBD0_CDC2                      0
#define USBD0_CDC2_EP_INT_IN            0
#define USBD0_CDC2_EP_BULK_IN           0
#define USBD0_CDC2_EP_BULK_OUT          0
#define USBD0_CDC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_3) && (USBD_CDC3_DEV == 0))
#define USBD0_CDC3                      1
#define USBD0_CDC3_EP_INT_IN            USBD_CDC3_EP_INT_IN
#define USBD0_CDC3_EP_BULK_IN           USBD_CDC3_EP_BULK_IN
#define USBD0_CDC3_EP_BULK_OUT          USBD_CDC3_EP_BULK_OUT
#define USBD0_CDC3_IF_NUM               2
#if   ((USBD_CDC3_EP_INT_IN ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_cdc_thread_id[6]
#endif 
#if   ((USBD_CDC3_EP_BULK_IN ==  1) || (USBD_CDC3_EP_BULK_OUT ==  1))
#ifdef  USBD0_EP1
#error  Endpoint 1 on Device 0 is already used!
#endif
#define USBD0_EP1                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  2) || (USBD_CDC3_EP_BULK_OUT ==  2))
#ifdef  USBD0_EP2
#error  Endpoint 2 on Device 0 is already used!
#endif
#define USBD0_EP2                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  3) || (USBD_CDC3_EP_BULK_OUT ==  3))
#ifdef  USBD0_EP3
#error  Endpoint 3 on Device 0 is already used!
#endif
#define USBD0_EP3                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  4) || (USBD_CDC3_EP_BULK_OUT ==  4))
#ifdef  USBD0_EP4
#error  Endpoint 4 on Device 0 is already used!
#endif
#define USBD0_EP4                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  5) || (USBD_CDC3_EP_BULK_OUT ==  5))
#ifdef  USBD0_EP5
#error  Endpoint 5 on Device 0 is already used!
#endif
#define USBD0_EP5                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  6) || (USBD_CDC3_EP_BULK_OUT ==  6))
#ifdef  USBD0_EP6
#error  Endpoint 6 on Device 0 is already used!
#endif
#define USBD0_EP6                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  7) || (USBD_CDC3_EP_BULK_OUT ==  7))
#ifdef  USBD0_EP7
#error  Endpoint 7 on Device 0 is already used!
#endif
#define USBD0_EP7                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  8) || (USBD_CDC3_EP_BULK_OUT ==  8))
#ifdef  USBD0_EP8
#error  Endpoint 8 on Device 0 is already used!
#endif
#define USBD0_EP8                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  9) || (USBD_CDC3_EP_BULK_OUT ==  9))
#ifdef  USBD0_EP9
#error  Endpoint 9 on Device 0 is already used!
#endif
#define USBD0_EP9                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 10) || (USBD_CDC3_EP_BULK_OUT == 10))
#ifdef  USBD0_EP10
#error  Endpoint 10 on Device 0 is already used!
#endif
#define USBD0_EP10                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 11) || (USBD_CDC3_EP_BULK_OUT == 11))
#ifdef  USBD0_EP11
#error  Endpoint 11 on Device 0 is already used!
#endif
#define USBD0_EP11                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 12) || (USBD_CDC3_EP_BULK_OUT == 12))
#ifdef  USBD0_EP12
#error  Endpoint 12 on Device 0 is already used!
#endif
#define USBD0_EP12                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 13) || (USBD_CDC3_EP_BULK_OUT == 13))
#ifdef  USBD0_EP13
#error  Endpoint 13 on Device 0 is already used!
#endif
#define USBD0_EP13                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 14) || (USBD_CDC3_EP_BULK_OUT == 14))
#ifdef  USBD0_EP14
#error  Endpoint 14 on Device 0 is already used!
#endif
#define USBD0_EP14                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 15) || (USBD_CDC3_EP_BULK_OUT == 15))
#ifdef  USBD0_EP15
#error  Endpoint 15 on Device 0 is already used!
#endif
#define USBD0_EP15                     &usbd_cdc_thread_id[7]
#endif 
#else
#define USBD0_CDC3                      0
#define USBD0_CDC3_EP_INT_IN            0
#define USBD0_CDC3_EP_BULK_IN           0
#define USBD0_CDC3_EP_BULK_OUT          0
#define USBD0_CDC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CLS_0) && (USBD_CLS0_DEV == 0))
#define USBD0_CLS0                      1
#else
#define USBD0_CLS0                      0
#endif

#if    (defined(RTE_USB_Device_CLS_1) && (USBD_CLS1_DEV == 0))
#define USBD0_CLS1                      1
#else
#define USBD0_CLS1                      0
#endif

#if    (defined(RTE_USB_Device_CLS_2) && (USBD_CLS2_DEV == 0))
#define USBD0_CLS2                      1
#else
#define USBD0_CLS2                      0
#endif

#if    (defined(RTE_USB_Device_CLS_3) && (USBD_CLS3_DEV == 0))
#define USBD0_CLS3                      1
#else
#define USBD0_CLS3                      0
#endif

#define USBD0_HID_NUM                  (USBDn_c_NUM(0,HID))
#define USBD0_MSC_NUM                  (USBDn_c_NUM(0,MSC))
#define USBD0_ADC_NUM                  (USBDn_c_NUM(0,ADC))
#define USBD0_CDC_NUM                  (USBDn_c_NUM(0,CDC))
#define USBD0_CLS_NUM                  (USBDn_c_NUM(0,CLS))
#define USBD0_ANY                     ((USBD0_HID_NUM + USBD0_MSC_NUM + USBD0_ADC_NUM + USBD0_CDC_NUM + USBD0_CLS_NUM) != 0)

#define USBD0_HID_EP_CALC    (MAX_OF_8 (USBD0_HID0_EP_INT_IN, USBD0_HID0_EP_INT_OUT,  \
                                        USBD0_HID1_EP_INT_IN, USBD0_HID1_EP_INT_OUT,  \
                                        USBD0_HID2_EP_INT_IN, USBD0_HID2_EP_INT_OUT,  \
                                        USBD0_HID3_EP_INT_IN, USBD0_HID3_EP_INT_OUT))
#define USBD0_MSC_EP_CALC    (MAX_OF_8 (USBD0_MSC0_EP_BULK_IN, USBD0_MSC0_EP_BULK_OUT,  \
                                        USBD0_MSC1_EP_BULK_IN, USBD0_MSC1_EP_BULK_OUT,  \
                                        USBD0_MSC2_EP_BULK_IN, USBD0_MSC2_EP_BULK_OUT,  \
                                        USBD0_MSC3_EP_BULK_IN, USBD0_MSC3_EP_BULK_OUT))
#define USBD0_ADC_EP_CALC    (MAX_OF_4 (USBD0_ADC0_EP_ISO_OUT,  \
                                        USBD0_ADC1_EP_ISO_OUT,  \
                                        USBD0_ADC2_EP_ISO_OUT,  \
                                        USBD0_ADC3_EP_ISO_OUT))
#define USBD0_CDC_EP_CALC    (MAX_OF_12(USBD0_CDC0_EP_INT_IN, USBD0_CDC0_EP_BULK_IN, USBD0_CDC0_EP_BULK_OUT, \
                                        USBD0_CDC1_EP_INT_IN, USBD0_CDC1_EP_BULK_IN, USBD0_CDC1_EP_BULK_OUT, \
                                        USBD0_CDC2_EP_INT_IN, USBD0_CDC2_EP_BULK_IN, USBD0_CDC2_EP_BULK_OUT, \
                                        USBD0_CDC3_EP_INT_IN, USBD0_CDC3_EP_BULK_IN, USBD0_CDC3_EP_BULK_OUT))

#define USBD0_IF_NUM                   (USBD0_HID_NUM+USBD0_MSC_NUM+(USBD0_ADC_NUM*2)+(USBD0_CDC_NUM*2)+USBD0_CLS_NUM)
#define USBD0_EP_NUM         (MAX_OF_4 (USBD0_HID_EP_CALC,USBD0_MSC_EP_CALC,USBD0_ADC_EP_CALC,USBD0_CDC_EP_CALC))

#if    (USBD0_ADC0_IF_NUM != 0)
#define USBD_ADC0_CIF                (0)
#define USBD_ADC0_SIF                (1+USBD_ADC0_CIF)
#define USBD_ADC0_CIF_STR            (0+USBD0_STR_DESC_SER_EN+3)
#define USBD_ADC0_SIF1_STR           (1+USBD_ADC0_CIF_STR)
#define USBD_ADC0_SIF2_STR           (2+USBD_ADC0_CIF_STR)
#endif
#if    (USBD0_ADC1_IF_NUM != 0)
#define USBD_ADC1_CIF                  (USBD0_ADC0_IF_NUM                                                                       )
#define USBD_ADC1_SIF                (1+USBD_ADC1_CIF)
#define USBD_ADC1_CIF_STR             ((USBD0_ADC0_IF_NUM/2                                                                     )*3+\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC1_SIF1_STR           (1+USBD_ADC1_CIF_STR)
#define USBD_ADC1_SIF2_STR           (2+USBD_ADC1_CIF_STR)
#endif
#if    (USBD0_ADC2_IF_NUM != 0)
#define USBD_ADC2_CIF                  (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM                                                 )
#define USBD_ADC2_SIF                (1+USBD_ADC2_CIF)
#define USBD_ADC2_CIF_STR             ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2                                               )*3+\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC2_SIF1_STR           (1+USBD_ADC2_CIF_STR)
#define USBD_ADC2_SIF2_STR           (2+USBD_ADC2_CIF_STR)
#endif
#if    (USBD0_ADC3_IF_NUM != 0)
#define USBD_ADC3_CIF                  (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM                           )
#define USBD_ADC3_SIF                (1+USBD_ADC3_CIF)
#define USBD_ADC3_CIF_STR             ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2                         )*3+\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC3_SIF1_STR           (1+USBD_ADC3_CIF_STR)
#define USBD_ADC3_SIF2_STR           (2+USBD_ADC3_CIF_STR)
#endif

#if    (USBD0_CDC0_IF_NUM != 0)
#define USBD_CDC0_CIF                  (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM     )
#define USBD_CDC0_DIF                (1+USBD_CDC0_CIF)
#define USBD_CDC0_CIF_STR             ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC0_DIF_STR            (1+USBD_CDC0_CIF_STR)
#endif
#if    (USBD0_CDC1_IF_NUM != 0)
#define USBD_CDC1_CIF                  (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM                                                                       )
#define USBD_CDC1_DIF                (1+USBD_CDC1_CIF)
#define USBD_CDC1_CIF_STR             ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM                                                                        +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC1_DIF_STR            (1+USBD_CDC1_CIF_STR)
#endif
#if    (USBD0_CDC2_IF_NUM != 0)
#define USBD_CDC2_CIF                  (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM                                                 )
#define USBD_CDC2_DIF                (1+USBD_CDC2_CIF)
#define USBD_CDC2_CIF_STR             ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM                                                  +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC2_DIF_STR            (1+USBD_CDC2_CIF_STR)
#endif
#if    (USBD0_CDC3_IF_NUM != 0)
#define USBD_CDC3_CIF                  (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM                           )
#define USBD_CDC3_DIF                (1+USBD_CDC3_CIF)
#define USBD_CDC3_CIF_STR             ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM                            +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC3_DIF_STR            (1+USBD_CDC3_CIF_STR)
#endif

#if    (USBD0_HID0_IF_NUM != 0)
#define USBD_HID0_IF                   (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM     )
#define USBD_HID0_STR                 ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD0_HID1_IF_NUM != 0)
#define USBD_HID1_IF                   (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM                                                                       )
#define USBD_HID1_STR                 ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM                                                                        +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD0_HID2_IF_NUM != 0)
#define USBD_HID2_IF                   (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM                                                 )
#define USBD_HID2_STR                 ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM                                                  +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD0_HID3_IF_NUM != 0)
#define USBD_HID3_IF                   (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM                           )
#define USBD_HID3_STR                 ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM                            +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#endif

#if    (USBD0_MSC0_IF_NUM != 0)
#define USBD_MSC0_IF                   (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM    +USBD0_HID3_IF_NUM     )
#define USBD_MSC0_STR                 ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM    +USBD0_HID2_IF_NUM      +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD0_MSC1_IF_NUM != 0)
#define USBD_MSC1_IF                   (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM    +USBD0_HID3_IF_NUM      +\
                                        USBD0_MSC0_IF_NUM                                                                       )
#define USBD_MSC1_STR                 ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM    +USBD0_HID2_IF_NUM      +\
                                        USBD0_MSC0_IF_NUM                                                                        +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD0_MSC2_IF_NUM != 0)
#define USBD_MSC2_IF                   (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM    +USBD0_HID3_IF_NUM      +\
                                        USBD0_MSC0_IF_NUM    +USBD0_MSC1_IF_NUM                                                 )
#define USBD_MSC2_STR                 ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM    +USBD0_HID2_IF_NUM      +\
                                        USBD0_MSC0_IF_NUM    +USBD0_MSC1_IF_NUM                                                  +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD0_MSC3_IF_NUM != 0)
#define USBD_MSC3_IF                   (USBD0_ADC0_IF_NUM    +USBD0_ADC1_IF_NUM    +USBD0_ADC2_IF_NUM    +USBD0_ADC3_IF_NUM      +\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM    +USBD0_HID3_IF_NUM      +\
                                        USBD0_MSC0_IF_NUM    +USBD0_MSC1_IF_NUM    +USBD0_MSC2_IF_NUM                           )
#define USBD_MSC3_STR                 ((USBD0_ADC0_IF_NUM/2  +USBD0_ADC1_IF_NUM/2  +USBD0_ADC2_IF_NUM/2  +USBD0_ADC3_IF_NUM/2   )*3+\
                                        USBD0_CDC0_IF_NUM    +USBD0_CDC1_IF_NUM    +USBD0_CDC2_IF_NUM    +USBD0_CDC3_IF_NUM      +\
                                        USBD0_HID0_IF_NUM    +USBD0_HID1_IF_NUM    +USBD0_HID2_IF_NUM    +USBD0_HID2_IF_NUM      +\
                                        USBD0_MSC0_IF_NUM    +USBD0_MSC1_IF_NUM    +USBD0_MSC2_IF_NUM                            +\
                                        USBD0_STR_DESC_SER_EN+3                                                                 )
#endif

#define USBD0_USE_IAD                 ((USBD0_CDC_NUM>0)*((USBD0_HID_NUM>0)|(USBD0_MSC_NUM>0)|(USBD0_ADC_NUM>0)|(USBD0_CDC_NUM>0)))

#define USBD0_ADC_DESC_LEN             (USBD0_ADC_NUM *                                                                 \
                                       (USBD0_USE_IAD * USB_INTERFACE_ASSOC_DESC_SIZE +                                 \
                                        3 * USB_INTERFACE_DESC_SIZE + AUDIO_CONTROL_INTERFACE_DESC_SZ(1) +              \
                                        AUDIO_INPUT_TERMINAL_DESC_SIZE + AUDIO_FEATURE_UNIT_DESC_SZ(1,1) +              \
                                        AUDIO_OUTPUT_TERMINAL_DESC_SIZE + AUDIO_STREAMING_INTERFACE_DESC_SIZE +         \
                                        AUDIO_FORMAT_TYPE_I_DESC_SZ(1) + AUDIO_STANDARD_ENDPOINT_DESC_SIZE +            \
                                        AUDIO_STREAMING_ENDPOINT_DESC_SIZE))
#define USBD0_CDC_DESC_LEN             (USBD0_CDC_NUM *                                                                 \
                                       (USBD0_USE_IAD * USB_INTERFACE_ASSOC_DESC_SIZE +                                 \
                                        USB_INTERFACE_DESC_SIZE + 0x0013 + USB_ENDPOINT_DESC_SIZE +                     \
                                        USB_INTERFACE_DESC_SIZE + 2* USB_ENDPOINT_DESC_SIZE))
#define USBD0_MSC_DESC_LEN             (USBD0_MSC_NUM *                                                                 \
                                       (USB_INTERFACE_DESC_SIZE + 2 * USB_ENDPOINT_DESC_SIZE))
#if (USBD0_HID0 == 1)
#define USBD0_HID0_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD0_HID0_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD0_HID0_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD0_ADC_NUM * USBD0_ADC_DESC_LEN +                                            \
                                        USBD0_CDC_NUM * USBD0_CDC_DESC_LEN)
#define USBD_HID0_DESC_LEN             (USBD0_HID0_DESC_LEN)
#define USBD_HID0_DESC_OFS             (USBD0_HID0_DESC_OFS)
#else
#define USBD0_HID0_DESC_LEN            (0)
#define USBD0_HID0_DESC_OFS            (0)
#endif
#if (USBD0_HID1 == 1)
#define USBD0_HID1_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD0_HID1_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD0_HID1_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD0_ADC_NUM * USBD0_ADC_DESC_LEN +                                            \
                                        USBD0_CDC_NUM * USBD0_CDC_DESC_LEN +                                            \
                                        USBD0_HID0_DESC_OFS)
#define USBD_HID1_DESC_LEN             (USBD0_HID1_DESC_LEN)
#define USBD_HID1_DESC_OFS             (USBD0_HID1_DESC_OFS)
#else
#define USBD0_HID1_DESC_LEN            (0)
#define USBD0_HID1_DESC_OFS            (0)
#endif
#if (USBD0_HID2 == 1)
#define USBD0_HID2_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD0_HID2_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD0_HID2_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD0_ADC_NUM * USBD0_ADC_DESC_LEN +                                            \
                                        USBD0_CDC_NUM * USBD0_CDC_DESC_LEN +                                            \
                                        USBD0_HID0_DESC_OFS +                                                           \
                                        USBD0_HID1_DESC_OFS)
#define USBD_HID2_DESC_LEN             (USBD0_HID2_DESC_LEN)
#define USBD_HID2_DESC_OFS             (USBD0_HID2_DESC_OFS)
#else
#define USBD0_HID2_DESC_LEN            (0)
#define USBD0_HID2_DESC_OFS            (0)
#endif
#if (USBD0_HID3 == 1)
#define USBD0_HID3_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD0_HID3_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD0_HID3_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD0_ADC_NUM * USBD0_ADC_DESC_LEN +                                            \
                                        USBD0_CDC_NUM * USBD0_CDC_DESC_LEN +                                            \
                                        USBD0_HID0_DESC_OFS +                                                           \
                                        USBD0_HID1_DESC_OFS +                                                           \
                                        USBD0_HID2_DESC_OFS)
#define USBD_HID3_DESC_LEN             (USBD0_HID3_DESC_LEN)
#define USBD_HID3_DESC_OFS             (USBD0_HID3_DESC_OFS)
#else
#define USBD0_HID3_DESC_LEN            (0)
#define USBD0_HID3_DESC_OFS            (0)
#endif
#define USBD0_HID_DESC_LEN             (USBD0_HID0_DESC_LEN +                                                           \
                                        USBD0_HID1_DESC_LEN +                                                           \
                                        USBD0_HID2_DESC_LEN +                                                           \
                                        USBD0_HID3_DESC_LEN )                                                           \

#define USBD0_WTOTALLENGTH             (USB_CONFIGURATION_DESC_SIZE +                                                   \
                                        USBD0_ADC_DESC_LEN +                                                            \
                                        USBD0_CDC_DESC_LEN +                                                            \
                                        USBD0_HID_DESC_LEN +                                                            \
                                        USBD0_MSC_DESC_LEN )

#if ((USBD0_HS) || (USBD0_USE_IAD))
#define USBD0_BDC_USB                   0x0200
#else
#define USBD0_BDC_USB                   0x0110
#endif

#endif  /* USBD0_MACROS */

#if     (defined(USBD0_PORT) && !defined(USBD0_PORT_DEF))
#define  USBD0_PORT_DEF                 1
#ifndef  __NO_USB_LIB_C

extern  ARM_DRIVER_USBD  USBDn_DRIVER(USBD0_PORT);

uint8_t         usbd0_alt_setting      [USBD0_IF_NUM];
uint8_t         usbd0_ep0_buf          [USBD0_MAX_PACKET0] __attribute__((aligned(4)));
usbd_data_t     usbd0_data;
usbd_dev_t      usbd0_dev           = { 
                                       &USBDn_DRIVER(USBD0_PORT), 
                                        usbd0_alt_setting, 
                                        usbd0_ep0_buf, 
                                       &usbd0_data, 
                                        USBD0_CFG_DESC_BMATTRIBUTES, 
                                        USBD0_HS, 
                                        USBD0_IF_NUM, 
                                        USBD0_EP_NUM, 
                                        USBD0_MAX_PACKET0, 
                                      };

#endif  /* __NO_USB_LIB_C   */
#endif


/*------------------------------------------------------------------------------
 *      USB Device 1 Macros, Calculations and Constants for Library
 *----------------------------------------------------------------------------*/

#ifndef USBD1_MACROS
#define USBD1_MACROS                    1

#if    (defined(RTE_USB_Device_HID_0) && (USBD_HID0_DEV == 1))
#define USBD1_HID0                      1
#define USBD1_HID0_EP_INT_IN            USBD_HID0_EP_INT_IN
#define USBD1_HID0_EP_INT_OUT           USBD_HID0_EP_INT_OUT
#define USBD1_HID0_IF_NUM               1
#if   ((USBD_HID0_EP_INT_IN ==  1) || (USBD_HID0_EP_INT_OUT ==  1))
#ifdef  USBD1_EP1 
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  2) || (USBD_HID0_EP_INT_OUT ==  2))
#ifdef  USBD1_EP2 
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  3) || (USBD_HID0_EP_INT_OUT ==  3))
#ifdef  USBD1_EP3 
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  4) || (USBD_HID0_EP_INT_OUT ==  4))
#ifdef  USBD1_EP4 
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  5) || (USBD_HID0_EP_INT_OUT ==  5))
#ifdef  USBD1_EP5 
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  6) || (USBD_HID0_EP_INT_OUT ==  6))
#ifdef  USBD1_EP6 
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  7) || (USBD_HID0_EP_INT_OUT ==  7))
#ifdef  USBD1_EP7 
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  8) || (USBD_HID0_EP_INT_OUT ==  8))
#ifdef  USBD1_EP8 
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  9) || (USBD_HID0_EP_INT_OUT ==  9))
#ifdef  USBD1_EP9 
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 10) || (USBD_HID0_EP_INT_OUT == 10))
#ifdef  USBD1_EP10 
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 11) || (USBD_HID0_EP_INT_OUT == 11))
#ifdef  USBD1_EP11 
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 12) || (USBD_HID0_EP_INT_OUT == 12))
#ifdef  USBD1_EP12 
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 13) || (USBD_HID0_EP_INT_OUT == 13))
#ifdef  USBD1_EP13 
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 14) || (USBD_HID0_EP_INT_OUT == 14))
#ifdef  USBD1_EP14 
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 15) || (USBD_HID0_EP_INT_OUT == 15))
#ifdef  USBD1_EP15 
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_hid_thread_id[0]
#endif 
#else
#define USBD1_HID0                      0
#define USBD1_HID0_EP_INT_IN            0
#define USBD1_HID0_EP_INT_OUT           0
#define USBD1_HID0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_1) && (USBD_HID1_DEV == 1))
#define USBD1_HID1                      1
#define USBD1_HID1_EP_INT_IN            USBD_HID1_EP_INT_IN
#define USBD1_HID1_EP_INT_OUT           USBD_HID1_EP_INT_OUT
#define USBD1_HID1_IF_NUM               1
#if   ((USBD_HID1_EP_INT_IN ==  1) || (USBD_HID1_EP_INT_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  2) || (USBD_HID1_EP_INT_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  3) || (USBD_HID1_EP_INT_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  4) || (USBD_HID1_EP_INT_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  5) || (USBD_HID1_EP_INT_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  6) || (USBD_HID1_EP_INT_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  7) || (USBD_HID1_EP_INT_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  8) || (USBD_HID1_EP_INT_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  9) || (USBD_HID1_EP_INT_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 10) || (USBD_HID1_EP_INT_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 11) || (USBD_HID1_EP_INT_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 12) || (USBD_HID1_EP_INT_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 13) || (USBD_HID1_EP_INT_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 14) || (USBD_HID1_EP_INT_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 15) || (USBD_HID1_EP_INT_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_hid_thread_id[1]
#endif 
#else
#define USBD1_HID1                      0
#define USBD1_HID1_EP_INT_IN            0
#define USBD1_HID1_EP_INT_OUT           0
#define USBD1_HID1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_2) && (USBD_HID2_DEV == 1))
#define USBD1_HID2                      1
#define USBD1_HID2_EP_INT_IN            USBD_HID2_EP_INT_IN
#define USBD1_HID2_EP_INT_OUT           USBD_HID2_EP_INT_OUT
#define USBD1_HID2_IF_NUM               1
#if   ((USBD_HID2_EP_INT_IN ==  1) || (USBD_HID2_EP_INT_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  2) || (USBD_HID2_EP_INT_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  3) || (USBD_HID2_EP_INT_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  4) || (USBD_HID2_EP_INT_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  5) || (USBD_HID2_EP_INT_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  6) || (USBD_HID2_EP_INT_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  7) || (USBD_HID2_EP_INT_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  8) || (USBD_HID2_EP_INT_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  9) || (USBD_HID2_EP_INT_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 10) || (USBD_HID2_EP_INT_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 11) || (USBD_HID2_EP_INT_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 12) || (USBD_HID2_EP_INT_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 13) || (USBD_HID2_EP_INT_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 14) || (USBD_HID2_EP_INT_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 15) || (USBD_HID2_EP_INT_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_hid_thread_id[2]
#endif 
#else
#define USBD1_HID2                      0
#define USBD1_HID2_EP_INT_IN            0
#define USBD1_HID2_EP_INT_OUT           0
#define USBD1_HID2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_3) && (USBD_HID3_DEV == 1))
#define USBD1_HID3                      1
#define USBD1_HID3_EP_INT_IN            USBD_HID3_EP_INT_IN
#define USBD1_HID3_EP_INT_OUT           USBD_HID3_EP_INT_OUT
#define USBD1_HID3_IF_NUM               1
#if   ((USBD_HID3_EP_INT_IN ==  1) || (USBD_HID3_EP_INT_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  2) || (USBD_HID3_EP_INT_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  3) || (USBD_HID3_EP_INT_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  4) || (USBD_HID3_EP_INT_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  5) || (USBD_HID3_EP_INT_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  6) || (USBD_HID3_EP_INT_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  7) || (USBD_HID3_EP_INT_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  8) || (USBD_HID3_EP_INT_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  9) || (USBD_HID3_EP_INT_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 10) || (USBD_HID3_EP_INT_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 11) || (USBD_HID3_EP_INT_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 12) || (USBD_HID3_EP_INT_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 13) || (USBD_HID3_EP_INT_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 14) || (USBD_HID3_EP_INT_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 15) || (USBD_HID3_EP_INT_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_hid_thread_id[3]
#endif 
#else
#define USBD1_HID3                      0
#define USBD1_HID3_EP_INT_IN            0
#define USBD1_HID3_EP_INT_OUT           0
#define USBD1_HID3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_0) && (USBD_MSC0_DEV == 1))
#define USBD1_MSC0                      1
#define USBD1_MSC0_EP_BULK_IN           USBD_MSC0_EP_BULK_IN
#define USBD1_MSC0_EP_BULK_OUT          USBD_MSC0_EP_BULK_OUT
#define USBD1_MSC0_IF_NUM               1
#if   ((USBD_MSC0_EP_BULK_IN ==  1) || (USBD_MSC0_EP_BULK_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  2) || (USBD_MSC0_EP_BULK_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif            
#define USBD1_EP2                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  3) || (USBD_MSC0_EP_BULK_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif           
#define USBD1_EP3                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  4) || (USBD_MSC0_EP_BULK_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  5) || (USBD_MSC0_EP_BULK_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  6) || (USBD_MSC0_EP_BULK_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  7) || (USBD_MSC0_EP_BULK_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  8) || (USBD_MSC0_EP_BULK_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  9) || (USBD_MSC0_EP_BULK_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 10) || (USBD_MSC0_EP_BULK_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 11) || (USBD_MSC0_EP_BULK_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 12) || (USBD_MSC0_EP_BULK_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 13) || (USBD_MSC0_EP_BULK_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 14) || (USBD_MSC0_EP_BULK_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 15) || (USBD_MSC0_EP_BULK_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_msc_thread_id[0]
#endif 
#else
#define USBD1_MSC0                      0
#define USBD1_MSC0_EP_BULK_IN           0
#define USBD1_MSC0_EP_BULK_OUT          0
#define USBD1_MSC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_1) && (USBD_MSC1_DEV == 1))
#define USBD1_MSC1                      1
#define USBD1_MSC1_EP_BULK_IN           USBD_MSC1_EP_BULK_IN
#define USBD1_MSC1_EP_BULK_OUT          USBD_MSC1_EP_BULK_OUT
#define USBD1_MSC1_IF_NUM               1
#if   ((USBD_MSC1_EP_BULK_IN ==  1) || (USBD_MSC1_EP_BULK_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  2) || (USBD_MSC1_EP_BULK_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  3) || (USBD_MSC1_EP_BULK_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  4) || (USBD_MSC1_EP_BULK_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  5) || (USBD_MSC1_EP_BULK_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  6) || (USBD_MSC1_EP_BULK_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  7) || (USBD_MSC1_EP_BULK_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  8) || (USBD_MSC1_EP_BULK_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  9) || (USBD_MSC1_EP_BULK_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 10) || (USBD_MSC1_EP_BULK_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 11) || (USBD_MSC1_EP_BULK_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 12) || (USBD_MSC1_EP_BULK_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 13) || (USBD_MSC1_EP_BULK_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 14) || (USBD_MSC1_EP_BULK_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 15) || (USBD_MSC1_EP_BULK_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_msc_thread_id[1]
#endif 
#else
#define USBD1_MSC1                      0
#define USBD1_MSC1_EP_BULK_IN           0
#define USBD1_MSC1_EP_BULK_OUT          0
#define USBD1_MSC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_2) && (USBD_MSC2_DEV == 1))
#define USBD1_MSC2                      1
#define USBD1_MSC2_EP_BULK_IN           USBD_MSC2_EP_BULK_IN
#define USBD1_MSC2_EP_BULK_OUT          USBD_MSC2_EP_BULK_OUT
#define USBD1_MSC2_IF_NUM               1
#if   ((USBD_MSC2_EP_BULK_IN ==  1) || (USBD_MSC2_EP_BULK_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  2) || (USBD_MSC2_EP_BULK_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  3) || (USBD_MSC2_EP_BULK_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  4) || (USBD_MSC2_EP_BULK_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  5) || (USBD_MSC2_EP_BULK_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  6) || (USBD_MSC2_EP_BULK_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  7) || (USBD_MSC2_EP_BULK_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  8) || (USBD_MSC2_EP_BULK_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  9) || (USBD_MSC2_EP_BULK_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 10) || (USBD_MSC2_EP_BULK_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 11) || (USBD_MSC2_EP_BULK_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 12) || (USBD_MSC2_EP_BULK_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 13) || (USBD_MSC2_EP_BULK_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 14) || (USBD_MSC2_EP_BULK_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 15) || (USBD_MSC2_EP_BULK_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_msc_thread_id[2]
#endif 
#else
#define USBD1_MSC2                      0
#define USBD1_MSC2_EP_BULK_IN           0
#define USBD1_MSC2_EP_BULK_OUT          0
#define USBD1_MSC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_3) && (USBD_MSC3_DEV == 1))
#define USBD1_MSC3                      1
#define USBD1_MSC3_EP_BULK_IN           USBD_MSC3_EP_BULK_IN
#define USBD1_MSC3_EP_BULK_OUT          USBD_MSC3_EP_BULK_OUT
#define USBD1_MSC3_IF_NUM               1
#if   ((USBD_MSC3_EP_BULK_IN ==  1) || (USBD_MSC3_EP_BULK_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  2) || (USBD_MSC3_EP_BULK_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  3) || (USBD_MSC3_EP_BULK_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  4) || (USBD_MSC3_EP_BULK_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  5) || (USBD_MSC3_EP_BULK_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  6) || (USBD_MSC3_EP_BULK_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  7) || (USBD_MSC3_EP_BULK_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  8) || (USBD_MSC3_EP_BULK_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  9) || (USBD_MSC3_EP_BULK_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 10) || (USBD_MSC3_EP_BULK_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 11) || (USBD_MSC3_EP_BULK_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 12) || (USBD_MSC3_EP_BULK_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 13) || (USBD_MSC3_EP_BULK_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 14) || (USBD_MSC3_EP_BULK_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 15) || (USBD_MSC3_EP_BULK_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_msc_thread_id[3]
#endif 
#else
#define USBD1_MSC3                      0
#define USBD1_MSC3_EP_BULK_IN           0
#define USBD1_MSC3_EP_BULK_OUT          0
#define USBD1_MSC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_0) && (USBD_ADC0_DEV == 1))
#define USBD1_ADC0                      1
#define USBD1_ADC0_EP_ISO_OUT           USBD_ADC0_EP_ISO_OUT
#define USBD1_ADC0_IF_NUM               2
#if   ((USBD_ADC0_EP_ISO_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_adc_thread_id[0]
#endif 
#else
#define USBD1_ADC0                      0
#define USBD1_ADC0_EP_ISO_OUT           0
#define USBD1_ADC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_1) && (USBD_ADC1_DEV == 1))
#define USBD1_ADC1                      1
#define USBD1_ADC1_EP_ISO_OUT           USBD_ADC1_EP_ISO_OUT
#define USBD1_ADC1_IF_NUM               2
#if   ((USBD_ADC1_EP_ISO_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_adc_thread_id[1]
#endif 
#else
#define USBD1_ADC1                      0
#define USBD1_ADC1_EP_ISO_OUT           0
#define USBD1_ADC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_2) && (USBD_ADC2_DEV == 1))
#define USBD1_ADC2                      1
#define USBD1_ADC2_EP_ISO_OUT           USBD_ADC2_EP_ISO_OUT
#define USBD1_ADC2_IF_NUM               2
#if   ((USBD_ADC2_EP_ISO_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_adc_thread_id[2]
#endif 
#else
#define USBD1_ADC2                      0
#define USBD1_ADC2_EP_ISO_OUT           0
#define USBD1_ADC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_3) && (USBD_ADC3_DEV == 1))
#define USBD1_ADC3                      1
#define USBD1_ADC3_EP_ISO_OUT           USBD_ADC3_EP_ISO_OUT
#define USBD1_ADC3_IF_NUM               2
#if   ((USBD_ADC3_EP_ISO_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_adc_thread_id[3]
#endif 
#else
#define USBD1_ADC3                      0
#define USBD1_ADC3_EP_ISO_OUT           0
#define USBD1_ADC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_0) && (USBD_CDC0_DEV == 1))
#define USBD1_CDC0                      1
#define USBD1_CDC0_EP_INT_IN            USBD_CDC0_EP_INT_IN
#define USBD1_CDC0_EP_BULK_IN           USBD_CDC0_EP_BULK_IN
#define USBD1_CDC0_EP_BULK_OUT          USBD_CDC0_EP_BULK_OUT
#define USBD1_CDC0_IF_NUM               2
#if   ((USBD_CDC0_EP_INT_IN ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_cdc_thread_id[0]
#endif 
#if   ((USBD_CDC0_EP_BULK_IN ==  1) || (USBD_CDC0_EP_BULK_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  2) || (USBD_CDC0_EP_BULK_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  3) || (USBD_CDC0_EP_BULK_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  4) || (USBD_CDC0_EP_BULK_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  5) || (USBD_CDC0_EP_BULK_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  6) || (USBD_CDC0_EP_BULK_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  7) || (USBD_CDC0_EP_BULK_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  8) || (USBD_CDC0_EP_BULK_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  9) || (USBD_CDC0_EP_BULK_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 10) || (USBD_CDC0_EP_BULK_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 11) || (USBD_CDC0_EP_BULK_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 12) || (USBD_CDC0_EP_BULK_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 13) || (USBD_CDC0_EP_BULK_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 14) || (USBD_CDC0_EP_BULK_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 15) || (USBD_CDC0_EP_BULK_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_cdc_thread_id[1]
#endif 
#else
#define USBD1_CDC0                      0
#define USBD1_CDC0_EP_INT_IN            0
#define USBD1_CDC0_EP_BULK_IN           0
#define USBD1_CDC0_EP_BULK_OUT          0
#define USBD1_CDC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_1) && (USBD_CDC1_DEV == 1))
#define USBD1_CDC1                      1
#define USBD1_CDC1_EP_INT_IN            USBD_CDC1_EP_INT_IN
#define USBD1_CDC1_EP_BULK_IN           USBD_CDC1_EP_BULK_IN
#define USBD1_CDC1_EP_BULK_OUT          USBD_CDC1_EP_BULK_OUT
#define USBD1_CDC1_IF_NUM               2
#if   ((USBD_CDC1_EP_INT_IN ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_cdc_thread_id[2]
#endif 
#if   ((USBD_CDC1_EP_BULK_IN ==  1) || (USBD_CDC1_EP_BULK_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  2) || (USBD_CDC1_EP_BULK_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  3) || (USBD_CDC1_EP_BULK_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  4) || (USBD_CDC1_EP_BULK_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  5) || (USBD_CDC1_EP_BULK_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  6) || (USBD_CDC1_EP_BULK_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  7) || (USBD_CDC1_EP_BULK_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  8) || (USBD_CDC1_EP_BULK_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  9) || (USBD_CDC1_EP_BULK_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 10) || (USBD_CDC1_EP_BULK_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 11) || (USBD_CDC1_EP_BULK_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 12) || (USBD_CDC1_EP_BULK_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 13) || (USBD_CDC1_EP_BULK_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 14) || (USBD_CDC1_EP_BULK_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 15) || (USBD_CDC1_EP_BULK_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_cdc_thread_id[3]
#endif 
#else
#define USBD1_CDC1                      0
#define USBD1_CDC1_EP_INT_IN            0
#define USBD1_CDC1_EP_BULK_IN           0
#define USBD1_CDC1_EP_BULK_OUT          0
#define USBD1_CDC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_2) && (USBD_CDC2_DEV == 1))
#define USBD1_CDC2                      1
#define USBD1_CDC2_EP_INT_IN            USBD_CDC2_EP_INT_IN
#define USBD1_CDC2_EP_BULK_IN           USBD_CDC2_EP_BULK_IN
#define USBD1_CDC2_EP_BULK_OUT          USBD_CDC2_EP_BULK_OUT
#define USBD1_CDC2_IF_NUM               2
#if   ((USBD_CDC2_EP_INT_IN ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_cdc_thread_id[4]
#endif 
#if   ((USBD_CDC2_EP_BULK_IN ==  1) || (USBD_CDC2_EP_BULK_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  2) || (USBD_CDC2_EP_BULK_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  3) || (USBD_CDC2_EP_BULK_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  4) || (USBD_CDC2_EP_BULK_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  5) || (USBD_CDC2_EP_BULK_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  6) || (USBD_CDC2_EP_BULK_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  7) || (USBD_CDC2_EP_BULK_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  8) || (USBD_CDC2_EP_BULK_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  9) || (USBD_CDC2_EP_BULK_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 10) || (USBD_CDC2_EP_BULK_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 11) || (USBD_CDC2_EP_BULK_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 12) || (USBD_CDC2_EP_BULK_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 13) || (USBD_CDC2_EP_BULK_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 14) || (USBD_CDC2_EP_BULK_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 15) || (USBD_CDC2_EP_BULK_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_cdc_thread_id[5]
#endif 
#else
#define USBD1_CDC2                      0
#define USBD1_CDC2_EP_INT_IN            0
#define USBD1_CDC2_EP_BULK_IN           0
#define USBD1_CDC2_EP_BULK_OUT          0
#define USBD1_CDC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_3) && (USBD_CDC3_DEV == 1))
#define USBD1_CDC3                      1
#define USBD1_CDC3_EP_INT_IN            USBD_CDC3_EP_INT_IN
#define USBD1_CDC3_EP_BULK_IN           USBD_CDC3_EP_BULK_IN
#define USBD1_CDC3_EP_BULK_OUT          USBD_CDC3_EP_BULK_OUT
#define USBD1_CDC3_IF_NUM               2
#if   ((USBD_CDC3_EP_INT_IN ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_cdc_thread_id[6]
#endif 
#if   ((USBD_CDC3_EP_BULK_IN ==  1) || (USBD_CDC3_EP_BULK_OUT ==  1))
#ifdef  USBD1_EP1
#error  Endpoint 1 on Device 1 is already used!
#endif
#define USBD1_EP1                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  2) || (USBD_CDC3_EP_BULK_OUT ==  2))
#ifdef  USBD1_EP2
#error  Endpoint 2 on Device 1 is already used!
#endif
#define USBD1_EP2                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  3) || (USBD_CDC3_EP_BULK_OUT ==  3))
#ifdef  USBD1_EP3
#error  Endpoint 3 on Device 1 is already used!
#endif
#define USBD1_EP3                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  4) || (USBD_CDC3_EP_BULK_OUT ==  4))
#ifdef  USBD1_EP4
#error  Endpoint 4 on Device 1 is already used!
#endif
#define USBD1_EP4                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  5) || (USBD_CDC3_EP_BULK_OUT ==  5))
#ifdef  USBD1_EP5
#error  Endpoint 5 on Device 1 is already used!
#endif
#define USBD1_EP5                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  6) || (USBD_CDC3_EP_BULK_OUT ==  6))
#ifdef  USBD1_EP6
#error  Endpoint 6 on Device 1 is already used!
#endif
#define USBD1_EP6                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  7) || (USBD_CDC3_EP_BULK_OUT ==  7))
#ifdef  USBD1_EP7
#error  Endpoint 7 on Device 1 is already used!
#endif
#define USBD1_EP7                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  8) || (USBD_CDC3_EP_BULK_OUT ==  8))
#ifdef  USBD1_EP8
#error  Endpoint 8 on Device 1 is already used!
#endif
#define USBD1_EP8                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  9) || (USBD_CDC3_EP_BULK_OUT ==  9))
#ifdef  USBD1_EP9
#error  Endpoint 9 on Device 1 is already used!
#endif
#define USBD1_EP9                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 10) || (USBD_CDC3_EP_BULK_OUT == 10))
#ifdef  USBD1_EP10
#error  Endpoint 10 on Device 1 is already used!
#endif
#define USBD1_EP10                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 11) || (USBD_CDC3_EP_BULK_OUT == 11))
#ifdef  USBD1_EP11
#error  Endpoint 11 on Device 1 is already used!
#endif
#define USBD1_EP11                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 12) || (USBD_CDC3_EP_BULK_OUT == 12))
#ifdef  USBD1_EP12
#error  Endpoint 12 on Device 1 is already used!
#endif
#define USBD1_EP12                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 13) || (USBD_CDC3_EP_BULK_OUT == 13))
#ifdef  USBD1_EP13
#error  Endpoint 13 on Device 1 is already used!
#endif
#define USBD1_EP13                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 14) || (USBD_CDC3_EP_BULK_OUT == 14))
#ifdef  USBD1_EP14
#error  Endpoint 14 on Device 1 is already used!
#endif
#define USBD1_EP14                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 15) || (USBD_CDC3_EP_BULK_OUT == 15))
#ifdef  USBD1_EP15
#error  Endpoint 15 on Device 1 is already used!
#endif
#define USBD1_EP15                     &usbd_cdc_thread_id[7]
#endif 
#else
#define USBD1_CDC3                      0
#define USBD1_CDC3_EP_INT_IN            0
#define USBD1_CDC3_EP_BULK_IN           0
#define USBD1_CDC3_EP_BULK_OUT          0
#define USBD1_CDC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CLS_0) && (USBD_CLS0_DEV == 1))
#define USBD1_CLS0                      1
#else
#define USBD1_CLS0                      0
#endif

#if    (defined(RTE_USB_Device_CLS_1) && (USBD_CLS1_DEV == 1))
#define USBD1_CLS1                      1
#else
#define USBD1_CLS1                      0
#endif

#if    (defined(RTE_USB_Device_CLS_2) && (USBD_CLS2_DEV == 1))
#define USBD1_CLS2                      1
#else
#define USBD1_CLS2                      0
#endif

#if    (defined(RTE_USB_Device_CLS_3) && (USBD_CLS3_DEV == 1))
#define USBD1_CLS3                      1
#else
#define USBD1_CLS3                      0
#endif

#define USBD1_HID_NUM                  (USBDn_c_NUM(1,HID))
#define USBD1_MSC_NUM                  (USBDn_c_NUM(1,MSC))
#define USBD1_ADC_NUM                  (USBDn_c_NUM(1,ADC))
#define USBD1_CDC_NUM                  (USBDn_c_NUM(1,CDC))
#define USBD1_CLS_NUM                  (USBDn_c_NUM(1,CLS))
#define USBD1_ANY                     ((USBD1_HID_NUM + USBD1_MSC_NUM + USBD1_ADC_NUM + USBD1_CDC_NUM + USBD1_CLS_NUM) != 0)

#define USBD1_HID_EP_CALC    (MAX_OF_8 (USBD1_HID0_EP_INT_IN, USBD1_HID0_EP_INT_OUT,  \
                                        USBD1_HID1_EP_INT_IN, USBD1_HID1_EP_INT_OUT,  \
                                        USBD1_HID2_EP_INT_IN, USBD1_HID2_EP_INT_OUT,  \
                                        USBD1_HID3_EP_INT_IN, USBD1_HID3_EP_INT_OUT))
#define USBD1_MSC_EP_CALC    (MAX_OF_8 (USBD1_MSC0_EP_BULK_IN, USBD1_MSC0_EP_BULK_OUT,  \
                                        USBD1_MSC1_EP_BULK_IN, USBD1_MSC1_EP_BULK_OUT,  \
                                        USBD1_MSC2_EP_BULK_IN, USBD1_MSC2_EP_BULK_OUT,  \
                                        USBD1_MSC3_EP_BULK_IN, USBD1_MSC3_EP_BULK_OUT))
#define USBD1_ADC_EP_CALC    (MAX_OF_4 (USBD1_ADC0_EP_ISO_OUT,  \
                                        USBD1_ADC1_EP_ISO_OUT,  \
                                        USBD1_ADC2_EP_ISO_OUT,  \
                                        USBD1_ADC3_EP_ISO_OUT))
#define USBD1_CDC_EP_CALC    (MAX_OF_12(USBD1_CDC0_EP_INT_IN, USBD1_CDC0_EP_BULK_IN, USBD1_CDC0_EP_BULK_OUT, \
                                        USBD1_CDC1_EP_INT_IN, USBD1_CDC1_EP_BULK_IN, USBD1_CDC1_EP_BULK_OUT, \
                                        USBD1_CDC2_EP_INT_IN, USBD1_CDC2_EP_BULK_IN, USBD1_CDC2_EP_BULK_OUT, \
                                        USBD1_CDC3_EP_INT_IN, USBD1_CDC3_EP_BULK_IN, USBD1_CDC3_EP_BULK_OUT))

#define USBD1_IF_NUM                   (USBD1_HID_NUM+USBD1_MSC_NUM+(USBD1_ADC_NUM*2)+(USBD1_CDC_NUM*2)+USBD1_CLS_NUM)
#define USBD1_EP_NUM         (MAX_OF_4 (USBD1_HID_EP_CALC,USBD1_MSC_EP_CALC,USBD1_ADC_EP_CALC,USBD1_CDC_EP_CALC))

#if    (USBD1_ADC0_IF_NUM != 0)
#define USBD_ADC0_CIF                (0)
#define USBD_ADC0_SIF                (1+USBD_ADC0_CIF)
#define USBD_ADC0_CIF_STR            (0+USBD1_STR_DESC_SER_EN+3)
#define USBD_ADC0_SIF1_STR           (1+USBD_ADC0_CIF_STR)
#define USBD_ADC0_SIF2_STR           (2+USBD_ADC0_CIF_STR)
#endif
#if    (USBD1_ADC1_IF_NUM != 0)
#define USBD_ADC1_CIF                  (USBD1_ADC0_IF_NUM                                                                       )
#define USBD_ADC1_SIF                (1+USBD_ADC1_CIF)
#define USBD_ADC1_CIF_STR             ((USBD1_ADC0_IF_NUM/2                                                                     )*3+\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC1_SIF1_STR           (1+USBD_ADC1_CIF_STR)
#define USBD_ADC1_SIF2_STR           (2+USBD_ADC1_CIF_STR)
#endif
#if    (USBD1_ADC2_IF_NUM != 0)
#define USBD_ADC2_CIF                  (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM                                                 )
#define USBD_ADC2_SIF                (1+USBD_ADC2_CIF)
#define USBD_ADC2_CIF_STR             ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2                                               )*3+\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC2_SIF1_STR           (1+USBD_ADC2_CIF_STR)
#define USBD_ADC2_SIF2_STR           (2+USBD_ADC2_CIF_STR)
#endif
#if    (USBD1_ADC3_IF_NUM != 0)
#define USBD_ADC3_CIF                  (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM                           )
#define USBD_ADC3_SIF                (1+USBD_ADC3_CIF)
#define USBD_ADC3_CIF_STR             ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2                         )*3+\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC3_SIF1_STR           (1+USBD_ADC3_CIF_STR)
#define USBD_ADC3_SIF2_STR           (2+USBD_ADC3_CIF_STR)
#endif

#if    (USBD1_CDC0_IF_NUM != 0)
#define USBD_CDC0_CIF                  (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM     )
#define USBD_CDC0_DIF                (1+USBD_CDC0_CIF)
#define USBD_CDC0_CIF_STR             ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC0_DIF_STR            (1+USBD_CDC0_CIF_STR)
#endif
#if    (USBD1_CDC1_IF_NUM != 0)
#define USBD_CDC1_CIF                  (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM                                                                       )
#define USBD_CDC1_DIF                (1+USBD_CDC1_CIF)
#define USBD_CDC1_CIF_STR             ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM                                                                        +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC1_DIF_STR            (1+USBD_CDC1_CIF_STR)
#endif
#if    (USBD1_CDC2_IF_NUM != 0)
#define USBD_CDC2_CIF                  (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM                                                 )
#define USBD_CDC2_DIF                (1+USBD_CDC2_CIF)
#define USBD_CDC2_CIF_STR             ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM                                                  +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC2_DIF_STR            (1+USBD_CDC2_CIF_STR)
#endif
#if    (USBD1_CDC3_IF_NUM != 0)
#define USBD_CDC3_CIF                  (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM                           )
#define USBD_CDC3_DIF                (1+USBD_CDC3_CIF)
#define USBD_CDC3_CIF_STR             ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM                            +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC3_DIF_STR            (1+USBD_CDC3_CIF_STR)
#endif

#if    (USBD1_HID0_IF_NUM != 0)
#define USBD_HID0_IF                   (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM     )
#define USBD_HID0_STR                 ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD1_HID1_IF_NUM != 0)
#define USBD_HID1_IF                   (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM                                                                       )
#define USBD_HID1_STR                 ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM                                                                        +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD1_HID2_IF_NUM != 0)
#define USBD_HID2_IF                   (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM                                                 )
#define USBD_HID2_STR                 ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM                                                  +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD1_HID3_IF_NUM != 0)
#define USBD_HID3_IF                   (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM                           )
#define USBD_HID3_STR                 ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM                            +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#endif

#if    (USBD1_MSC0_IF_NUM != 0)
#define USBD_MSC0_IF                   (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM    +USBD1_HID3_IF_NUM     )
#define USBD_MSC0_STR                 ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM    +USBD1_HID2_IF_NUM      +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD1_MSC1_IF_NUM != 0)
#define USBD_MSC1_IF                   (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM    +USBD1_HID3_IF_NUM      +\
                                        USBD1_MSC0_IF_NUM                                                                       )
#define USBD_MSC1_STR                 ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM    +USBD1_HID2_IF_NUM      +\
                                        USBD1_MSC0_IF_NUM                                                                        +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD1_MSC2_IF_NUM != 0)
#define USBD_MSC2_IF                   (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM    +USBD1_HID3_IF_NUM      +\
                                        USBD1_MSC0_IF_NUM    +USBD1_MSC1_IF_NUM                                                 )
#define USBD_MSC2_STR                 ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM    +USBD1_HID2_IF_NUM      +\
                                        USBD1_MSC0_IF_NUM    +USBD1_MSC1_IF_NUM                                                  +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD1_MSC3_IF_NUM != 0)
#define USBD_MSC3_IF                   (USBD1_ADC0_IF_NUM    +USBD1_ADC1_IF_NUM    +USBD1_ADC2_IF_NUM    +USBD1_ADC3_IF_NUM      +\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM    +USBD1_HID3_IF_NUM      +\
                                        USBD1_MSC0_IF_NUM    +USBD1_MSC1_IF_NUM    +USBD1_MSC2_IF_NUM                           )
#define USBD_MSC3_STR                 ((USBD1_ADC0_IF_NUM/2  +USBD1_ADC1_IF_NUM/2  +USBD1_ADC2_IF_NUM/2  +USBD1_ADC3_IF_NUM/2   )*3+\
                                        USBD1_CDC0_IF_NUM    +USBD1_CDC1_IF_NUM    +USBD1_CDC2_IF_NUM    +USBD1_CDC3_IF_NUM      +\
                                        USBD1_HID0_IF_NUM    +USBD1_HID1_IF_NUM    +USBD1_HID2_IF_NUM    +USBD1_HID2_IF_NUM      +\
                                        USBD1_MSC0_IF_NUM    +USBD1_MSC1_IF_NUM    +USBD1_MSC2_IF_NUM                            +\
                                        USBD1_STR_DESC_SER_EN+3                                                                 )
#endif

#define USBD1_USE_IAD                 ((USBD1_CDC_NUM>0)*((USBD1_HID_NUM>0)|(USBD1_MSC_NUM>0)|(USBD1_ADC_NUM>0)|(USBD1_CDC_NUM>0)))

#define USBD1_ADC_DESC_LEN             (USBD1_ADC_NUM *                                                                 \
                                       (USBD1_USE_IAD * USB_INTERFACE_ASSOC_DESC_SIZE +                                 \
                                        3 * USB_INTERFACE_DESC_SIZE + AUDIO_CONTROL_INTERFACE_DESC_SZ(1) +              \
                                        AUDIO_INPUT_TERMINAL_DESC_SIZE + AUDIO_FEATURE_UNIT_DESC_SZ(1,1) +              \
                                        AUDIO_OUTPUT_TERMINAL_DESC_SIZE + AUDIO_STREAMING_INTERFACE_DESC_SIZE +         \
                                        AUDIO_FORMAT_TYPE_I_DESC_SZ(1) + AUDIO_STANDARD_ENDPOINT_DESC_SIZE +            \
                                        AUDIO_STREAMING_ENDPOINT_DESC_SIZE))
#define USBD1_CDC_DESC_LEN             (USBD1_CDC_NUM *                                                                 \
                                       (USBD1_USE_IAD * USB_INTERFACE_ASSOC_DESC_SIZE +                                 \
                                        USB_INTERFACE_DESC_SIZE + 0x0013 + USB_ENDPOINT_DESC_SIZE +                     \
                                        USB_INTERFACE_DESC_SIZE + 2* USB_ENDPOINT_DESC_SIZE))
#define USBD1_MSC_DESC_LEN             (USBD1_MSC_NUM *                                                                 \
                                       (USB_INTERFACE_DESC_SIZE + 2 * USB_ENDPOINT_DESC_SIZE))
#if (USBD1_HID0 == 1)
#define USBD1_HID0_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD1_HID0_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD1_HID0_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD1_ADC_NUM * USBD1_ADC_DESC_LEN +                                            \
                                        USBD1_CDC_NUM * USBD1_CDC_DESC_LEN)
#define USBD_HID0_DESC_LEN             (USBD1_HID0_DESC_LEN)
#define USBD_HID0_DESC_OFS             (USBD1_HID0_DESC_OFS)
#else
#define USBD1_HID0_DESC_LEN            (0)
#define USBD1_HID0_DESC_OFS            (0)
#endif
#if (USBD1_HID1 == 1)
#define USBD1_HID1_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD1_HID1_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD1_HID1_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD1_ADC_NUM * USBD1_ADC_DESC_LEN +                                            \
                                        USBD1_CDC_NUM * USBD1_CDC_DESC_LEN +                                            \
                                        USBD1_HID0_DESC_OFS)
#define USBD_HID1_DESC_LEN             (USBD1_HID1_DESC_LEN)
#define USBD_HID1_DESC_OFS             (USBD1_HID1_DESC_OFS)
#else
#define USBD1_HID1_DESC_LEN            (0)
#define USBD1_HID1_DESC_OFS            (0)
#endif
#if (USBD1_HID2 == 1)
#define USBD1_HID2_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD1_HID2_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD1_HID2_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD1_ADC_NUM * USBD1_ADC_DESC_LEN +                                            \
                                        USBD1_CDC_NUM * USBD1_CDC_DESC_LEN +                                            \
                                        USBD1_HID0_DESC_OFS +                                                           \
                                        USBD1_HID1_DESC_OFS)
#define USBD_HID2_DESC_LEN             (USBD1_HID2_DESC_LEN)
#define USBD_HID2_DESC_OFS             (USBD1_HID2_DESC_OFS)
#else
#define USBD1_HID2_DESC_LEN            (0)
#define USBD1_HID2_DESC_OFS            (0)
#endif
#if (USBD1_HID3 == 1)
#define USBD1_HID3_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD1_HID3_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD1_HID3_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD1_ADC_NUM * USBD1_ADC_DESC_LEN +                                            \
                                        USBD1_CDC_NUM * USBD1_CDC_DESC_LEN +                                            \
                                        USBD1_HID0_DESC_OFS +                                                           \
                                        USBD1_HID1_DESC_OFS +                                                           \
                                        USBD1_HID2_DESC_OFS)
#define USBD_HID3_DESC_LEN             (USBD1_HID3_DESC_LEN)
#define USBD_HID3_DESC_OFS             (USBD1_HID3_DESC_OFS)
#else
#define USBD1_HID3_DESC_LEN            (0)
#define USBD1_HID3_DESC_OFS            (0)
#endif
#define USBD1_HID_DESC_LEN             (USBD1_HID0_DESC_LEN +                                                           \
                                        USBD1_HID1_DESC_LEN +                                                           \
                                        USBD1_HID2_DESC_LEN +                                                           \
                                        USBD1_HID3_DESC_LEN )                                                           \

#define USBD1_WTOTALLENGTH             (USB_CONFIGURATION_DESC_SIZE +                                                   \
                                        USBD1_ADC_DESC_LEN +                                                            \
                                        USBD1_CDC_DESC_LEN +                                                            \
                                        USBD1_HID_DESC_LEN +                                                            \
                                        USBD1_MSC_DESC_LEN )

#if ((USBD1_HS) || (USBD1_USE_IAD))
#define USBD1_BDC_USB                   0x0200
#else
#define USBD1_BDC_USB                   0x0110
#endif

#endif  /* USBD1_MACROS */

#if     (defined(USBD1_PORT) && !defined(USBD1_PORT_DEF))
#define  USBD1_PORT_DEF                 1
#ifndef  __NO_USB_LIB_C

extern  ARM_DRIVER_USBD  USBDn_DRIVER(USBD1_PORT);

uint8_t         usbd1_alt_setting      [USBD1_IF_NUM];
uint8_t         usbd1_ep0_buf          [USBD1_MAX_PACKET0] __attribute__((aligned(4)));
usbd_data_t     usbd1_data;
usbd_dev_t      usbd1_dev           = { 
                                       &USBDn_DRIVER(USBD1_PORT), 
                                        usbd1_alt_setting, 
                                        usbd1_ep0_buf, 
                                       &usbd1_data, 
                                        USBD1_CFG_DESC_BMATTRIBUTES, 
                                        USBD1_HS, 
                                        USBD1_IF_NUM, 
                                        USBD1_EP_NUM, 
                                        USBD1_MAX_PACKET0, 
                                      };

#endif  /* __NO_USB_LIB_C   */
#endif


/*------------------------------------------------------------------------------
 *      USB Device 2 Macros, Calculations and Constants for Library
 *----------------------------------------------------------------------------*/

#ifndef USBD2_MACROS
#define USBD2_MACROS                    1

#if    (defined(RTE_USB_Device_HID_0) && (USBD_HID0_DEV == 2))
#define USBD2_HID0                      1
#define USBD2_HID0_EP_INT_IN            USBD_HID0_EP_INT_IN
#define USBD2_HID0_EP_INT_OUT           USBD_HID0_EP_INT_OUT
#define USBD2_HID0_IF_NUM               1
#if   ((USBD_HID0_EP_INT_IN ==  1) || (USBD_HID0_EP_INT_OUT ==  1))
#ifdef  USBD2_EP1 
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  2) || (USBD_HID0_EP_INT_OUT ==  2))
#ifdef  USBD2_EP2 
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  3) || (USBD_HID0_EP_INT_OUT ==  3))
#ifdef  USBD2_EP3 
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  4) || (USBD_HID0_EP_INT_OUT ==  4))
#ifdef  USBD2_EP4 
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  5) || (USBD_HID0_EP_INT_OUT ==  5))
#ifdef  USBD2_EP5 
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  6) || (USBD_HID0_EP_INT_OUT ==  6))
#ifdef  USBD2_EP6 
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  7) || (USBD_HID0_EP_INT_OUT ==  7))
#ifdef  USBD2_EP7 
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  8) || (USBD_HID0_EP_INT_OUT ==  8))
#ifdef  USBD2_EP8 
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  9) || (USBD_HID0_EP_INT_OUT ==  9))
#ifdef  USBD2_EP9 
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 10) || (USBD_HID0_EP_INT_OUT == 10))
#ifdef  USBD2_EP10 
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 11) || (USBD_HID0_EP_INT_OUT == 11))
#ifdef  USBD2_EP11 
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 12) || (USBD_HID0_EP_INT_OUT == 12))
#ifdef  USBD2_EP12 
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 13) || (USBD_HID0_EP_INT_OUT == 13))
#ifdef  USBD2_EP13 
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 14) || (USBD_HID0_EP_INT_OUT == 14))
#ifdef  USBD2_EP14 
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 15) || (USBD_HID0_EP_INT_OUT == 15))
#ifdef  USBD2_EP15 
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_hid_thread_id[0]
#endif 
#else
#define USBD2_HID0                      0
#define USBD2_HID0_EP_INT_IN            0
#define USBD2_HID0_EP_INT_OUT           0
#define USBD2_HID0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_1) && (USBD_HID1_DEV == 2))
#define USBD2_HID1                      1
#define USBD2_HID1_EP_INT_IN            USBD_HID1_EP_INT_IN
#define USBD2_HID1_EP_INT_OUT           USBD_HID1_EP_INT_OUT
#define USBD2_HID1_IF_NUM               1
#if   ((USBD_HID1_EP_INT_IN ==  1) || (USBD_HID1_EP_INT_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  2) || (USBD_HID1_EP_INT_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  3) || (USBD_HID1_EP_INT_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  4) || (USBD_HID1_EP_INT_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  5) || (USBD_HID1_EP_INT_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  6) || (USBD_HID1_EP_INT_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  7) || (USBD_HID1_EP_INT_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  8) || (USBD_HID1_EP_INT_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  9) || (USBD_HID1_EP_INT_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 10) || (USBD_HID1_EP_INT_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 11) || (USBD_HID1_EP_INT_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 12) || (USBD_HID1_EP_INT_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 13) || (USBD_HID1_EP_INT_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 14) || (USBD_HID1_EP_INT_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 15) || (USBD_HID1_EP_INT_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_hid_thread_id[1]
#endif 
#else
#define USBD2_HID1                      0
#define USBD2_HID1_EP_INT_IN            0
#define USBD2_HID1_EP_INT_OUT           0
#define USBD2_HID1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_2) && (USBD_HID2_DEV == 2))
#define USBD2_HID2                      1
#define USBD2_HID2_EP_INT_IN            USBD_HID2_EP_INT_IN
#define USBD2_HID2_EP_INT_OUT           USBD_HID2_EP_INT_OUT
#define USBD2_HID2_IF_NUM               1
#if   ((USBD_HID2_EP_INT_IN ==  1) || (USBD_HID2_EP_INT_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  2) || (USBD_HID2_EP_INT_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  3) || (USBD_HID2_EP_INT_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  4) || (USBD_HID2_EP_INT_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  5) || (USBD_HID2_EP_INT_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  6) || (USBD_HID2_EP_INT_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  7) || (USBD_HID2_EP_INT_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  8) || (USBD_HID2_EP_INT_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  9) || (USBD_HID2_EP_INT_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 10) || (USBD_HID2_EP_INT_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 11) || (USBD_HID2_EP_INT_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 12) || (USBD_HID2_EP_INT_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 13) || (USBD_HID2_EP_INT_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 14) || (USBD_HID2_EP_INT_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 15) || (USBD_HID2_EP_INT_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_hid_thread_id[2]
#endif 
#else
#define USBD2_HID2                      0
#define USBD2_HID2_EP_INT_IN            0
#define USBD2_HID2_EP_INT_OUT           0
#define USBD2_HID2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_3) && (USBD_HID3_DEV == 2))
#define USBD2_HID3                      1
#define USBD2_HID3_EP_INT_IN            USBD_HID3_EP_INT_IN
#define USBD2_HID3_EP_INT_OUT           USBD_HID3_EP_INT_OUT
#define USBD2_HID3_IF_NUM               1
#if   ((USBD_HID3_EP_INT_IN ==  1) || (USBD_HID3_EP_INT_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  2) || (USBD_HID3_EP_INT_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  3) || (USBD_HID3_EP_INT_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  4) || (USBD_HID3_EP_INT_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  5) || (USBD_HID3_EP_INT_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  6) || (USBD_HID3_EP_INT_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  7) || (USBD_HID3_EP_INT_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  8) || (USBD_HID3_EP_INT_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  9) || (USBD_HID3_EP_INT_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 10) || (USBD_HID3_EP_INT_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 11) || (USBD_HID3_EP_INT_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 12) || (USBD_HID3_EP_INT_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 13) || (USBD_HID3_EP_INT_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 14) || (USBD_HID3_EP_INT_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 15) || (USBD_HID3_EP_INT_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_hid_thread_id[3]
#endif 
#else
#define USBD2_HID3                      0
#define USBD2_HID3_EP_INT_IN            0
#define USBD2_HID3_EP_INT_OUT           0
#define USBD2_HID3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_0) && (USBD_MSC0_DEV == 2))
#define USBD2_MSC0                      1
#define USBD2_MSC0_EP_BULK_IN           USBD_MSC0_EP_BULK_IN
#define USBD2_MSC0_EP_BULK_OUT          USBD_MSC0_EP_BULK_OUT
#define USBD2_MSC0_IF_NUM               1
#if   ((USBD_MSC0_EP_BULK_IN ==  1) || (USBD_MSC0_EP_BULK_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  2) || (USBD_MSC0_EP_BULK_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif            
#define USBD2_EP2                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  3) || (USBD_MSC0_EP_BULK_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif           
#define USBD2_EP3                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  4) || (USBD_MSC0_EP_BULK_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  5) || (USBD_MSC0_EP_BULK_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  6) || (USBD_MSC0_EP_BULK_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  7) || (USBD_MSC0_EP_BULK_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  8) || (USBD_MSC0_EP_BULK_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  9) || (USBD_MSC0_EP_BULK_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 10) || (USBD_MSC0_EP_BULK_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 11) || (USBD_MSC0_EP_BULK_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 12) || (USBD_MSC0_EP_BULK_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 13) || (USBD_MSC0_EP_BULK_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 14) || (USBD_MSC0_EP_BULK_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 15) || (USBD_MSC0_EP_BULK_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_msc_thread_id[0]
#endif 
#else
#define USBD2_MSC0                      0
#define USBD2_MSC0_EP_BULK_IN           0
#define USBD2_MSC0_EP_BULK_OUT          0
#define USBD2_MSC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_1) && (USBD_MSC1_DEV == 2))
#define USBD2_MSC1                      1
#define USBD2_MSC1_EP_BULK_IN           USBD_MSC1_EP_BULK_IN
#define USBD2_MSC1_EP_BULK_OUT          USBD_MSC1_EP_BULK_OUT
#define USBD2_MSC1_IF_NUM               1
#if   ((USBD_MSC1_EP_BULK_IN ==  1) || (USBD_MSC1_EP_BULK_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  2) || (USBD_MSC1_EP_BULK_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  3) || (USBD_MSC1_EP_BULK_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  4) || (USBD_MSC1_EP_BULK_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  5) || (USBD_MSC1_EP_BULK_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  6) || (USBD_MSC1_EP_BULK_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  7) || (USBD_MSC1_EP_BULK_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  8) || (USBD_MSC1_EP_BULK_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  9) || (USBD_MSC1_EP_BULK_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 10) || (USBD_MSC1_EP_BULK_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 11) || (USBD_MSC1_EP_BULK_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 12) || (USBD_MSC1_EP_BULK_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 13) || (USBD_MSC1_EP_BULK_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 14) || (USBD_MSC1_EP_BULK_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 15) || (USBD_MSC1_EP_BULK_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_msc_thread_id[1]
#endif 
#else
#define USBD2_MSC1                      0
#define USBD2_MSC1_EP_BULK_IN           0
#define USBD2_MSC1_EP_BULK_OUT          0
#define USBD2_MSC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_2) && (USBD_MSC2_DEV == 2))
#define USBD2_MSC2                      1
#define USBD2_MSC2_EP_BULK_IN           USBD_MSC2_EP_BULK_IN
#define USBD2_MSC2_EP_BULK_OUT          USBD_MSC2_EP_BULK_OUT
#define USBD2_MSC2_IF_NUM               1
#if   ((USBD_MSC2_EP_BULK_IN ==  1) || (USBD_MSC2_EP_BULK_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  2) || (USBD_MSC2_EP_BULK_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  3) || (USBD_MSC2_EP_BULK_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  4) || (USBD_MSC2_EP_BULK_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  5) || (USBD_MSC2_EP_BULK_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  6) || (USBD_MSC2_EP_BULK_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  7) || (USBD_MSC2_EP_BULK_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  8) || (USBD_MSC2_EP_BULK_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  9) || (USBD_MSC2_EP_BULK_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 10) || (USBD_MSC2_EP_BULK_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 11) || (USBD_MSC2_EP_BULK_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 12) || (USBD_MSC2_EP_BULK_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 13) || (USBD_MSC2_EP_BULK_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 14) || (USBD_MSC2_EP_BULK_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 15) || (USBD_MSC2_EP_BULK_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_msc_thread_id[2]
#endif 
#else
#define USBD2_MSC2                      0
#define USBD2_MSC2_EP_BULK_IN           0
#define USBD2_MSC2_EP_BULK_OUT          0
#define USBD2_MSC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_3) && (USBD_MSC3_DEV == 2))
#define USBD2_MSC3                      1
#define USBD2_MSC3_EP_BULK_IN           USBD_MSC3_EP_BULK_IN
#define USBD2_MSC3_EP_BULK_OUT          USBD_MSC3_EP_BULK_OUT
#define USBD2_MSC3_IF_NUM               1
#if   ((USBD_MSC3_EP_BULK_IN ==  1) || (USBD_MSC3_EP_BULK_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  2) || (USBD_MSC3_EP_BULK_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  3) || (USBD_MSC3_EP_BULK_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  4) || (USBD_MSC3_EP_BULK_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  5) || (USBD_MSC3_EP_BULK_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  6) || (USBD_MSC3_EP_BULK_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  7) || (USBD_MSC3_EP_BULK_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  8) || (USBD_MSC3_EP_BULK_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  9) || (USBD_MSC3_EP_BULK_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 10) || (USBD_MSC3_EP_BULK_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 11) || (USBD_MSC3_EP_BULK_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 12) || (USBD_MSC3_EP_BULK_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 13) || (USBD_MSC3_EP_BULK_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 14) || (USBD_MSC3_EP_BULK_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 15) || (USBD_MSC3_EP_BULK_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_msc_thread_id[3]
#endif 
#else
#define USBD2_MSC3                      0
#define USBD2_MSC3_EP_BULK_IN           0
#define USBD2_MSC3_EP_BULK_OUT          0
#define USBD2_MSC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_0) && (USBD_ADC0_DEV == 2))
#define USBD2_ADC0                      1
#define USBD2_ADC0_EP_ISO_OUT           USBD_ADC0_EP_ISO_OUT
#define USBD2_ADC0_IF_NUM               2
#if   ((USBD_ADC0_EP_ISO_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_adc_thread_id[0]
#endif 
#else
#define USBD2_ADC0                      0
#define USBD2_ADC0_EP_ISO_OUT           0
#define USBD2_ADC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_1) && (USBD_ADC1_DEV == 2))
#define USBD2_ADC1                      1
#define USBD2_ADC1_EP_ISO_OUT           USBD_ADC1_EP_ISO_OUT
#define USBD2_ADC1_IF_NUM               2
#if   ((USBD_ADC1_EP_ISO_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_adc_thread_id[1]
#endif 
#else
#define USBD2_ADC1                      0
#define USBD2_ADC1_EP_ISO_OUT           0
#define USBD2_ADC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_2) && (USBD_ADC2_DEV == 2))
#define USBD2_ADC2                      1
#define USBD2_ADC2_EP_ISO_OUT           USBD_ADC2_EP_ISO_OUT
#define USBD2_ADC2_IF_NUM               2
#if   ((USBD_ADC2_EP_ISO_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_adc_thread_id[2]
#endif 
#else
#define USBD2_ADC2                      0
#define USBD2_ADC2_EP_ISO_OUT           0
#define USBD2_ADC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_3) && (USBD_ADC3_DEV == 2))
#define USBD2_ADC3                      1
#define USBD2_ADC3_EP_ISO_OUT           USBD_ADC3_EP_ISO_OUT
#define USBD2_ADC3_IF_NUM               2
#if   ((USBD_ADC3_EP_ISO_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_adc_thread_id[3]
#endif 
#else
#define USBD2_ADC3                      0
#define USBD2_ADC3_EP_ISO_OUT           0
#define USBD2_ADC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_0) && (USBD_CDC0_DEV == 2))
#define USBD2_CDC0                      1
#define USBD2_CDC0_EP_INT_IN            USBD_CDC0_EP_INT_IN
#define USBD2_CDC0_EP_BULK_IN           USBD_CDC0_EP_BULK_IN
#define USBD2_CDC0_EP_BULK_OUT          USBD_CDC0_EP_BULK_OUT
#define USBD2_CDC0_IF_NUM               2
#if   ((USBD_CDC0_EP_INT_IN ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_cdc_thread_id[0]
#endif 
#if   ((USBD_CDC0_EP_BULK_IN ==  1) || (USBD_CDC0_EP_BULK_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  2) || (USBD_CDC0_EP_BULK_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  3) || (USBD_CDC0_EP_BULK_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  4) || (USBD_CDC0_EP_BULK_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  5) || (USBD_CDC0_EP_BULK_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  6) || (USBD_CDC0_EP_BULK_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  7) || (USBD_CDC0_EP_BULK_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  8) || (USBD_CDC0_EP_BULK_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  9) || (USBD_CDC0_EP_BULK_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 10) || (USBD_CDC0_EP_BULK_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 11) || (USBD_CDC0_EP_BULK_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 12) || (USBD_CDC0_EP_BULK_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 13) || (USBD_CDC0_EP_BULK_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 14) || (USBD_CDC0_EP_BULK_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 15) || (USBD_CDC0_EP_BULK_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_cdc_thread_id[1]
#endif 
#else
#define USBD2_CDC0                      0
#define USBD2_CDC0_EP_INT_IN            0
#define USBD2_CDC0_EP_BULK_IN           0
#define USBD2_CDC0_EP_BULK_OUT          0
#define USBD2_CDC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_1) && (USBD_CDC1_DEV == 2))
#define USBD2_CDC1                      1
#define USBD2_CDC1_EP_INT_IN            USBD_CDC1_EP_INT_IN
#define USBD2_CDC1_EP_BULK_IN           USBD_CDC1_EP_BULK_IN
#define USBD2_CDC1_EP_BULK_OUT          USBD_CDC1_EP_BULK_OUT
#define USBD2_CDC1_IF_NUM               2
#if   ((USBD_CDC1_EP_INT_IN ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_cdc_thread_id[2]
#endif 
#if   ((USBD_CDC1_EP_BULK_IN ==  1) || (USBD_CDC1_EP_BULK_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  2) || (USBD_CDC1_EP_BULK_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  3) || (USBD_CDC1_EP_BULK_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  4) || (USBD_CDC1_EP_BULK_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  5) || (USBD_CDC1_EP_BULK_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  6) || (USBD_CDC1_EP_BULK_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  7) || (USBD_CDC1_EP_BULK_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  8) || (USBD_CDC1_EP_BULK_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  9) || (USBD_CDC1_EP_BULK_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 10) || (USBD_CDC1_EP_BULK_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 11) || (USBD_CDC1_EP_BULK_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 12) || (USBD_CDC1_EP_BULK_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 13) || (USBD_CDC1_EP_BULK_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 14) || (USBD_CDC1_EP_BULK_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 15) || (USBD_CDC1_EP_BULK_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_cdc_thread_id[3]
#endif 
#else
#define USBD2_CDC1                      0
#define USBD2_CDC1_EP_INT_IN            0
#define USBD2_CDC1_EP_BULK_IN           0
#define USBD2_CDC1_EP_BULK_OUT          0
#define USBD2_CDC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_2) && (USBD_CDC2_DEV == 2))
#define USBD2_CDC2                      1
#define USBD2_CDC2_EP_INT_IN            USBD_CDC2_EP_INT_IN
#define USBD2_CDC2_EP_BULK_IN           USBD_CDC2_EP_BULK_IN
#define USBD2_CDC2_EP_BULK_OUT          USBD_CDC2_EP_BULK_OUT
#define USBD2_CDC2_IF_NUM               2
#if   ((USBD_CDC2_EP_INT_IN ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_cdc_thread_id[4]
#endif 
#if   ((USBD_CDC2_EP_BULK_IN ==  1) || (USBD_CDC2_EP_BULK_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  2) || (USBD_CDC2_EP_BULK_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  3) || (USBD_CDC2_EP_BULK_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  4) || (USBD_CDC2_EP_BULK_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  5) || (USBD_CDC2_EP_BULK_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  6) || (USBD_CDC2_EP_BULK_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  7) || (USBD_CDC2_EP_BULK_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  8) || (USBD_CDC2_EP_BULK_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  9) || (USBD_CDC2_EP_BULK_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 10) || (USBD_CDC2_EP_BULK_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 11) || (USBD_CDC2_EP_BULK_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 12) || (USBD_CDC2_EP_BULK_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 13) || (USBD_CDC2_EP_BULK_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 14) || (USBD_CDC2_EP_BULK_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 15) || (USBD_CDC2_EP_BULK_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_cdc_thread_id[5]
#endif 
#else
#define USBD2_CDC2                      0
#define USBD2_CDC2_EP_INT_IN            0
#define USBD2_CDC2_EP_BULK_IN           0
#define USBD2_CDC2_EP_BULK_OUT          0
#define USBD2_CDC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_3) && (USBD_CDC3_DEV == 2))
#define USBD2_CDC3                      1
#define USBD2_CDC3_EP_INT_IN            USBD_CDC3_EP_INT_IN
#define USBD2_CDC3_EP_BULK_IN           USBD_CDC3_EP_BULK_IN
#define USBD2_CDC3_EP_BULK_OUT          USBD_CDC3_EP_BULK_OUT
#define USBD2_CDC3_IF_NUM               2
#if   ((USBD_CDC3_EP_INT_IN ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_cdc_thread_id[6]
#endif 
#if   ((USBD_CDC3_EP_BULK_IN ==  1) || (USBD_CDC3_EP_BULK_OUT ==  1))
#ifdef  USBD2_EP1
#error  Endpoint 1 on Device 2 is already used!
#endif
#define USBD2_EP1                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  2) || (USBD_CDC3_EP_BULK_OUT ==  2))
#ifdef  USBD2_EP2
#error  Endpoint 2 on Device 2 is already used!
#endif
#define USBD2_EP2                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  3) || (USBD_CDC3_EP_BULK_OUT ==  3))
#ifdef  USBD2_EP3
#error  Endpoint 3 on Device 2 is already used!
#endif
#define USBD2_EP3                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  4) || (USBD_CDC3_EP_BULK_OUT ==  4))
#ifdef  USBD2_EP4
#error  Endpoint 4 on Device 2 is already used!
#endif
#define USBD2_EP4                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  5) || (USBD_CDC3_EP_BULK_OUT ==  5))
#ifdef  USBD2_EP5
#error  Endpoint 5 on Device 2 is already used!
#endif
#define USBD2_EP5                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  6) || (USBD_CDC3_EP_BULK_OUT ==  6))
#ifdef  USBD2_EP6
#error  Endpoint 6 on Device 2 is already used!
#endif
#define USBD2_EP6                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  7) || (USBD_CDC3_EP_BULK_OUT ==  7))
#ifdef  USBD2_EP7
#error  Endpoint 7 on Device 2 is already used!
#endif
#define USBD2_EP7                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  8) || (USBD_CDC3_EP_BULK_OUT ==  8))
#ifdef  USBD2_EP8
#error  Endpoint 8 on Device 2 is already used!
#endif
#define USBD2_EP8                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  9) || (USBD_CDC3_EP_BULK_OUT ==  9))
#ifdef  USBD2_EP9
#error  Endpoint 9 on Device 2 is already used!
#endif
#define USBD2_EP9                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 10) || (USBD_CDC3_EP_BULK_OUT == 10))
#ifdef  USBD2_EP10
#error  Endpoint 10 on Device 2 is already used!
#endif
#define USBD2_EP10                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 11) || (USBD_CDC3_EP_BULK_OUT == 11))
#ifdef  USBD2_EP11
#error  Endpoint 11 on Device 2 is already used!
#endif
#define USBD2_EP11                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 12) || (USBD_CDC3_EP_BULK_OUT == 12))
#ifdef  USBD2_EP12
#error  Endpoint 12 on Device 2 is already used!
#endif
#define USBD2_EP12                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 13) || (USBD_CDC3_EP_BULK_OUT == 13))
#ifdef  USBD2_EP13
#error  Endpoint 13 on Device 2 is already used!
#endif
#define USBD2_EP13                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 14) || (USBD_CDC3_EP_BULK_OUT == 14))
#ifdef  USBD2_EP14
#error  Endpoint 14 on Device 2 is already used!
#endif
#define USBD2_EP14                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 15) || (USBD_CDC3_EP_BULK_OUT == 15))
#ifdef  USBD2_EP15
#error  Endpoint 15 on Device 2 is already used!
#endif
#define USBD2_EP15                     &usbd_cdc_thread_id[7]
#endif 
#else
#define USBD2_CDC3                      0
#define USBD2_CDC3_EP_INT_IN            0
#define USBD2_CDC3_EP_BULK_IN           0
#define USBD2_CDC3_EP_BULK_OUT          0
#define USBD2_CDC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CLS_0) && (USBD_CLS0_DEV == 2))
#define USBD2_CLS0                      1
#else
#define USBD2_CLS0                      0
#endif

#if    (defined(RTE_USB_Device_CLS_1) && (USBD_CLS1_DEV == 2))
#define USBD2_CLS1                      1
#else
#define USBD2_CLS1                      0
#endif

#if    (defined(RTE_USB_Device_CLS_2) && (USBD_CLS2_DEV == 2))
#define USBD2_CLS2                      1
#else
#define USBD2_CLS2                      0
#endif

#if    (defined(RTE_USB_Device_CLS_3) && (USBD_CLS3_DEV == 2))
#define USBD2_CLS3                      1
#else
#define USBD2_CLS3                      0
#endif

#define USBD2_HID_NUM                  (USBDn_c_NUM(2,HID))
#define USBD2_MSC_NUM                  (USBDn_c_NUM(2,MSC))
#define USBD2_ADC_NUM                  (USBDn_c_NUM(2,ADC))
#define USBD2_CDC_NUM                  (USBDn_c_NUM(2,CDC))
#define USBD2_CLS_NUM                  (USBDn_c_NUM(2,CLS))
#define USBD2_ANY                     ((USBD2_HID_NUM + USBD2_MSC_NUM + USBD2_ADC_NUM + USBD2_CDC_NUM + USBD2_CLS_NUM) != 0)

#define USBD2_HID_EP_CALC    (MAX_OF_8 (USBD2_HID0_EP_INT_IN, USBD2_HID0_EP_INT_OUT,  \
                                        USBD2_HID1_EP_INT_IN, USBD2_HID1_EP_INT_OUT,  \
                                        USBD2_HID2_EP_INT_IN, USBD2_HID2_EP_INT_OUT,  \
                                        USBD2_HID3_EP_INT_IN, USBD2_HID3_EP_INT_OUT))
#define USBD2_MSC_EP_CALC    (MAX_OF_8 (USBD2_MSC0_EP_BULK_IN, USBD2_MSC0_EP_BULK_OUT,  \
                                        USBD2_MSC1_EP_BULK_IN, USBD2_MSC1_EP_BULK_OUT,  \
                                        USBD2_MSC2_EP_BULK_IN, USBD2_MSC2_EP_BULK_OUT,  \
                                        USBD2_MSC3_EP_BULK_IN, USBD2_MSC3_EP_BULK_OUT))
#define USBD2_ADC_EP_CALC    (MAX_OF_4 (USBD2_ADC0_EP_ISO_OUT,  \
                                        USBD2_ADC1_EP_ISO_OUT,  \
                                        USBD2_ADC2_EP_ISO_OUT,  \
                                        USBD2_ADC3_EP_ISO_OUT))
#define USBD2_CDC_EP_CALC    (MAX_OF_12(USBD2_CDC0_EP_INT_IN, USBD2_CDC0_EP_BULK_IN, USBD2_CDC0_EP_BULK_OUT, \
                                        USBD2_CDC1_EP_INT_IN, USBD2_CDC1_EP_BULK_IN, USBD2_CDC1_EP_BULK_OUT, \
                                        USBD2_CDC2_EP_INT_IN, USBD2_CDC2_EP_BULK_IN, USBD2_CDC2_EP_BULK_OUT, \
                                        USBD2_CDC3_EP_INT_IN, USBD2_CDC3_EP_BULK_IN, USBD2_CDC3_EP_BULK_OUT))

#define USBD2_IF_NUM                   (USBD2_HID_NUM+USBD2_MSC_NUM+(USBD2_ADC_NUM*2)+(USBD2_CDC_NUM*2)+USBD2_CLS_NUM)
#define USBD2_EP_NUM         (MAX_OF_4 (USBD2_HID_EP_CALC,USBD2_MSC_EP_CALC,USBD2_ADC_EP_CALC,USBD2_CDC_EP_CALC))

#if    (USBD2_ADC0_IF_NUM != 0)
#define USBD_ADC0_CIF                (0)
#define USBD_ADC0_SIF                (1+USBD_ADC0_CIF)
#define USBD_ADC0_CIF_STR            (0+USBD2_STR_DESC_SER_EN+3)
#define USBD_ADC0_SIF1_STR           (1+USBD_ADC0_CIF_STR)
#define USBD_ADC0_SIF2_STR           (2+USBD_ADC0_CIF_STR)
#endif
#if    (USBD2_ADC1_IF_NUM != 0)
#define USBD_ADC1_CIF                  (USBD2_ADC0_IF_NUM                                                                       )
#define USBD_ADC1_SIF                (1+USBD_ADC1_CIF)
#define USBD_ADC1_CIF_STR             ((USBD2_ADC0_IF_NUM/2                                                                     )*3+\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC1_SIF1_STR           (1+USBD_ADC1_CIF_STR)
#define USBD_ADC1_SIF2_STR           (2+USBD_ADC1_CIF_STR)
#endif
#if    (USBD2_ADC2_IF_NUM != 0)
#define USBD_ADC2_CIF                  (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM                                                 )
#define USBD_ADC2_SIF                (1+USBD_ADC2_CIF)
#define USBD_ADC2_CIF_STR             ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2                                               )*3+\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC2_SIF1_STR           (1+USBD_ADC2_CIF_STR)
#define USBD_ADC2_SIF2_STR           (2+USBD_ADC2_CIF_STR)
#endif
#if    (USBD2_ADC3_IF_NUM != 0)
#define USBD_ADC3_CIF                  (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM                           )
#define USBD_ADC3_SIF                (1+USBD_ADC3_CIF)
#define USBD_ADC3_CIF_STR             ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2                         )*3+\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC3_SIF1_STR           (1+USBD_ADC3_CIF_STR)
#define USBD_ADC3_SIF2_STR           (2+USBD_ADC3_CIF_STR)
#endif

#if    (USBD2_CDC0_IF_NUM != 0)
#define USBD_CDC0_CIF                  (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM     )
#define USBD_CDC0_DIF                (1+USBD_CDC0_CIF)
#define USBD_CDC0_CIF_STR             ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC0_DIF_STR            (1+USBD_CDC0_CIF_STR)
#endif
#if    (USBD2_CDC1_IF_NUM != 0)
#define USBD_CDC1_CIF                  (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM                                                                       )
#define USBD_CDC1_DIF                (1+USBD_CDC1_CIF)
#define USBD_CDC1_CIF_STR             ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM                                                                        +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC1_DIF_STR            (1+USBD_CDC1_CIF_STR)
#endif
#if    (USBD2_CDC2_IF_NUM != 0)
#define USBD_CDC2_CIF                  (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM                                                 )
#define USBD_CDC2_DIF                (1+USBD_CDC2_CIF)
#define USBD_CDC2_CIF_STR             ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM                                                  +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC2_DIF_STR            (1+USBD_CDC2_CIF_STR)
#endif
#if    (USBD2_CDC3_IF_NUM != 0)
#define USBD_CDC3_CIF                  (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM                           )
#define USBD_CDC3_DIF                (1+USBD_CDC3_CIF)
#define USBD_CDC3_CIF_STR             ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM                            +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC3_DIF_STR            (1+USBD_CDC3_CIF_STR)
#endif

#if    (USBD2_HID0_IF_NUM != 0)
#define USBD_HID0_IF                   (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM     )
#define USBD_HID0_STR                 ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD2_HID1_IF_NUM != 0)
#define USBD_HID1_IF                   (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM                                                                       )
#define USBD_HID1_STR                 ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM                                                                        +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD2_HID2_IF_NUM != 0)
#define USBD_HID2_IF                   (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM                                                 )
#define USBD_HID2_STR                 ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM                                                  +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD2_HID3_IF_NUM != 0)
#define USBD_HID3_IF                   (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM                           )
#define USBD_HID3_STR                 ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM                            +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#endif

#if    (USBD2_MSC0_IF_NUM != 0)
#define USBD_MSC0_IF                   (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM    +USBD2_HID3_IF_NUM     )
#define USBD_MSC0_STR                 ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM    +USBD2_HID2_IF_NUM      +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD2_MSC1_IF_NUM != 0)
#define USBD_MSC1_IF                   (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM    +USBD2_HID3_IF_NUM      +\
                                        USBD2_MSC0_IF_NUM                                                                       )
#define USBD_MSC1_STR                 ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM    +USBD2_HID2_IF_NUM      +\
                                        USBD2_MSC0_IF_NUM                                                                        +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD2_MSC2_IF_NUM != 0)
#define USBD_MSC2_IF                   (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM    +USBD2_HID3_IF_NUM      +\
                                        USBD2_MSC0_IF_NUM    +USBD2_MSC1_IF_NUM                                                 )
#define USBD_MSC2_STR                 ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM    +USBD2_HID2_IF_NUM      +\
                                        USBD2_MSC0_IF_NUM    +USBD2_MSC1_IF_NUM                                                  +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD2_MSC3_IF_NUM != 0)
#define USBD_MSC3_IF                   (USBD2_ADC0_IF_NUM    +USBD2_ADC1_IF_NUM    +USBD2_ADC2_IF_NUM    +USBD2_ADC3_IF_NUM      +\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM    +USBD2_HID3_IF_NUM      +\
                                        USBD2_MSC0_IF_NUM    +USBD2_MSC1_IF_NUM    +USBD2_MSC2_IF_NUM                           )
#define USBD_MSC3_STR                 ((USBD2_ADC0_IF_NUM/2  +USBD2_ADC1_IF_NUM/2  +USBD2_ADC2_IF_NUM/2  +USBD2_ADC3_IF_NUM/2   )*3+\
                                        USBD2_CDC0_IF_NUM    +USBD2_CDC1_IF_NUM    +USBD2_CDC2_IF_NUM    +USBD2_CDC3_IF_NUM      +\
                                        USBD2_HID0_IF_NUM    +USBD2_HID1_IF_NUM    +USBD2_HID2_IF_NUM    +USBD2_HID2_IF_NUM      +\
                                        USBD2_MSC0_IF_NUM    +USBD2_MSC1_IF_NUM    +USBD2_MSC2_IF_NUM                            +\
                                        USBD2_STR_DESC_SER_EN+3                                                                 )
#endif

#define USBD2_USE_IAD                 ((USBD2_CDC_NUM>0)*((USBD2_HID_NUM>0)|(USBD2_MSC_NUM>0)|(USBD2_ADC_NUM>0)|(USBD2_CDC_NUM>0)))

#define USBD2_ADC_DESC_LEN             (USBD2_ADC_NUM *                                                                 \
                                       (USBD2_USE_IAD * USB_INTERFACE_ASSOC_DESC_SIZE +                                 \
                                        3 * USB_INTERFACE_DESC_SIZE + AUDIO_CONTROL_INTERFACE_DESC_SZ(1) +              \
                                        AUDIO_INPUT_TERMINAL_DESC_SIZE + AUDIO_FEATURE_UNIT_DESC_SZ(1,1) +              \
                                        AUDIO_OUTPUT_TERMINAL_DESC_SIZE + AUDIO_STREAMING_INTERFACE_DESC_SIZE +         \
                                        AUDIO_FORMAT_TYPE_I_DESC_SZ(1) + AUDIO_STANDARD_ENDPOINT_DESC_SIZE +            \
                                        AUDIO_STREAMING_ENDPOINT_DESC_SIZE))
#define USBD2_CDC_DESC_LEN             (USBD2_CDC_NUM *                                                                 \
                                       (USBD2_USE_IAD * USB_INTERFACE_ASSOC_DESC_SIZE +                                 \
                                        USB_INTERFACE_DESC_SIZE + 0x0013 + USB_ENDPOINT_DESC_SIZE +                     \
                                        USB_INTERFACE_DESC_SIZE + 2* USB_ENDPOINT_DESC_SIZE))
#define USBD2_MSC_DESC_LEN             (USBD2_MSC_NUM *                                                                 \
                                       (USB_INTERFACE_DESC_SIZE + 2 * USB_ENDPOINT_DESC_SIZE))
#if (USBD2_HID0 == 1)
#define USBD2_HID0_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD2_HID0_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD2_HID0_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD2_ADC_NUM * USBD2_ADC_DESC_LEN +                                            \
                                        USBD2_CDC_NUM * USBD2_CDC_DESC_LEN)
#define USBD_HID0_DESC_LEN             (USBD2_HID0_DESC_LEN)
#define USBD_HID0_DESC_OFS             (USBD2_HID0_DESC_OFS)
#else
#define USBD2_HID0_DESC_LEN            (0)
#define USBD2_HID0_DESC_OFS            (0)
#endif
#if (USBD2_HID1 == 1)
#define USBD2_HID1_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD2_HID1_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD2_HID1_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD2_ADC_NUM * USBD2_ADC_DESC_LEN +                                            \
                                        USBD2_CDC_NUM * USBD2_CDC_DESC_LEN +                                            \
                                        USBD2_HID0_DESC_OFS)
#define USBD_HID1_DESC_LEN             (USBD2_HID1_DESC_LEN)
#define USBD_HID1_DESC_OFS             (USBD2_HID1_DESC_OFS)
#else
#define USBD2_HID1_DESC_LEN            (0)
#define USBD2_HID1_DESC_OFS            (0)
#endif
#if (USBD2_HID2 == 1)
#define USBD2_HID2_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD2_HID2_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD2_HID2_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD2_ADC_NUM * USBD2_ADC_DESC_LEN +                                            \
                                        USBD2_CDC_NUM * USBD2_CDC_DESC_LEN +                                            \
                                        USBD2_HID0_DESC_OFS +                                                           \
                                        USBD2_HID1_DESC_OFS)
#define USBD_HID2_DESC_LEN             (USBD2_HID2_DESC_LEN)
#define USBD_HID2_DESC_OFS             (USBD2_HID2_DESC_OFS)
#else
#define USBD2_HID2_DESC_LEN            (0)
#define USBD2_HID2_DESC_OFS            (0)
#endif
#if (USBD2_HID3 == 1)
#define USBD2_HID3_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD2_HID3_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD2_HID3_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD2_ADC_NUM * USBD2_ADC_DESC_LEN +                                            \
                                        USBD2_CDC_NUM * USBD2_CDC_DESC_LEN +                                            \
                                        USBD2_HID0_DESC_OFS +                                                           \
                                        USBD2_HID1_DESC_OFS +                                                           \
                                        USBD2_HID2_DESC_OFS)
#define USBD_HID3_DESC_LEN             (USBD2_HID3_DESC_LEN)
#define USBD_HID3_DESC_OFS             (USBD2_HID3_DESC_OFS)
#else
#define USBD2_HID3_DESC_LEN            (0)
#define USBD2_HID3_DESC_OFS            (0)
#endif
#define USBD2_HID_DESC_LEN             (USBD2_HID0_DESC_LEN +                                                           \
                                        USBD2_HID1_DESC_LEN +                                                           \
                                        USBD2_HID2_DESC_LEN +                                                           \
                                        USBD2_HID3_DESC_LEN )                                                           \

#define USBD2_WTOTALLENGTH             (USB_CONFIGURATION_DESC_SIZE +                                                   \
                                        USBD2_ADC_DESC_LEN +                                                            \
                                        USBD2_CDC_DESC_LEN +                                                            \
                                        USBD2_HID_DESC_LEN +                                                            \
                                        USBD2_MSC_DESC_LEN )

#if ((USBD2_HS) || (USBD2_USE_IAD))
#define USBD2_BDC_USB                   0x0200
#else
#define USBD2_BDC_USB                   0x0110
#endif

#endif  /* USBD2_MACROS */

#if     (defined(USBD2_PORT) && !defined(USBD2_PORT_DEF))
#define  USBD2_PORT_DEF                 1
#ifndef  __NO_USB_LIB_C

extern  ARM_DRIVER_USBD  USBDn_DRIVER(USBD2_PORT);

uint8_t         usbd2_alt_setting      [USBD2_IF_NUM];
uint8_t         usbd2_ep0_buf          [USBD2_MAX_PACKET0] __attribute__((aligned(4)));
usbd_data_t     usbd2_data;
usbd_dev_t      usbd2_dev           = { 
                                       &USBDn_DRIVER(USBD2_PORT), 
                                        usbd2_alt_setting, 
                                        usbd2_ep0_buf, 
                                       &usbd2_data, 
                                        USBD2_CFG_DESC_BMATTRIBUTES, 
                                        USBD2_HS, 
                                        USBD2_IF_NUM, 
                                        USBD2_EP_NUM, 
                                        USBD2_MAX_PACKET0, 
                                      };

#endif  /* __NO_USB_LIB_C   */
#endif


/*------------------------------------------------------------------------------
 *      USB Device 3 Macros, Calculations and Constants for Library
 *----------------------------------------------------------------------------*/

#ifndef USBD3_MACROS
#define USBD3_MACROS                    1

#if    (defined(RTE_USB_Device_HID_0) && (USBD_HID0_DEV == 3))
#define USBD3_HID0                      1
#define USBD3_HID0_EP_INT_IN            USBD_HID0_EP_INT_IN
#define USBD3_HID0_EP_INT_OUT           USBD_HID0_EP_INT_OUT
#define USBD3_HID0_IF_NUM               1
#if   ((USBD_HID0_EP_INT_IN ==  1) || (USBD_HID0_EP_INT_OUT ==  1))
#ifdef  USBD3_EP1 
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  2) || (USBD_HID0_EP_INT_OUT ==  2))
#ifdef  USBD3_EP2 
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  3) || (USBD_HID0_EP_INT_OUT ==  3))
#ifdef  USBD3_EP3 
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  4) || (USBD_HID0_EP_INT_OUT ==  4))
#ifdef  USBD3_EP4 
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  5) || (USBD_HID0_EP_INT_OUT ==  5))
#ifdef  USBD3_EP5 
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  6) || (USBD_HID0_EP_INT_OUT ==  6))
#ifdef  USBD3_EP6 
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  7) || (USBD_HID0_EP_INT_OUT ==  7))
#ifdef  USBD3_EP7 
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  8) || (USBD_HID0_EP_INT_OUT ==  8))
#ifdef  USBD3_EP8 
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN ==  9) || (USBD_HID0_EP_INT_OUT ==  9))
#ifdef  USBD3_EP9 
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 10) || (USBD_HID0_EP_INT_OUT == 10))
#ifdef  USBD3_EP10 
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 11) || (USBD_HID0_EP_INT_OUT == 11))
#ifdef  USBD3_EP11 
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 12) || (USBD_HID0_EP_INT_OUT == 12))
#ifdef  USBD3_EP12 
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 13) || (USBD_HID0_EP_INT_OUT == 13))
#ifdef  USBD3_EP13 
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 14) || (USBD_HID0_EP_INT_OUT == 14))
#ifdef  USBD3_EP14 
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_hid_thread_id[0]
#elif ((USBD_HID0_EP_INT_IN == 15) || (USBD_HID0_EP_INT_OUT == 15))
#ifdef  USBD3_EP15 
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_hid_thread_id[0]
#endif 
#else
#define USBD3_HID0                      0
#define USBD3_HID0_EP_INT_IN            0
#define USBD3_HID0_EP_INT_OUT           0
#define USBD3_HID0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_1) && (USBD_HID1_DEV == 3))
#define USBD3_HID1                      1
#define USBD3_HID1_EP_INT_IN            USBD_HID1_EP_INT_IN
#define USBD3_HID1_EP_INT_OUT           USBD_HID1_EP_INT_OUT
#define USBD3_HID1_IF_NUM               1
#if   ((USBD_HID1_EP_INT_IN ==  1) || (USBD_HID1_EP_INT_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  2) || (USBD_HID1_EP_INT_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  3) || (USBD_HID1_EP_INT_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  4) || (USBD_HID1_EP_INT_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  5) || (USBD_HID1_EP_INT_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  6) || (USBD_HID1_EP_INT_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  7) || (USBD_HID1_EP_INT_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  8) || (USBD_HID1_EP_INT_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN ==  9) || (USBD_HID1_EP_INT_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 10) || (USBD_HID1_EP_INT_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 11) || (USBD_HID1_EP_INT_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 12) || (USBD_HID1_EP_INT_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 13) || (USBD_HID1_EP_INT_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 14) || (USBD_HID1_EP_INT_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_hid_thread_id[1]
#elif ((USBD_HID1_EP_INT_IN == 15) || (USBD_HID1_EP_INT_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_hid_thread_id[1]
#endif 
#else
#define USBD3_HID1                      0
#define USBD3_HID1_EP_INT_IN            0
#define USBD3_HID1_EP_INT_OUT           0
#define USBD3_HID1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_2) && (USBD_HID2_DEV == 3))
#define USBD3_HID2                      1
#define USBD3_HID2_EP_INT_IN            USBD_HID2_EP_INT_IN
#define USBD3_HID2_EP_INT_OUT           USBD_HID2_EP_INT_OUT
#define USBD3_HID2_IF_NUM               1
#if   ((USBD_HID2_EP_INT_IN ==  1) || (USBD_HID2_EP_INT_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  2) || (USBD_HID2_EP_INT_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  3) || (USBD_HID2_EP_INT_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  4) || (USBD_HID2_EP_INT_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  5) || (USBD_HID2_EP_INT_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  6) || (USBD_HID2_EP_INT_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  7) || (USBD_HID2_EP_INT_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  8) || (USBD_HID2_EP_INT_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN ==  9) || (USBD_HID2_EP_INT_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 10) || (USBD_HID2_EP_INT_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 11) || (USBD_HID2_EP_INT_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 12) || (USBD_HID2_EP_INT_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 13) || (USBD_HID2_EP_INT_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 14) || (USBD_HID2_EP_INT_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_hid_thread_id[2]
#elif ((USBD_HID2_EP_INT_IN == 15) || (USBD_HID2_EP_INT_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_hid_thread_id[2]
#endif 
#else
#define USBD3_HID2                      0
#define USBD3_HID2_EP_INT_IN            0
#define USBD3_HID2_EP_INT_OUT           0
#define USBD3_HID2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_HID_3) && (USBD_HID3_DEV == 3))
#define USBD3_HID3                      1
#define USBD3_HID3_EP_INT_IN            USBD_HID3_EP_INT_IN
#define USBD3_HID3_EP_INT_OUT           USBD_HID3_EP_INT_OUT
#define USBD3_HID3_IF_NUM               1
#if   ((USBD_HID3_EP_INT_IN ==  1) || (USBD_HID3_EP_INT_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  2) || (USBD_HID3_EP_INT_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  3) || (USBD_HID3_EP_INT_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  4) || (USBD_HID3_EP_INT_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  5) || (USBD_HID3_EP_INT_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  6) || (USBD_HID3_EP_INT_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  7) || (USBD_HID3_EP_INT_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  8) || (USBD_HID3_EP_INT_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN ==  9) || (USBD_HID3_EP_INT_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 10) || (USBD_HID3_EP_INT_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 11) || (USBD_HID3_EP_INT_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 12) || (USBD_HID3_EP_INT_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 13) || (USBD_HID3_EP_INT_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 14) || (USBD_HID3_EP_INT_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_hid_thread_id[3]
#elif ((USBD_HID3_EP_INT_IN == 15) || (USBD_HID3_EP_INT_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_hid_thread_id[3]
#endif 
#else
#define USBD3_HID3                      0
#define USBD3_HID3_EP_INT_IN            0
#define USBD3_HID3_EP_INT_OUT           0
#define USBD3_HID3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_0) && (USBD_MSC0_DEV == 3))
#define USBD3_MSC0                      1
#define USBD3_MSC0_EP_BULK_IN           USBD_MSC0_EP_BULK_IN
#define USBD3_MSC0_EP_BULK_OUT          USBD_MSC0_EP_BULK_OUT
#define USBD3_MSC0_IF_NUM               1
#if   ((USBD_MSC0_EP_BULK_IN ==  1) || (USBD_MSC0_EP_BULK_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  2) || (USBD_MSC0_EP_BULK_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif            
#define USBD3_EP2                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  3) || (USBD_MSC0_EP_BULK_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif           
#define USBD3_EP3                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  4) || (USBD_MSC0_EP_BULK_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  5) || (USBD_MSC0_EP_BULK_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  6) || (USBD_MSC0_EP_BULK_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  7) || (USBD_MSC0_EP_BULK_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  8) || (USBD_MSC0_EP_BULK_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN ==  9) || (USBD_MSC0_EP_BULK_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 10) || (USBD_MSC0_EP_BULK_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 11) || (USBD_MSC0_EP_BULK_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 12) || (USBD_MSC0_EP_BULK_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 13) || (USBD_MSC0_EP_BULK_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 14) || (USBD_MSC0_EP_BULK_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_msc_thread_id[0]
#elif ((USBD_MSC0_EP_BULK_IN == 15) || (USBD_MSC0_EP_BULK_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_msc_thread_id[0]
#endif 
#else
#define USBD3_MSC0                      0
#define USBD3_MSC0_EP_BULK_IN           0
#define USBD3_MSC0_EP_BULK_OUT          0
#define USBD3_MSC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_1) && (USBD_MSC1_DEV == 3))
#define USBD3_MSC1                      1
#define USBD3_MSC1_EP_BULK_IN           USBD_MSC1_EP_BULK_IN
#define USBD3_MSC1_EP_BULK_OUT          USBD_MSC1_EP_BULK_OUT
#define USBD3_MSC1_IF_NUM               1
#if   ((USBD_MSC1_EP_BULK_IN ==  1) || (USBD_MSC1_EP_BULK_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  2) || (USBD_MSC1_EP_BULK_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  3) || (USBD_MSC1_EP_BULK_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  4) || (USBD_MSC1_EP_BULK_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  5) || (USBD_MSC1_EP_BULK_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  6) || (USBD_MSC1_EP_BULK_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  7) || (USBD_MSC1_EP_BULK_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  8) || (USBD_MSC1_EP_BULK_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN ==  9) || (USBD_MSC1_EP_BULK_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 10) || (USBD_MSC1_EP_BULK_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 11) || (USBD_MSC1_EP_BULK_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 12) || (USBD_MSC1_EP_BULK_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 13) || (USBD_MSC1_EP_BULK_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 14) || (USBD_MSC1_EP_BULK_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_msc_thread_id[1]
#elif ((USBD_MSC1_EP_BULK_IN == 15) || (USBD_MSC1_EP_BULK_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_msc_thread_id[1]
#endif 
#else
#define USBD3_MSC1                      0
#define USBD3_MSC1_EP_BULK_IN           0
#define USBD3_MSC1_EP_BULK_OUT          0
#define USBD3_MSC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_2) && (USBD_MSC2_DEV == 3))
#define USBD3_MSC2                      1
#define USBD3_MSC2_EP_BULK_IN           USBD_MSC2_EP_BULK_IN
#define USBD3_MSC2_EP_BULK_OUT          USBD_MSC2_EP_BULK_OUT
#define USBD3_MSC2_IF_NUM               1
#if   ((USBD_MSC2_EP_BULK_IN ==  1) || (USBD_MSC2_EP_BULK_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  2) || (USBD_MSC2_EP_BULK_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  3) || (USBD_MSC2_EP_BULK_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  4) || (USBD_MSC2_EP_BULK_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  5) || (USBD_MSC2_EP_BULK_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  6) || (USBD_MSC2_EP_BULK_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  7) || (USBD_MSC2_EP_BULK_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  8) || (USBD_MSC2_EP_BULK_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN ==  9) || (USBD_MSC2_EP_BULK_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 10) || (USBD_MSC2_EP_BULK_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 11) || (USBD_MSC2_EP_BULK_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 12) || (USBD_MSC2_EP_BULK_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 13) || (USBD_MSC2_EP_BULK_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 14) || (USBD_MSC2_EP_BULK_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_msc_thread_id[2]
#elif ((USBD_MSC2_EP_BULK_IN == 15) || (USBD_MSC2_EP_BULK_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_msc_thread_id[2]
#endif 
#else
#define USBD3_MSC2                      0
#define USBD3_MSC2_EP_BULK_IN           0
#define USBD3_MSC2_EP_BULK_OUT          0
#define USBD3_MSC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_MSC_3) && (USBD_MSC3_DEV == 3))
#define USBD3_MSC3                      1
#define USBD3_MSC3_EP_BULK_IN           USBD_MSC3_EP_BULK_IN
#define USBD3_MSC3_EP_BULK_OUT          USBD_MSC3_EP_BULK_OUT
#define USBD3_MSC3_IF_NUM               1
#if   ((USBD_MSC3_EP_BULK_IN ==  1) || (USBD_MSC3_EP_BULK_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  2) || (USBD_MSC3_EP_BULK_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  3) || (USBD_MSC3_EP_BULK_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  4) || (USBD_MSC3_EP_BULK_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  5) || (USBD_MSC3_EP_BULK_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  6) || (USBD_MSC3_EP_BULK_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  7) || (USBD_MSC3_EP_BULK_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  8) || (USBD_MSC3_EP_BULK_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN ==  9) || (USBD_MSC3_EP_BULK_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 10) || (USBD_MSC3_EP_BULK_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 11) || (USBD_MSC3_EP_BULK_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 12) || (USBD_MSC3_EP_BULK_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 13) || (USBD_MSC3_EP_BULK_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 14) || (USBD_MSC3_EP_BULK_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_msc_thread_id[3]
#elif ((USBD_MSC3_EP_BULK_IN == 15) || (USBD_MSC3_EP_BULK_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_msc_thread_id[3]
#endif 
#else
#define USBD3_MSC3                      0
#define USBD3_MSC3_EP_BULK_IN           0
#define USBD3_MSC3_EP_BULK_OUT          0
#define USBD3_MSC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_0) && (USBD_ADC0_DEV == 3))
#define USBD3_ADC0                      1
#define USBD3_ADC0_EP_ISO_OUT           USBD_ADC0_EP_ISO_OUT
#define USBD3_ADC0_IF_NUM               2
#if   ((USBD_ADC0_EP_ISO_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_adc_thread_id[0]
#elif ((USBD_ADC0_EP_ISO_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_adc_thread_id[0]
#endif 
#else
#define USBD3_ADC0                      0
#define USBD3_ADC0_EP_ISO_OUT           0
#define USBD3_ADC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_1) && (USBD_ADC1_DEV == 3))
#define USBD3_ADC1                      1
#define USBD3_ADC1_EP_ISO_OUT           USBD_ADC1_EP_ISO_OUT
#define USBD3_ADC1_IF_NUM               2
#if   ((USBD_ADC1_EP_ISO_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_adc_thread_id[1]
#elif ((USBD_ADC1_EP_ISO_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_adc_thread_id[1]
#endif 
#else
#define USBD3_ADC1                      0
#define USBD3_ADC1_EP_ISO_OUT           0
#define USBD3_ADC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_2) && (USBD_ADC2_DEV == 3))
#define USBD3_ADC2                      1
#define USBD3_ADC2_EP_ISO_OUT           USBD_ADC2_EP_ISO_OUT
#define USBD3_ADC2_IF_NUM               2
#if   ((USBD_ADC2_EP_ISO_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_adc_thread_id[2]
#elif ((USBD_ADC2_EP_ISO_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_adc_thread_id[2]
#endif 
#else
#define USBD3_ADC2                      0
#define USBD3_ADC2_EP_ISO_OUT           0
#define USBD3_ADC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_ADC_3) && (USBD_ADC3_DEV == 3))
#define USBD3_ADC3                      1
#define USBD3_ADC3_EP_ISO_OUT           USBD_ADC3_EP_ISO_OUT
#define USBD3_ADC3_IF_NUM               2
#if   ((USBD_ADC3_EP_ISO_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_adc_thread_id[3]
#elif ((USBD_ADC3_EP_ISO_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_adc_thread_id[3]
#endif 
#else
#define USBD3_ADC3                      0
#define USBD3_ADC3_EP_ISO_OUT           0
#define USBD3_ADC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_0) && (USBD_CDC0_DEV == 3))
#define USBD3_CDC0                      1
#define USBD3_CDC0_EP_INT_IN            USBD_CDC0_EP_INT_IN
#define USBD3_CDC0_EP_BULK_IN           USBD_CDC0_EP_BULK_IN
#define USBD3_CDC0_EP_BULK_OUT          USBD_CDC0_EP_BULK_OUT
#define USBD3_CDC0_IF_NUM               2
#if   ((USBD_CDC0_EP_INT_IN ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_cdc_thread_id[0]
#elif ((USBD_CDC0_EP_INT_IN == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_cdc_thread_id[0]
#endif 
#if   ((USBD_CDC0_EP_BULK_IN ==  1) || (USBD_CDC0_EP_BULK_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  2) || (USBD_CDC0_EP_BULK_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  3) || (USBD_CDC0_EP_BULK_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  4) || (USBD_CDC0_EP_BULK_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  5) || (USBD_CDC0_EP_BULK_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  6) || (USBD_CDC0_EP_BULK_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  7) || (USBD_CDC0_EP_BULK_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  8) || (USBD_CDC0_EP_BULK_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN ==  9) || (USBD_CDC0_EP_BULK_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 10) || (USBD_CDC0_EP_BULK_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 11) || (USBD_CDC0_EP_BULK_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 12) || (USBD_CDC0_EP_BULK_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 13) || (USBD_CDC0_EP_BULK_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 14) || (USBD_CDC0_EP_BULK_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_cdc_thread_id[1]
#elif ((USBD_CDC0_EP_BULK_IN == 15) || (USBD_CDC0_EP_BULK_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_cdc_thread_id[1]
#endif 
#else
#define USBD3_CDC0                      0
#define USBD3_CDC0_EP_INT_IN            0
#define USBD3_CDC0_EP_BULK_IN           0
#define USBD3_CDC0_EP_BULK_OUT          0
#define USBD3_CDC0_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_1) && (USBD_CDC1_DEV == 3))
#define USBD3_CDC1                      1
#define USBD3_CDC1_EP_INT_IN            USBD_CDC1_EP_INT_IN
#define USBD3_CDC1_EP_BULK_IN           USBD_CDC1_EP_BULK_IN
#define USBD3_CDC1_EP_BULK_OUT          USBD_CDC1_EP_BULK_OUT
#define USBD3_CDC1_IF_NUM               2
#if   ((USBD_CDC1_EP_INT_IN ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_cdc_thread_id[2]
#elif ((USBD_CDC1_EP_INT_IN == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_cdc_thread_id[2]
#endif 
#if   ((USBD_CDC1_EP_BULK_IN ==  1) || (USBD_CDC1_EP_BULK_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  2) || (USBD_CDC1_EP_BULK_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  3) || (USBD_CDC1_EP_BULK_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  4) || (USBD_CDC1_EP_BULK_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  5) || (USBD_CDC1_EP_BULK_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  6) || (USBD_CDC1_EP_BULK_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  7) || (USBD_CDC1_EP_BULK_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  8) || (USBD_CDC1_EP_BULK_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN ==  9) || (USBD_CDC1_EP_BULK_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 10) || (USBD_CDC1_EP_BULK_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 11) || (USBD_CDC1_EP_BULK_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 12) || (USBD_CDC1_EP_BULK_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 13) || (USBD_CDC1_EP_BULK_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 14) || (USBD_CDC1_EP_BULK_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_cdc_thread_id[3]
#elif ((USBD_CDC1_EP_BULK_IN == 15) || (USBD_CDC1_EP_BULK_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_cdc_thread_id[3]
#endif 
#else
#define USBD3_CDC1                      0
#define USBD3_CDC1_EP_INT_IN            0
#define USBD3_CDC1_EP_BULK_IN           0
#define USBD3_CDC1_EP_BULK_OUT          0
#define USBD3_CDC1_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_2) && (USBD_CDC2_DEV == 3))
#define USBD3_CDC2                      1
#define USBD3_CDC2_EP_INT_IN            USBD_CDC2_EP_INT_IN
#define USBD3_CDC2_EP_BULK_IN           USBD_CDC2_EP_BULK_IN
#define USBD3_CDC2_EP_BULK_OUT          USBD_CDC2_EP_BULK_OUT
#define USBD3_CDC2_IF_NUM               2
#if   ((USBD_CDC2_EP_INT_IN ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_cdc_thread_id[4]
#elif ((USBD_CDC2_EP_INT_IN == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_cdc_thread_id[4]
#endif 
#if   ((USBD_CDC2_EP_BULK_IN ==  1) || (USBD_CDC2_EP_BULK_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  2) || (USBD_CDC2_EP_BULK_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  3) || (USBD_CDC2_EP_BULK_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  4) || (USBD_CDC2_EP_BULK_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  5) || (USBD_CDC2_EP_BULK_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  6) || (USBD_CDC2_EP_BULK_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  7) || (USBD_CDC2_EP_BULK_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  8) || (USBD_CDC2_EP_BULK_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN ==  9) || (USBD_CDC2_EP_BULK_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 10) || (USBD_CDC2_EP_BULK_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 11) || (USBD_CDC2_EP_BULK_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 12) || (USBD_CDC2_EP_BULK_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 13) || (USBD_CDC2_EP_BULK_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 14) || (USBD_CDC2_EP_BULK_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_cdc_thread_id[5]
#elif ((USBD_CDC2_EP_BULK_IN == 15) || (USBD_CDC2_EP_BULK_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_cdc_thread_id[5]
#endif 
#else
#define USBD3_CDC2                      0
#define USBD3_CDC2_EP_INT_IN            0
#define USBD3_CDC2_EP_BULK_IN           0
#define USBD3_CDC2_EP_BULK_OUT          0
#define USBD3_CDC2_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CDC_3) && (USBD_CDC3_DEV == 3))
#define USBD3_CDC3                      1
#define USBD3_CDC3_EP_INT_IN            USBD_CDC3_EP_INT_IN
#define USBD3_CDC3_EP_BULK_IN           USBD_CDC3_EP_BULK_IN
#define USBD3_CDC3_EP_BULK_OUT          USBD_CDC3_EP_BULK_OUT
#define USBD3_CDC3_IF_NUM               2
#if   ((USBD_CDC3_EP_INT_IN ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_cdc_thread_id[6]
#elif ((USBD_CDC3_EP_INT_IN == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_cdc_thread_id[6]
#endif 
#if   ((USBD_CDC3_EP_BULK_IN ==  1) || (USBD_CDC3_EP_BULK_OUT ==  1))
#ifdef  USBD3_EP1
#error  Endpoint 1 on Device 3 is already used!
#endif
#define USBD3_EP1                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  2) || (USBD_CDC3_EP_BULK_OUT ==  2))
#ifdef  USBD3_EP2
#error  Endpoint 2 on Device 3 is already used!
#endif
#define USBD3_EP2                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  3) || (USBD_CDC3_EP_BULK_OUT ==  3))
#ifdef  USBD3_EP3
#error  Endpoint 3 on Device 3 is already used!
#endif
#define USBD3_EP3                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  4) || (USBD_CDC3_EP_BULK_OUT ==  4))
#ifdef  USBD3_EP4
#error  Endpoint 4 on Device 3 is already used!
#endif
#define USBD3_EP4                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  5) || (USBD_CDC3_EP_BULK_OUT ==  5))
#ifdef  USBD3_EP5
#error  Endpoint 5 on Device 3 is already used!
#endif
#define USBD3_EP5                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  6) || (USBD_CDC3_EP_BULK_OUT ==  6))
#ifdef  USBD3_EP6
#error  Endpoint 6 on Device 3 is already used!
#endif
#define USBD3_EP6                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  7) || (USBD_CDC3_EP_BULK_OUT ==  7))
#ifdef  USBD3_EP7
#error  Endpoint 7 on Device 3 is already used!
#endif
#define USBD3_EP7                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  8) || (USBD_CDC3_EP_BULK_OUT ==  8))
#ifdef  USBD3_EP8
#error  Endpoint 8 on Device 3 is already used!
#endif
#define USBD3_EP8                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN ==  9) || (USBD_CDC3_EP_BULK_OUT ==  9))
#ifdef  USBD3_EP9
#error  Endpoint 9 on Device 3 is already used!
#endif
#define USBD3_EP9                      &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 10) || (USBD_CDC3_EP_BULK_OUT == 10))
#ifdef  USBD3_EP10
#error  Endpoint 10 on Device 3 is already used!
#endif
#define USBD3_EP10                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 11) || (USBD_CDC3_EP_BULK_OUT == 11))
#ifdef  USBD3_EP11
#error  Endpoint 11 on Device 3 is already used!
#endif
#define USBD3_EP11                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 12) || (USBD_CDC3_EP_BULK_OUT == 12))
#ifdef  USBD3_EP12
#error  Endpoint 12 on Device 3 is already used!
#endif
#define USBD3_EP12                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 13) || (USBD_CDC3_EP_BULK_OUT == 13))
#ifdef  USBD3_EP13
#error  Endpoint 13 on Device 3 is already used!
#endif
#define USBD3_EP13                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 14) || (USBD_CDC3_EP_BULK_OUT == 14))
#ifdef  USBD3_EP14
#error  Endpoint 14 on Device 3 is already used!
#endif
#define USBD3_EP14                     &usbd_cdc_thread_id[7]
#elif ((USBD_CDC3_EP_BULK_IN == 15) || (USBD_CDC3_EP_BULK_OUT == 15))
#ifdef  USBD3_EP15
#error  Endpoint 15 on Device 3 is already used!
#endif
#define USBD3_EP15                     &usbd_cdc_thread_id[7]
#endif 
#else
#define USBD3_CDC3                      0
#define USBD3_CDC3_EP_INT_IN            0
#define USBD3_CDC3_EP_BULK_IN           0
#define USBD3_CDC3_EP_BULK_OUT          0
#define USBD3_CDC3_IF_NUM               0
#endif

#if    (defined(RTE_USB_Device_CLS_0) && (USBD_CLS0_DEV == 3))
#define USBD3_CLS0                      1
#else
#define USBD3_CLS0                      0
#endif

#if    (defined(RTE_USB_Device_CLS_1) && (USBD_CLS1_DEV == 3))
#define USBD3_CLS1                      1
#else
#define USBD3_CLS1                      0
#endif

#if    (defined(RTE_USB_Device_CLS_2) && (USBD_CLS2_DEV == 3))
#define USBD3_CLS2                      1
#else
#define USBD3_CLS2                      0
#endif

#if    (defined(RTE_USB_Device_CLS_3) && (USBD_CLS3_DEV == 3))
#define USBD3_CLS3                      1
#else
#define USBD3_CLS3                      0
#endif

#define USBD3_HID_NUM                  (USBDn_c_NUM(3,HID))
#define USBD3_MSC_NUM                  (USBDn_c_NUM(3,MSC))
#define USBD3_ADC_NUM                  (USBDn_c_NUM(3,ADC))
#define USBD3_CDC_NUM                  (USBDn_c_NUM(3,CDC))
#define USBD3_CLS_NUM                  (USBDn_c_NUM(3,CLS))
#define USBD3_ANY                     ((USBD3_HID_NUM + USBD3_MSC_NUM + USBD3_ADC_NUM + USBD3_CDC_NUM + USBD3_CLS_NUM) != 0)

#define USBD3_HID_EP_CALC    (MAX_OF_8 (USBD3_HID0_EP_INT_IN, USBD3_HID0_EP_INT_OUT,  \
                                        USBD3_HID1_EP_INT_IN, USBD3_HID1_EP_INT_OUT,  \
                                        USBD3_HID2_EP_INT_IN, USBD3_HID2_EP_INT_OUT,  \
                                        USBD3_HID3_EP_INT_IN, USBD3_HID3_EP_INT_OUT))
#define USBD3_MSC_EP_CALC    (MAX_OF_8 (USBD3_MSC0_EP_BULK_IN, USBD3_MSC0_EP_BULK_OUT,  \
                                        USBD3_MSC1_EP_BULK_IN, USBD3_MSC1_EP_BULK_OUT,  \
                                        USBD3_MSC2_EP_BULK_IN, USBD3_MSC2_EP_BULK_OUT,  \
                                        USBD3_MSC3_EP_BULK_IN, USBD3_MSC3_EP_BULK_OUT))
#define USBD3_ADC_EP_CALC    (MAX_OF_4 (USBD3_ADC0_EP_ISO_OUT,  \
                                        USBD3_ADC1_EP_ISO_OUT,  \
                                        USBD3_ADC2_EP_ISO_OUT,  \
                                        USBD3_ADC3_EP_ISO_OUT))
#define USBD3_CDC_EP_CALC    (MAX_OF_12(USBD3_CDC0_EP_INT_IN, USBD3_CDC0_EP_BULK_IN, USBD3_CDC0_EP_BULK_OUT, \
                                        USBD3_CDC1_EP_INT_IN, USBD3_CDC1_EP_BULK_IN, USBD3_CDC1_EP_BULK_OUT, \
                                        USBD3_CDC2_EP_INT_IN, USBD3_CDC2_EP_BULK_IN, USBD3_CDC2_EP_BULK_OUT, \
                                        USBD3_CDC3_EP_INT_IN, USBD3_CDC3_EP_BULK_IN, USBD3_CDC3_EP_BULK_OUT))

#define USBD3_IF_NUM                   (USBD3_HID_NUM+USBD3_MSC_NUM+(USBD3_ADC_NUM*2)+(USBD3_CDC_NUM*2)+USBD3_CLS_NUM)
#define USBD3_EP_NUM         (MAX_OF_4 (USBD3_HID_EP_CALC,USBD3_MSC_EP_CALC,USBD3_ADC_EP_CALC,USBD3_CDC_EP_CALC))

#if    (USBD3_ADC0_IF_NUM != 0)
#define USBD_ADC0_CIF                (0)
#define USBD_ADC0_SIF                (1+USBD_ADC0_CIF)
#define USBD_ADC0_CIF_STR            (0+USBD3_STR_DESC_SER_EN+3)
#define USBD_ADC0_SIF1_STR           (1+USBD_ADC0_CIF_STR)
#define USBD_ADC0_SIF2_STR           (2+USBD_ADC0_CIF_STR)
#endif
#if    (USBD3_ADC1_IF_NUM != 0)
#define USBD_ADC1_CIF                  (USBD3_ADC0_IF_NUM                                                                       )
#define USBD_ADC1_SIF                (1+USBD_ADC1_CIF)
#define USBD_ADC1_CIF_STR             ((USBD3_ADC0_IF_NUM/2                                                                     )*3+\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC1_SIF1_STR           (1+USBD_ADC1_CIF_STR)
#define USBD_ADC1_SIF2_STR           (2+USBD_ADC1_CIF_STR)
#endif
#if    (USBD3_ADC2_IF_NUM != 0)
#define USBD_ADC2_CIF                  (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM                                                 )
#define USBD_ADC2_SIF                (1+USBD_ADC2_CIF)
#define USBD_ADC2_CIF_STR             ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2                                               )*3+\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC2_SIF1_STR           (1+USBD_ADC2_CIF_STR)
#define USBD_ADC2_SIF2_STR           (2+USBD_ADC2_CIF_STR)
#endif
#if    (USBD3_ADC3_IF_NUM != 0)
#define USBD_ADC3_CIF                  (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM                           )
#define USBD_ADC3_SIF                (1+USBD_ADC3_CIF)
#define USBD_ADC3_CIF_STR             ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2                         )*3+\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#define USBD_ADC3_SIF1_STR           (1+USBD_ADC3_CIF_STR)
#define USBD_ADC3_SIF2_STR           (2+USBD_ADC3_CIF_STR)
#endif

#if    (USBD3_CDC0_IF_NUM != 0)
#define USBD_CDC0_CIF                  (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM     )
#define USBD_CDC0_DIF                (1+USBD_CDC0_CIF)
#define USBD_CDC0_CIF_STR             ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC0_DIF_STR            (1+USBD_CDC0_CIF_STR)
#endif
#if    (USBD3_CDC1_IF_NUM != 0)
#define USBD_CDC1_CIF                  (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM                                                                       )
#define USBD_CDC1_DIF                (1+USBD_CDC1_CIF)
#define USBD_CDC1_CIF_STR             ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM                                                                        +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC1_DIF_STR            (1+USBD_CDC1_CIF_STR)
#endif
#if    (USBD3_CDC2_IF_NUM != 0)
#define USBD_CDC2_CIF                  (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM                                                 )
#define USBD_CDC2_DIF                (1+USBD_CDC2_CIF)
#define USBD_CDC2_CIF_STR             ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM                                                  +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC2_DIF_STR            (1+USBD_CDC2_CIF_STR)
#endif
#if    (USBD3_CDC3_IF_NUM != 0)
#define USBD_CDC3_CIF                  (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM                           )
#define USBD_CDC3_DIF                (1+USBD_CDC3_CIF)
#define USBD_CDC3_CIF_STR             ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM                            +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#define USBD_CDC3_DIF_STR            (1+USBD_CDC3_CIF_STR)
#endif

#if    (USBD3_HID0_IF_NUM != 0)
#define USBD_HID0_IF                   (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM     )
#define USBD_HID0_STR                 ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD3_HID1_IF_NUM != 0)
#define USBD_HID1_IF                   (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM                                                                       )
#define USBD_HID1_STR                 ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM                                                                        +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD3_HID2_IF_NUM != 0)
#define USBD_HID2_IF                   (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM                                                 )
#define USBD_HID2_STR                 ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM                                                  +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD3_HID3_IF_NUM != 0)
#define USBD_HID3_IF                   (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM                           )
#define USBD_HID3_STR                 ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM                            +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#endif

#if    (USBD3_MSC0_IF_NUM != 0)
#define USBD_MSC0_IF                   (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM    +USBD3_HID3_IF_NUM     )
#define USBD_MSC0_STR                 ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM    +USBD3_HID2_IF_NUM      +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD3_MSC1_IF_NUM != 0)
#define USBD_MSC1_IF                   (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM    +USBD3_HID3_IF_NUM      +\
                                        USBD3_MSC0_IF_NUM                                                                       )
#define USBD_MSC1_STR                 ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM    +USBD3_HID2_IF_NUM      +\
                                        USBD3_MSC0_IF_NUM                                                                        +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD3_MSC2_IF_NUM != 0)
#define USBD_MSC2_IF                   (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM    +USBD3_HID3_IF_NUM      +\
                                        USBD3_MSC0_IF_NUM    +USBD3_MSC1_IF_NUM                                                 )
#define USBD_MSC2_STR                 ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM    +USBD3_HID2_IF_NUM      +\
                                        USBD3_MSC0_IF_NUM    +USBD3_MSC1_IF_NUM                                                  +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#endif
#if    (USBD3_MSC3_IF_NUM != 0)
#define USBD_MSC3_IF                   (USBD3_ADC0_IF_NUM    +USBD3_ADC1_IF_NUM    +USBD3_ADC2_IF_NUM    +USBD3_ADC3_IF_NUM      +\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM    +USBD3_HID3_IF_NUM      +\
                                        USBD3_MSC0_IF_NUM    +USBD3_MSC1_IF_NUM    +USBD3_MSC2_IF_NUM                           )
#define USBD_MSC3_STR                 ((USBD3_ADC0_IF_NUM/2  +USBD3_ADC1_IF_NUM/2  +USBD3_ADC2_IF_NUM/2  +USBD3_ADC3_IF_NUM/2   )*3+\
                                        USBD3_CDC0_IF_NUM    +USBD3_CDC1_IF_NUM    +USBD3_CDC2_IF_NUM    +USBD3_CDC3_IF_NUM      +\
                                        USBD3_HID0_IF_NUM    +USBD3_HID1_IF_NUM    +USBD3_HID2_IF_NUM    +USBD3_HID2_IF_NUM      +\
                                        USBD3_MSC0_IF_NUM    +USBD3_MSC1_IF_NUM    +USBD3_MSC2_IF_NUM                            +\
                                        USBD3_STR_DESC_SER_EN+3                                                                 )
#endif

#define USBD3_USE_IAD                 ((USBD3_CDC_NUM>0)*((USBD3_HID_NUM>0)|(USBD3_MSC_NUM>0)|(USBD3_ADC_NUM>0)|(USBD3_CDC_NUM>0)))

#define USBD3_ADC_DESC_LEN             (USBD3_ADC_NUM *                                                                 \
                                       (USBD3_USE_IAD * USB_INTERFACE_ASSOC_DESC_SIZE +                                 \
                                        3 * USB_INTERFACE_DESC_SIZE + AUDIO_CONTROL_INTERFACE_DESC_SZ(1) +              \
                                        AUDIO_INPUT_TERMINAL_DESC_SIZE + AUDIO_FEATURE_UNIT_DESC_SZ(1,1) +              \
                                        AUDIO_OUTPUT_TERMINAL_DESC_SIZE + AUDIO_STREAMING_INTERFACE_DESC_SIZE +         \
                                        AUDIO_FORMAT_TYPE_I_DESC_SZ(1) + AUDIO_STANDARD_ENDPOINT_DESC_SIZE +            \
                                        AUDIO_STREAMING_ENDPOINT_DESC_SIZE))
#define USBD3_CDC_DESC_LEN             (USBD3_CDC_NUM *                                                                 \
                                       (USBD3_USE_IAD * USB_INTERFACE_ASSOC_DESC_SIZE +                                 \
                                        USB_INTERFACE_DESC_SIZE + 0x0013 + USB_ENDPOINT_DESC_SIZE +                     \
                                        USB_INTERFACE_DESC_SIZE + 2* USB_ENDPOINT_DESC_SIZE))
#define USBD3_MSC_DESC_LEN             (USBD3_MSC_NUM *                                                                 \
                                       (USB_INTERFACE_DESC_SIZE + 2 * USB_ENDPOINT_DESC_SIZE))
#if (USBD3_HID0 == 1)
#define USBD3_HID0_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD3_HID0_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD3_HID0_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD3_ADC_NUM * USBD3_ADC_DESC_LEN +                                            \
                                        USBD3_CDC_NUM * USBD3_CDC_DESC_LEN)
#define USBD_HID0_DESC_LEN             (USBD3_HID0_DESC_LEN)
#define USBD_HID0_DESC_OFS             (USBD3_HID0_DESC_OFS)
#else
#define USBD3_HID0_DESC_LEN            (0)
#define USBD3_HID0_DESC_OFS            (0)
#endif
#if (USBD3_HID1 == 1)
#define USBD3_HID1_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD3_HID1_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD3_HID1_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD3_ADC_NUM * USBD3_ADC_DESC_LEN +                                            \
                                        USBD3_CDC_NUM * USBD3_CDC_DESC_LEN +                                            \
                                        USBD3_HID0_DESC_OFS)
#define USBD_HID1_DESC_LEN             (USBD3_HID1_DESC_LEN)
#define USBD_HID1_DESC_OFS             (USBD3_HID1_DESC_OFS)
#else
#define USBD3_HID1_DESC_LEN            (0)
#define USBD3_HID1_DESC_OFS            (0)
#endif
#if (USBD3_HID2 == 1)
#define USBD3_HID2_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD3_HID2_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD3_HID2_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD3_ADC_NUM * USBD3_ADC_DESC_LEN +                                            \
                                        USBD3_CDC_NUM * USBD3_CDC_DESC_LEN +                                            \
                                        USBD3_HID0_DESC_OFS +                                                           \
                                        USBD3_HID1_DESC_OFS)
#define USBD_HID2_DESC_LEN             (USBD3_HID2_DESC_LEN)
#define USBD_HID2_DESC_OFS             (USBD3_HID2_DESC_OFS)
#else
#define USBD3_HID2_DESC_LEN            (0)
#define USBD3_HID2_DESC_OFS            (0)
#endif
#if (USBD3_HID3 == 1)
#define USBD3_HID3_DESC_LEN            (USB_INTERFACE_DESC_SIZE + USB_HID_DESC_SIZE +                                   \
                                       (1 + (USBD3_HID3_EP_INT_OUT != 0)) *                                             \
                                        USB_ENDPOINT_DESC_SIZE)
#define USBD3_HID3_DESC_OFS            (USB_CONFIGURATION_DESC_SIZE + USB_INTERFACE_DESC_SIZE +                         \
                                        USBD3_ADC_NUM * USBD3_ADC_DESC_LEN +                                            \
                                        USBD3_CDC_NUM * USBD3_CDC_DESC_LEN +                                            \
                                        USBD3_HID0_DESC_OFS +                                                           \
                                        USBD3_HID1_DESC_OFS +                                                           \
                                        USBD3_HID2_DESC_OFS)
#define USBD_HID3_DESC_LEN             (USBD3_HID3_DESC_LEN)
#define USBD_HID3_DESC_OFS             (USBD3_HID3_DESC_OFS)
#else
#define USBD3_HID3_DESC_LEN            (0)
#define USBD3_HID3_DESC_OFS            (0)
#endif
#define USBD3_HID_DESC_LEN             (USBD3_HID0_DESC_LEN +                                                           \
                                        USBD3_HID1_DESC_LEN +                                                           \
                                        USBD3_HID2_DESC_LEN +                                                           \
                                        USBD3_HID3_DESC_LEN )                                                           \

#define USBD3_WTOTALLENGTH             (USB_CONFIGURATION_DESC_SIZE +                                                   \
                                        USBD3_ADC_DESC_LEN +                                                            \
                                        USBD3_CDC_DESC_LEN +                                                            \
                                        USBD3_HID_DESC_LEN +                                                            \
                                        USBD3_MSC_DESC_LEN )

#if ((USBD3_HS) || (USBD3_USE_IAD))
#define USBD3_BDC_USB                   0x0200
#else
#define USBD3_BDC_USB                   0x0110
#endif

#endif  /* USBD3_MACROS */

#if     (defined(USBD3_PORT) && !defined(USBD3_PORT_DEF))
#define  USBD3_PORT_DEF                 1
#ifndef  __NO_USB_LIB_C

extern  ARM_DRIVER_USBD  USBDn_DRIVER(USBD3_PORT);

uint8_t         usbd3_alt_setting      [USBD3_IF_NUM];
uint8_t         usbd3_ep0_buf          [USBD3_MAX_PACKET0] __attribute__((aligned(4)));
usbd_data_t     usbd3_data;
usbd_dev_t      usbd3_dev           = { 
                                       &USBDn_DRIVER(USBD3_PORT), 
                                        usbd3_alt_setting, 
                                        usbd3_ep0_buf, 
                                       &usbd3_data, 
                                        USBD3_CFG_DESC_BMATTRIBUTES, 
                                        USBD3_HS, 
                                        USBD3_IF_NUM, 
                                        USBD3_EP_NUM, 
                                        USBD3_MAX_PACKET0, 
                                      };

#endif  /* __NO_USB_LIB_C   */
#endif

                                      
/*------------------------------------------------------------------------------
 *      USB Device Global Macros
 *----------------------------------------------------------------------------*/

#ifndef __NO_USB_LIB_C

#if    (defined(USBD3_PORT))
#define LAST_DEVICE_COMPILE             1
#elif  (defined(USBD2_PORT) && !defined(RTE_USB_Device_3))
#define LAST_DEVICE_COMPILE             1
#elif  (defined(USBD1_PORT) && !defined(RTE_USB_Device_3) && !defined(RTE_USB_Device_2))
#define LAST_DEVICE_COMPILE             1
#elif  (defined(USBD0_PORT) && !defined(RTE_USB_Device_3) && !defined(RTE_USB_Device_2) && !defined(RTE_USB_Device_1))
#define LAST_DEVICE_COMPILE             1
#endif

#if    (defined(LAST_DEVICE_COMPILE))

#define USBD_EP_LAST_NUM     (MAX_OF_4 (USBD0_EP_NUM, USBD1_EP_NUM, USBD2_EP_NUM, USBD3_EP_NUM))

#if     defined(RTE_USB_Device_0)
#define USBD0_DEV_NUM                   1
#else
#define USBD0_DEV_NUM                   0
#endif
#if     defined(RTE_USB_Device_1)
#define USBD1_DEV_NUM                   1
#else
#define USBD1_DEV_NUM                   0
#endif
#if     defined(RTE_USB_Device_2)
#define USBD2_DEV_NUM                   1
#else
#define USBD2_DEV_NUM                   0
#endif
#if     defined(RTE_USB_Device_3)
#define USBD3_DEV_NUM                   1
#else
#define USBD3_DEV_NUM                   0
#endif

#define USBD_DEV_NUM                   (USBD0_DEV_NUM + USBD1_DEV_NUM + USBD2_DEV_NUM + USBD3_DEV_NUM)

#ifndef USBD0_EP1
#define USBD0_EP1       0
#endif
#ifndef USBD0_EP2
#define USBD0_EP2       0
#endif
#ifndef USBD0_EP3
#define USBD0_EP3       0
#endif
#ifndef USBD0_EP4
#define USBD0_EP4       0
#endif
#ifndef USBD0_EP5
#define USBD0_EP5       0
#endif
#ifndef USBD0_EP6
#define USBD0_EP6       0
#endif
#ifndef USBD0_EP7
#define USBD0_EP7       0
#endif
#ifndef USBD0_EP8
#define USBD0_EP8       0
#endif
#ifndef USBD0_EP9
#define USBD0_EP9       0
#endif
#ifndef USBD0_EP10
#define USBD0_EP10      0
#endif
#ifndef USBD0_EP11
#define USBD0_EP11      0
#endif
#ifndef USBD0_EP12
#define USBD0_EP12      0
#endif
#ifndef USBD0_EP13
#define USBD0_EP13      0
#endif
#ifndef USBD0_EP14
#define USBD0_EP14      0
#endif
#ifndef USBD0_EP15
#define USBD0_EP15      0
#endif

#ifndef USBD1_EP1
#define USBD1_EP1       0
#endif
#ifndef USBD1_EP2
#define USBD1_EP2       0
#endif
#ifndef USBD1_EP3
#define USBD1_EP3       0
#endif
#ifndef USBD1_EP4
#define USBD1_EP4       0
#endif
#ifndef USBD1_EP5
#define USBD1_EP5       0
#endif
#ifndef USBD1_EP6
#define USBD1_EP6       0
#endif
#ifndef USBD1_EP7
#define USBD1_EP7       0
#endif
#ifndef USBD1_EP8
#define USBD1_EP8       0
#endif
#ifndef USBD1_EP9
#define USBD1_EP9       0
#endif
#ifndef USBD1_EP10
#define USBD1_EP10      0
#endif
#ifndef USBD1_EP11
#define USBD1_EP11      0
#endif
#ifndef USBD1_EP12
#define USBD1_EP12      0
#endif
#ifndef USBD1_EP13
#define USBD1_EP13      0
#endif
#ifndef USBD1_EP14
#define USBD1_EP14      0
#endif
#ifndef USBD1_EP15
#define USBD1_EP15      0
#endif

#ifndef USBD2_EP1
#define USBD2_EP1       0
#endif
#ifndef USBD2_EP2
#define USBD2_EP2       0
#endif
#ifndef USBD2_EP3
#define USBD2_EP3       0
#endif
#ifndef USBD2_EP4
#define USBD2_EP4       0
#endif
#ifndef USBD2_EP5
#define USBD2_EP5       0
#endif
#ifndef USBD2_EP6
#define USBD2_EP6       0
#endif
#ifndef USBD2_EP7
#define USBD2_EP7       0
#endif
#ifndef USBD2_EP8
#define USBD2_EP8       0
#endif
#ifndef USBD2_EP9
#define USBD2_EP9       0
#endif
#ifndef USBD2_EP10
#define USBD2_EP10      0
#endif
#ifndef USBD2_EP11
#define USBD2_EP11      0
#endif
#ifndef USBD2_EP12
#define USBD2_EP12      0
#endif
#ifndef USBD2_EP13
#define USBD2_EP13      0
#endif
#ifndef USBD2_EP14
#define USBD2_EP14      0
#endif
#ifndef USBD2_EP15
#define USBD2_EP15      0
#endif

#ifndef USBD3_EP1
#define USBD3_EP1       0
#endif
#ifndef USBD3_EP2
#define USBD3_EP2       0
#endif
#ifndef USBD3_EP3
#define USBD3_EP3       0
#endif
#ifndef USBD3_EP4
#define USBD3_EP4       0
#endif
#ifndef USBD3_EP5
#define USBD3_EP5       0
#endif
#ifndef USBD3_EP6
#define USBD3_EP6       0
#endif
#ifndef USBD3_EP7
#define USBD3_EP7       0
#endif
#ifndef USBD3_EP8
#define USBD3_EP8       0
#endif
#ifndef USBD3_EP9
#define USBD3_EP9       0
#endif
#ifndef USBD3_EP10
#define USBD3_EP10      0
#endif
#ifndef USBD3_EP11
#define USBD3_EP11      0
#endif
#ifndef USBD3_EP12
#define USBD3_EP12      0
#endif
#ifndef USBD3_EP13
#define USBD3_EP13      0
#endif
#ifndef USBD3_EP14
#define USBD3_EP14      0
#endif
#ifndef USBD3_EP15
#define USBD3_EP15      0
#endif


#if    (USBD_EP_LAST_NUM >= 14)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6  , USBD##n##_EP7 , USBD##n##_EP8 ,  \
                                                 USBD##n##_EP9 , USBD##n##_EP10 , USBD##n##_EP11 , USBD##n##_EP12 , USBD##n##_EP13 , USBD##n##_EP14
#elif  (USBD_EP_LAST_NUM >= 13)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6  , USBD##n##_EP7 , USBD##n##_EP8 ,  \
                                                 USBD##n##_EP9 , USBD##n##_EP10 , USBD##n##_EP11 , USBD##n##_EP12 , USBD##n##_EP13
#elif  (USBD_EP_LAST_NUM >= 12)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6  , USBD##n##_EP7 , USBD##n##_EP8 ,  \
                                                 USBD##n##_EP9 , USBD##n##_EP10 , USBD##n##_EP11 , USBD##n##_EP12
#elif  (USBD_EP_LAST_NUM >= 11)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6  , USBD##n##_EP7 , USBD##n##_EP8 ,  \
                                                 USBD##n##_EP9 , USBD##n##_EP10 , USBD##n##_EP11
#elif  (USBD_EP_LAST_NUM >= 10)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6  , USBD##n##_EP7 , USBD##n##_EP8 ,  \
                                                 USBD##n##_EP9 , USBD##n##_EP10
#elif  (USBD_EP_LAST_NUM >=  9)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6  , USBD##n##_EP7 , USBD##n##_EP8 ,  \
                                                 USBD##n##_EP9
#elif  (USBD_EP_LAST_NUM >=  8)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6  , USBD##n##_EP7 , USBD##n##_EP8
#elif  (USBD_EP_LAST_NUM >=  7)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6  , USBD##n##_EP7
#elif  (USBD_EP_LAST_NUM >=  6)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5  , USBD##n##_EP6
#elif  (USBD_EP_LAST_NUM >=  5)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4  , USBD##n##_EP5
#elif  (USBD_EP_LAST_NUM >=  4)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3  , USBD##n##_EP4
#elif  (USBD_EP_LAST_NUM >=  3)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2  , USBD##n##_EP3
#elif  (USBD_EP_LAST_NUM >=  2)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1 , USBD##n##_EP2
#elif  (USBD_EP_LAST_NUM >=  1)
#define EP_POINTERS(n)  &usbd_core_thread_id[n], USBD##n##_EP1
#elif  (USBD_EP_LAST_NUM >=  0)
#define EP_POINTERS(n)  &usbd_core_thread_id[n]
#else
#define EP_POINTERS(n)  
#endif
#endif  /* LAST_DEVICE_COMPILE */


/*------------------------------------------------------------------------------
 *      USB Device 0..3 Configurations
 *----------------------------------------------------------------------------*/

#if    (defined(LAST_DEVICE_COMPILE) && (USBD_DEV_NUM))

osThreadId usbd_core_thread_id          [USBD_DEV_NUM];

osMutexId  usbd_core_mutex_read_start_id[USBD_DEV_NUM];
osMutexId  usbd_core_mutex_read_id      [USBD_DEV_NUM];
osMutexId  usbd_core_mutex_write_id     [USBD_DEV_NUM];

#if     defined(RTE_USB_Device_0)
osMutexDef (usbd0_core_mutex_read_start);
osMutexDef (usbd0_core_mutex_read);
osMutexDef (usbd0_core_mutex_write);
#endif
#if     defined(RTE_USB_Device_1)
osMutexDef (usbd1_core_mutex_read_start);
osMutexDef (usbd1_core_mutex_read);
osMutexDef (usbd1_core_mutex_write);
#endif
#if     defined(RTE_USB_Device_2)
osMutexDef (usbd2_core_mutex_read_start);
osMutexDef (usbd2_core_mutex_read);
osMutexDef (usbd2_core_mutex_write);
#endif
#if     defined(RTE_USB_Device_3)
osMutexDef (usbd3_core_mutex_read_start);
osMutexDef (usbd3_core_mutex_read);
osMutexDef (usbd3_core_mutex_write);
#endif

const osMutexDef_t * const usbd_core_mutex_read_start_def_ptr[USBD_DEV_NUM] = {
#if     defined(RTE_USB_Device_0)
        osMutex(usbd0_core_mutex_read_start)
#endif
#if     defined(RTE_USB_Device_1)
      , osMutex(usbd1_core_mutex_read_start)
#endif
#if     defined(RTE_USB_Device_2)
      , osMutex(usbd2_core_mutex_read_start)
#endif
#if     defined(RTE_USB_Device_3)
      , osMutex(usbd3_core_mutex_read_start)
#endif
};

const osMutexDef_t * const usbd_core_mutex_read_def_ptr[USBD_DEV_NUM] = {
#if     defined(RTE_USB_Device_0)
        osMutex(usbd0_core_mutex_read)
#endif
#if     defined(RTE_USB_Device_1)
      , osMutex(usbd1_core_mutex_read)
#endif
#if     defined(RTE_USB_Device_2)
      , osMutex(usbd2_core_mutex_read)
#endif
#if     defined(RTE_USB_Device_3)
      , osMutex(usbd3_core_mutex_read)
#endif
};

const osMutexDef_t * const usbd_core_mutex_write_def_ptr[USBD_DEV_NUM] = {
#if     defined(RTE_USB_Device_0)
        osMutex(usbd0_core_mutex_write)
#endif
#if     defined(RTE_USB_Device_1)
      , osMutex(usbd1_core_mutex_write)
#endif
#if     defined(RTE_USB_Device_2)
      , osMutex(usbd2_core_mutex_write)
#endif
#if     defined(RTE_USB_Device_3)
      , osMutex(usbd3_core_mutex_write)
#endif
};

#if     defined(RTE_USB_Device_0)
extern  usbd_dev_t      usbd0_dev;
#endif
#if     defined(RTE_USB_Device_1)
extern  usbd_dev_t      usbd1_dev;
#endif
#if     defined(RTE_USB_Device_2)
extern  usbd_dev_t      usbd2_dev;
#endif
#if     defined(RTE_USB_Device_3)
extern  usbd_dev_t      usbd3_dev;
#endif

extern  const uint8_t   usbd_dev_num =  USBD_DEV_NUM;

const   usbd_dev_t * const usbd_dev_ptr[USBD_DEV_NUM]  = { 
#if     defined(RTE_USB_Device_0)
                                       &usbd0_dev
#endif
#if     defined(RTE_USB_Device_1)
                                     , &usbd1_dev
#endif
#if     defined(RTE_USB_Device_2)
                                     , &usbd2_dev
#endif
#if     defined(RTE_USB_Device_3)
                                     , &usbd3_dev
#endif
};

#endif


/*------------------------------------------------------------------------------
 *      USB Device HID 0..3 Configurations
 *----------------------------------------------------------------------------*/

#define USBD_HID0_MAX_PACKET           (MAX_OF_2(USBD_HID0_WMAXPACKETSIZE, USBD_HID0_HS_WMAXPACKETSIZE))
#define USBD_HID1_MAX_PACKET           (MAX_OF_2(USBD_HID1_WMAXPACKETSIZE, USBD_HID1_HS_WMAXPACKETSIZE))
#define USBD_HID2_MAX_PACKET           (MAX_OF_2(USBD_HID2_WMAXPACKETSIZE, USBD_HID2_HS_WMAXPACKETSIZE))
#define USBD_HID3_MAX_PACKET           (MAX_OF_2(USBD_HID3_WMAXPACKETSIZE, USBD_HID3_HS_WMAXPACKETSIZE))

#if     defined(RTE_USB_Device_HID_0)
#define USBD_HID0_NUM                   1
#else
#define USBD_HID0_NUM                   0
#endif
#if     defined(RTE_USB_Device_HID_1)
#define USBD_HID1_NUM                   1
#else
#define USBD_HID1_NUM                   0
#endif
#if     defined(RTE_USB_Device_HID_2)
#define USBD_HID2_NUM                   1
#else
#define USBD_HID2_NUM                   0
#endif
#if     defined(RTE_USB_Device_HID_3)
#define USBD_HID3_NUM                   1
#else
#define USBD_HID3_NUM                   0
#endif

#define USBD_HID_NUM                   (USBD_HID0_NUM + USBD_HID1_NUM + USBD_HID2_NUM + USBD_HID3_NUM)

#if    (defined(LAST_DEVICE_COMPILE) && (USBD_HID_NUM))

extern  void USBD_HID_Thread          (void const *arg);
osThreadDef        (USBD_HID_Thread, osPriorityAboveNormal, USBD_HID_NUM, 0);

osThreadId          usbd_hid_thread_id[USBD_HID_NUM];
const osThreadDef_t * const usbd_hid_thread_def_ptr[USBD_HID_NUM] = {
#if     defined(RTE_USB_Device_HID_0)
        osThread(USBD_HID_Thread)
#endif
#if     defined(RTE_USB_Device_HID_1)
      , osThread(USBD_HID_Thread)
#endif
#if     defined(RTE_USB_Device_HID_2)
      , osThread(USBD_HID_Thread)
#endif
#if     defined(RTE_USB_Device_HID_3)
      , osThread(USBD_HID_Thread)
#endif
};

extern  void USBD_HID_Timer            (void const *arg);
#if     defined(RTE_USB_Device_HID_0)
osTimerDef         (usbd_hid0_timer, USBD_HID_Timer);
#endif
#if     defined(RTE_USB_Device_HID_1)
osTimerDef         (usbd_hid1_timer, USBD_HID_Timer);
#endif
#if     defined(RTE_USB_Device_HID_2)
osTimerDef         (usbd_hid2_timer, USBD_HID_Timer);
#endif
#if     defined(RTE_USB_Device_HID_3)
osTimerDef         (usbd_hid3_timer, USBD_HID_Timer);
#endif
const osTimerDef_t * const usbd_hid_timer_def_ptr[USBD_HID_NUM] = {
#if     defined(RTE_USB_Device_HID_0)
        osTimer(usbd_hid0_timer)
#endif
#if     defined(RTE_USB_Device_HID_1)
      , osTimer(usbd_hid1_timer)
#endif
#if     defined(RTE_USB_Device_HID_2)
      , osTimer(usbd_hid2_timer)
#endif
#if     defined(RTE_USB_Device_HID_3)
      , osTimer(usbd_hid3_timer)
#endif
};

osTimerId           usbd_hid_timer_id[USBD_HID_NUM];

#if    (defined(RTE_USB_Device_HID_0))
uint16_t            usbd_hid0_idle_count [USBD_HID0_IN_REPORT_NUM];
uint16_t            usbd_hid0_idle_reload[USBD_HID0_IN_REPORT_NUM];
uint8_t             usbd_hid0_idle_set   [USBD_HID0_IN_REPORT_NUM];
uint8_t             usbd_hid0_in_report  [USBD_HID0_IN_REPORT_MAX_SZ  +(USBD_HID0_IN_REPORT_MAX_SZ>1)]   __attribute__((aligned(4)));
uint8_t             usbd_hid0_out_report [USBD_HID0_OUT_REPORT_MAX_SZ +(USBD_HID0_OUT_REPORT_MAX_SZ>1)]  __attribute__((aligned(4)));
uint8_t             usbd_hid0_feat_report[USBD_HID0_FEAT_REPORT_MAX_SZ+(USBD_HID0_FEAT_REPORT_MAX_SZ>1)] __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_HID_1))
uint16_t            usbd_hid1_idle_count [USBD_HID1_IN_REPORT_NUM];
uint16_t            usbd_hid1_idle_reload[USBD_HID1_IN_REPORT_NUM];
uint8_t             usbd_hid1_idle_set   [USBD_HID1_IN_REPORT_NUM];
uint8_t             usbd_hid1_in_report  [USBD_HID1_IN_REPORT_MAX_SZ  +(USBD_HID1_IN_REPORT_MAX_SZ>1)]   __attribute__((aligned(4)));
uint8_t             usbd_hid1_out_report [USBD_HID1_OUT_REPORT_MAX_SZ +(USBD_HID1_OUT_REPORT_MAX_SZ>1)]  __attribute__((aligned(4)));
uint8_t             usbd_hid1_feat_report[USBD_HID1_FEAT_REPORT_MAX_SZ+(USBD_HID1_FEAT_REPORT_MAX_SZ>1)] __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_HID_2))
uint16_t            usbd_hid2_idle_count [USBD_HID2_IN_REPORT_NUM];
uint16_t            usbd_hid2_idle_reload[USBD_HID2_IN_REPORT_NUM];
uint8_t             usbd_hid2_idle_set   [USBD_HID2_IN_REPORT_NUM];
uint8_t             usbd_hid2_in_report  [USBD_HID2_IN_REPORT_MAX_SZ  +(USBD_HID2_IN_REPORT_MAX_SZ>1)]   __attribute__((aligned(4)));
uint8_t             usbd_hid2_out_report [USBD_HID2_OUT_REPORT_MAX_SZ +(USBD_HID2_OUT_REPORT_MAX_SZ>1)]  __attribute__((aligned(4)));
uint8_t             usbd_hid2_feat_report[USBD_HID2_FEAT_REPORT_MAX_SZ+(USBD_HID2_FEAT_REPORT_MAX_SZ>1)] __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_HID_3))
uint16_t            usbd_hid3_idle_count [USBD_HID3_IN_REPORT_NUM];
uint16_t            usbd_hid3_idle_reload[USBD_HID3_IN_REPORT_NUM];
uint8_t             usbd_hid3_idle_set   [USBD_HID3_IN_REPORT_NUM];
uint8_t             usbd_hid3_in_report  [USBD_HID3_IN_REPORT_MAX_SZ  +(USBD_HID3_IN_REPORT_MAX_SZ>1)]   __attribute__((aligned(4)));
uint8_t             usbd_hid3_out_report [USBD_HID3_OUT_REPORT_MAX_SZ +(USBD_HID3_OUT_REPORT_MAX_SZ>1)]  __attribute__((aligned(4)));
uint8_t             usbd_hid3_feat_report[USBD_HID3_FEAT_REPORT_MAX_SZ+(USBD_HID3_FEAT_REPORT_MAX_SZ>1)] __attribute__((aligned(4)));
#endif

const   uint8_t     usbd_hid_num  =     USBD_HID_NUM;
usbd_hid_data_t     usbd_hid_data      [USBD_HID_NUM];
usbd_hid_t          usbd_hid           [USBD_HID_NUM] = { 
#if     defined(RTE_USB_Device_HID_0)
                                      { 
                                        usbd_hid0_idle_count, 
                                        usbd_hid0_idle_reload, 
                                        usbd_hid0_idle_set, 
                                        usbd_hid0_in_report, 
                                        usbd_hid0_out_report, 
                                        usbd_hid0_feat_report, 
                                       &usbd_hid_data[0], 
                                        USBD_HID0_DEV, 
                                        USBD_HID0_IF, 
                                        USBD_HID0_EP_INT_IN, 
                                        USBD_HID0_EP_INT_OUT, 
                                      { USBD_HID0_BINTERVAL, (2 << ((USBD_HID0_HS_BINTERVAL & 0x0F)-1)) }, 
                                      { USBD_HID0_WMAXPACKETSIZE, USBD_HID0_HS_WMAXPACKETSIZE },
                                        USBD_HID0_IN_REPORT_NUM, 
                                        USBD_HID0_OUT_REPORT_NUM, 
                                        USBD_HID0_IN_REPORT_MAX_SZ, 
                                        USBD_HID0_OUT_REPORT_MAX_SZ, 
                                        USBD_HID0_FEAT_REPORT_MAX_SZ, 
                                      }
#endif
#if     defined(RTE_USB_Device_HID_1)
                                    , { 
                                        usbd_hid1_idle_count, 
                                        usbd_hid1_idle_reload, 
                                        usbd_hid1_idle_set, 
                                        usbd_hid1_in_report, 
                                        usbd_hid1_out_report, 
                                        usbd_hid1_feat_report, 
                                       &usbd_hid_data[0], 
                                        USBD_HID1_DEV, 
                                        USBD_HID1_IF, 
                                        USBD_HID1_EP_INT_IN, 
                                        USBD_HID1_EP_INT_OUT, 
                                      { USBD_HID1_BINTERVAL, (2 << ((USBD_HID1_HS_BINTERVAL & 0x0F)-1)) }, 
                                      { USBD_HID1_WMAXPACKETSIZE, USBD_HID1_HS_WMAXPACKETSIZE },
                                        USBD_HID1_IN_REPORT_NUM, 
                                        USBD_HID1_OUT_REPORT_NUM, 
                                        USBD_HID1_IN_REPORT_MAX_SZ, 
                                        USBD_HID1_OUT_REPORT_MAX_SZ, 
                                        USBD_HID1_FEAT_REPORT_MAX_SZ, 
                                      }
#endif
#if     defined(RTE_USB_Device_HID_2)
                                    , { 
                                        usbd_hid2_idle_count, 
                                        usbd_hid2_idle_reload, 
                                        usbd_hid2_idle_set, 
                                        usbd_hid2_in_report, 
                                        usbd_hid2_out_report, 
                                        usbd_hid2_feat_report, 
                                       &usbd_hid_data[2], 
                                        USBD_HID2_DEV, 
                                        USBD_HID2_IF, 
                                        USBD_HID2_EP_INT_IN, 
                                        USBD_HID2_EP_INT_OUT, 
                                      { USBD_HID2_BINTERVAL, (2 << ((USBD_HID2_HS_BINTERVAL & 0x0F)-1)) }, 
                                      { USBD_HID2_WMAXPACKETSIZE, USBD_HID2_HS_WMAXPACKETSIZE },
                                        USBD_HID2_IN_REPORT_NUM, 
                                        USBD_HID2_OUT_REPORT_NUM, 
                                        USBD_HID2_IN_REPORT_MAX_SZ, 
                                        USBD_HID2_OUT_REPORT_MAX_SZ, 
                                        USBD_HID2_FEAT_REPORT_MAX_SZ, 
                                      }
#endif
#if     defined(RTE_USB_Device_HID_3)
                                    , { 
                                        usbd_hid3_idle_count, 
                                        usbd_hid3_idle_reload, 
                                        usbd_hid3_idle_set, 
                                        usbd_hid3_in_report, 
                                        usbd_hid3_out_report, 
                                        usbd_hid3_feat_report, 
                                       &usbd_hid_data[3], 
                                        USBD_HID3_DEV, 
                                        USBD_HID3_IF, 
                                        USBD_HID3_EP_INT_IN, 
                                        USBD_HID3_EP_INT_OUT, 
                                      { USBD_HID3_BINTERVAL, (2 << ((USBD_HID3_HS_BINTERVAL & 0x0F)-1)) }, 
                                      { USBD_HID3_WMAXPACKETSIZE, USBD_HID3_HS_WMAXPACKETSIZE },
                                        USBD_HID3_IN_REPORT_NUM, 
                                        USBD_HID3_OUT_REPORT_NUM, 
                                        USBD_HID3_IN_REPORT_MAX_SZ, 
                                        USBD_HID3_OUT_REPORT_MAX_SZ, 
                                        USBD_HID3_FEAT_REPORT_MAX_SZ, 
                                      }
#endif
};
const   usbd_hid_t *usbd_hid_ptr       [USBD_HID_NUM] = { 
#if     defined(RTE_USB_Device_HID_0)
                                       &usbd_hid[0]
#endif
#if     defined(RTE_USB_Device_HID_1)
                                     , &usbd_hid[1]
#endif
#if     defined(RTE_USB_Device_HID_2)
                                     , &usbd_hid[2]
#endif
#if     defined(RTE_USB_Device_HID_3)
                                     , &usbd_hid[3]
#endif
};

#endif


/*------------------------------------------------------------------------------
 *      USB Device MSC 0..3 Configurations
 *----------------------------------------------------------------------------*/

#define USBD_MSC0_MAX_PACKET           (MAX_OF_2(USBD_MSC0_WMAXPACKETSIZE, USBD_MSC0_HS_WMAXPACKETSIZE))
#define USBD_MSC1_MAX_PACKET           (MAX_OF_2(USBD_MSC1_WMAXPACKETSIZE, USBD_MSC1_HS_WMAXPACKETSIZE))
#define USBD_MSC2_MAX_PACKET           (MAX_OF_2(USBD_MSC2_WMAXPACKETSIZE, USBD_MSC2_HS_WMAXPACKETSIZE))
#define USBD_MSC3_MAX_PACKET           (MAX_OF_2(USBD_MSC3_WMAXPACKETSIZE, USBD_MSC3_HS_WMAXPACKETSIZE))

#if     defined(RTE_USB_Device_MSC_0)
#define USBD_MSC0_NUM                   1
#else
#define USBD_MSC0_NUM                   0
#endif
#if     defined(RTE_USB_Device_MSC_1)
#define USBD_MSC1_NUM                   1
#else
#define USBD_MSC1_NUM                   0
#endif
#if     defined(RTE_USB_Device_MSC_2)
#define USBD_MSC2_NUM                   1
#else
#define USBD_MSC2_NUM                   0
#endif
#if     defined(RTE_USB_Device_MSC_3)
#define USBD_MSC3_NUM                   1
#else
#define USBD_MSC3_NUM                   0
#endif

#define USBD_MSC_NUM                   (USBD_MSC0_NUM + USBD_MSC1_NUM + USBD_MSC2_NUM + USBD_MSC3_NUM)

#if    (defined(LAST_DEVICE_COMPILE) && (USBD_MSC_NUM))

extern  void USBD_MSC_Thread          (void const *arg);
osThreadDef        (USBD_MSC_Thread, osPriorityAboveNormal, USBD_MSC_NUM, 0);

osThreadId          usbd_msc_thread_id[USBD_MSC_NUM];
const osThreadDef_t * const usbd_msc_thread_def_ptr[USBD_MSC_NUM] = {
#if     defined(RTE_USB_Device_MSC_0)
        osThread(USBD_MSC_Thread)
#endif
#if     defined(RTE_USB_Device_MSC_1)
      , osThread(USBD_MSC_Thread)
#endif
#if     defined(RTE_USB_Device_MSC_2)
      , osThread(USBD_MSC_Thread)
#endif
#if     defined(RTE_USB_Device_MSC_3)
      , osThread(USBD_MSC_Thread)
#endif
};

#if    (defined(RTE_USB_Device_MSC_0))
uint8_t             usbd_msc0_bulk_buf [USBD_MSC0_MAX_PACKET] __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_MSC_1))
extern  void USBD_MSC1_Thread          (void const *arg);
uint8_t             usbd_msc1_bulk_buf [USBD_MSC1_MAX_PACKET] __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_MSC_2))
extern  void USBD_MSC2_Thread          (void const *arg);
uint8_t             usbd_msc2_bulk_buf [USBD_MSC2_MAX_PACKET] __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_MSC_3))
extern  void USBD_MSC3_Thread          (void const *arg);
uint8_t             usbd_msc3_bulk_buf [USBD_MSC3_MAX_PACKET] __attribute__((aligned(4)));
#endif

const   uint8_t     usbd_msc_num  =     USBD_MSC_NUM;
usbd_msc_data_t     usbd_msc_data      [USBD_MSC_NUM];
usbd_msc_t          usbd_msc           [USBD_MSC_NUM] = { 
#if     defined(RTE_USB_Device_MSC_0)
                                      { 
                                        usbd_msc0_bulk_buf, 
                                       &usbd_msc_data[0], 
                                        USBD_MSC0_DEV, 
                                        USBD_MSC0_IF, 
                                        USBD_MSC0_EP_BULK_IN, 
                                        USBD_MSC0_EP_BULK_OUT, 
                                      { USBD_MSC0_WMAXPACKETSIZE, 
                                        USBD_MSC0_HS_WMAXPACKETSIZE }, 
                                        USBD_MSC0_INQUIRY_DATA, 
                                      } 
#endif
#if     defined(RTE_USB_Device_MSC_1)
                                    , { 
                                        usbd_msc1_bulk_buf, 
                                       &usbd_msc_data[1], 
                                        USBD_MSC1_DEV, 
                                        USBD_MSC1_IF, 
                                        USBD_MSC1_EP_BULK_IN, 
                                        USBD_MSC1_EP_BULK_OUT, 
                                      { USBD_MSC1_WMAXPACKETSIZE, 
                                        USBD_MSC1_HS_WMAXPACKETSIZE }, 
                                        USBD_MSC1_INQUIRY_DATA, 
                                      }
#endif
#if     defined(RTE_USB_Device_MSC_2)
                                    , { 
                                        usbd_msc2_bulk_buf, 
                                       &usbd_msc_data[2], 
                                        USBD_MSC2_DEV, 
                                        USBD_MSC2_IF, 
                                        USBD_MSC2_EP_BULK_IN, 
                                        USBD_MSC2_EP_BULK_OUT, 
                                      { USBD_MSC2_WMAXPACKETSIZE, 
                                        USBD_MSC2_HS_WMAXPACKETSIZE }, 
                                        USBD_MSC2_INQUIRY_DATA, 
                                      }
#endif
#if     defined(RTE_USB_Device_MSC_3)
                                    , { 
                                        usbd_msc3_bulk_buf, 
                                       &usbd_msc_data[3], 
                                        USBD_MSC3_DEV, 
                                        USBD_MSC3_IF, 
                                        USBD_MSC3_EP_BULK_IN, 
                                        USBD_MSC3_EP_BULK_OUT, 
                                      { USBD_MSC3_WMAXPACKETSIZE, 
                                        USBD_MSC3_HS_WMAXPACKETSIZE }, 
                                        USBD_MSC3_INQUIRY_DATA, 
                                      }
#endif
};

const   usbd_msc_t * const usbd_msc_ptr[USBD_MSC_NUM] = {
#if     defined(RTE_USB_Device_MSC_0)
                                       &usbd_msc[0]
#endif
#if     defined(RTE_USB_Device_MSC_1)
                                     , &usbd_msc[1]
#endif
#if     defined(RTE_USB_Device_MSC_2)
                                     , &usbd_msc[2]
#endif
#if     defined(RTE_USB_Device_MSC_3)
                                     , &usbd_msc[3]
#endif
};

#endif


/*------------------------------------------------------------------------------
 *      USB Device ADC 0..3 Configurations
 *----------------------------------------------------------------------------*/

#if     defined(RTE_USB_Device_ADC_0)
#define USBD_ADC0_NUM                   1
#else
#define USBD_ADC0_NUM                   0
#endif
#if     defined(RTE_USB_Device_ADC_1)
#define USBD_ADC1_NUM                   1
#else
#define USBD_ADC1_NUM                   0
#endif
#if     defined(RTE_USB_Device_ADC_2)
#define USBD_ADC2_NUM                   1
#else
#define USBD_ADC2_NUM                   0
#endif
#if     defined(RTE_USB_Device_ADC_3)
#define USBD_ADC3_NUM                   1
#else
#define USBD_ADC3_NUM                   0
#endif

#define USBD_ADC_NUM                   (USBD_ADC0_NUM + USBD_ADC1_NUM + USBD_ADC2_NUM + USBD_ADC3_NUM)

#if    (defined(LAST_DEVICE_COMPILE) && (USBD_ADC_NUM))

extern  void USBD_ADC_Thread          (void const *arg);
osThreadDef        (USBD_ADC_Thread, osPriorityAboveNormal, USBD_ADC_NUM, 0);

osThreadId          usbd_adc_thread_id[USBD_ADC_NUM];
const osThreadDef_t * const usbd_adc_thread_def_ptr[USBD_ADC_NUM] = {
#if     defined(RTE_USB_Device_ADC_0)
        osThread(USBD_ADC_Thread)
#endif
#if     defined(RTE_USB_Device_ADC_1)
      , osThread(USBD_ADC_Thread)
#endif
#if     defined(RTE_USB_Device_ADC_2)
      , osThread(USBD_ADC_Thread)
#endif
#if     defined(RTE_USB_Device_ADC_3)
      , osThread(USBD_ADC_Thread)
#endif
};

#if    (defined(RTE_USB_Device_ADC_0))
int16_t             usbd_adc0_data_buf [8*USBD_ADC0_CFG_PACKET_COUNT*USBD_ADC0_CFG_PACKET_SIZE];
#endif

#if    (defined(RTE_USB_Device_ADC_1))
int16_t             usbd_adc1_data_buf [8*USBD_ADC1_CFG_PACKET_COUNT*USBD_ADC1_CFG_PACKET_SIZE];
#endif

#if    (defined(RTE_USB_Device_ADC_2))
int16_t             usbd_adc2_data_buf [8*USBD_ADC2_CFG_PACKET_COUNT*USBD_ADC2_CFG_PACKET_SIZE];
#endif

#if    (defined(RTE_USB_Device_ADC_3))
int16_t             usbd_adc3_data_buf [8*USBD_ADC3_CFG_PACKET_COUNT*USBD_ADC3_CFG_PACKET_SIZE];
#endif

const   uint8_t     usbd_adc_num  =     USBD_ADC_NUM;
usbd_adc_data_t     usbd_adc_data      [USBD_ADC_NUM];
usbd_adc            usbd_adc           [USBD_ADC_NUM] = { 
#if     defined(RTE_USB_Device_ADC_0)
                                      { 
                                        usbd_adc0_data_buf, 
                                       &usbd_adc_data[0], 
                                        USBD_ADC0_DEV, 
                                        USBD_ADC0_CIF, 
                                        USBD_ADC0_SIF, 
                                        USBD_ADC0_EP_ISO_OUT, 
                                        USBD_ADC0_TSAM_FREQ, 
                                        USBD_ADC0_CFG_PACKET_SIZE, 
                                        USBD_ADC0_CFG_PACKET_COUNT, 
                                       (8*USBD_ADC0_CFG_PACKET_COUNT*USBD_ADC0_CFG_PACKET_SIZE), 
                                      } 
#endif
#if     defined(RTE_USB_Device_ADC_1)
                                    , { 
                                        usbd_adc1_data_buf, 
                                       &usbd_adc_data[1], 
                                        USBD_ADC1_DEV, 
                                        USBD_ADC1_CIF, 
                                        USBD_ADC1_SIF, 
                                        USBD_ADC1_EP_ISO_OUT, 
                                        USBD_ADC1_TSAM_FREQ, 
                                        USBD_ADC1_CFG_PACKET_SIZE, 
                                        USBD_ADC1_CFG_PACKET_COUNT, 
                                       (8*USBD_ADC1_CFG_PACKET_COUNT*USBD_ADC1_CFG_PACKET_SIZE), 
                                      } 
#endif
#if     defined(RTE_USB_Device_ADC_2)
                                    , { 
                                        usbd_adc2_data_buf, 
                                       &usbd_adc_data[2], 
                                        USBD_ADC2_DEV, 
                                        USBD_ADC2_CIF, 
                                        USBD_ADC2_SIF, 
                                        USBD_ADC2_EP_ISO_OUT, 
                                        USBD_ADC2_TSAM_FREQ, 
                                        USBD_ADC2_CFG_PACKET_SIZE, 
                                        USBD_ADC2_CFG_PACKET_COUNT, 
                                       (8*USBD_ADC2_CFG_PACKET_COUNT*USBD_ADC2_CFG_PACKET_SIZE), 
                                      } 
#endif
#if     defined(RTE_USB_Device_ADC_3)
                                    , { 
                                        usbd_adc3_data_buf, 
                                       &usbd_adc_data[3], 
                                        USBD_ADC3_DEV, 
                                        USBD_ADC3_CIF, 
                                        USBD_ADC3_SIF, 
                                        USBD_ADC3_EP_ISO_OUT, 
                                        USBD_ADC3_TSAM_FREQ, 
                                        USBD_ADC3_CFG_PACKET_SIZE, 
                                        USBD_ADC3_CFG_PACKET_COUNT, 
                                       (8*USBD_ADC3_CFG_PACKET_COUNT*USBD_ADC3_CFG_PACKET_SIZE), 
                                      } 
#endif
};
const   usbd_adc_t * const usbd_adc_ptr[USBD_ADC_NUM] = { 
#if     defined(RTE_USB_Device_ADC_0)
                                       &usbd_adc[0]
#endif
#if     defined(RTE_USB_Device_ADC_1)
                                     , &usbd_adc[1]
#endif
#if     defined(RTE_USB_Device_ADC_2)
                                     , &usbd_adc[2]
#endif
#if     defined(RTE_USB_Device_ADC_3)
                                     , &usbd_adc[3]
#endif
};

#endif


/*------------------------------------------------------------------------------
 *      USB Device CDC 0..3 Configurations
 *----------------------------------------------------------------------------*/

#define USBD_CDC0_MAX_PACKET           (MAX_OF_2(USBD_CDC0_WMAXPACKETSIZE,  USBD_CDC0_HS_WMAXPACKETSIZE))
#define USBD_CDC1_MAX_PACKET           (MAX_OF_2(USBD_CDC1_WMAXPACKETSIZE,  USBD_CDC1_HS_WMAXPACKETSIZE))
#define USBD_CDC2_MAX_PACKET           (MAX_OF_2(USBD_CDC2_WMAXPACKETSIZE,  USBD_CDC2_HS_WMAXPACKETSIZE))
#define USBD_CDC3_MAX_PACKET           (MAX_OF_2(USBD_CDC3_WMAXPACKETSIZE,  USBD_CDC3_HS_WMAXPACKETSIZE))
#define USBD_CDC0_MAX_PACKET1          (MAX_OF_2(USBD_CDC0_WMAXPACKETSIZE1, USBD_CDC0_HS_WMAXPACKETSIZE1))
#define USBD_CDC1_MAX_PACKET1          (MAX_OF_2(USBD_CDC1_WMAXPACKETSIZE1, USBD_CDC1_HS_WMAXPACKETSIZE1))
#define USBD_CDC2_MAX_PACKET1          (MAX_OF_2(USBD_CDC2_WMAXPACKETSIZE1, USBD_CDC2_HS_WMAXPACKETSIZE1))
#define USBD_CDC3_MAX_PACKET1          (MAX_OF_2(USBD_CDC3_WMAXPACKETSIZE1, USBD_CDC3_HS_WMAXPACKETSIZE1))

#if     defined(RTE_USB_Device_CDC_0)
#define USBD_CDC0_NUM                   1
#else
#define USBD_CDC0_NUM                   0
#endif
#if     defined(RTE_USB_Device_CDC_1)
#define USBD_CDC1_NUM                   1
#else
#define USBD_CDC1_NUM                   0
#endif
#if     defined(RTE_USB_Device_CDC_2)
#define USBD_CDC2_NUM                   1
#else
#define USBD_CDC2_NUM                   0
#endif
#if     defined(RTE_USB_Device_CDC_3)
#define USBD_CDC3_NUM                   1
#else
#define USBD_CDC3_NUM                   0
#endif

#define USBD_CDC_NUM                   (USBD_CDC0_NUM + USBD_CDC1_NUM + USBD_CDC2_NUM + USBD_CDC3_NUM)

#if    (defined(LAST_DEVICE_COMPILE) && (USBD_CDC_NUM))

extern  void USBD_CDC_Thread0         (void const *arg);
extern  void USBD_CDC_Thread1         (void const *arg);
osThreadDef        (USBD_CDC_Thread0, osPriorityAboveNormal, USBD_CDC_NUM, 0);
osThreadDef        (USBD_CDC_Thread1, osPriorityAboveNormal, USBD_CDC_NUM, 0);

osThreadId          usbd_cdc_thread_id[2 * USBD_CDC_NUM];
const osThreadDef_t * const usbd_cdc_thread_def_ptr[2 * USBD_CDC_NUM] = {
#if     defined(RTE_USB_Device_CDC_0)
        osThread(USBD_CDC_Thread0)
      , osThread(USBD_CDC_Thread1)
#endif
#if     defined(RTE_USB_Device_CDC_1)
      , osThread(USBD_CDC_Thread0)
      , osThread(USBD_CDC_Thread1)
#endif
#if     defined(RTE_USB_Device_CDC_2)
      , osThread(USBD_CDC_Thread0)
      , osThread(USBD_CDC_Thread1)
#endif
#if     defined(RTE_USB_Device_CDC_3)
      , osThread(USBD_CDC_Thread0)
      , osThread(USBD_CDC_Thread1)
#endif
};


#if    (defined(RTE_USB_Device_CDC_0))
uint8_t             usbd_cdc0_acm_send_buf   [USBD_CDC0_SEND_BUF_SIZE]    __attribute__((aligned(4)));
uint8_t             usbd_cdc0_acm_receive_buf[USBD_CDC0_RECEIVE_BUF_SIZE] __attribute__((aligned(4)));
uint8_t             usbd_cdc0_acm_notify_buf [10]                         __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_CDC_1))
uint8_t             usbd_cdc1_acm_send_buf   [USBD_CDC1_SEND_BUF_SIZE]    __attribute__((aligned(4)));
uint8_t             usbd_cdc1_acm_receive_buf[USBD_CDC1_RECEIVE_BUF_SIZE] __attribute__((aligned(4)));
uint8_t             usbd_cdc1_acm_notify_buf [10]                         __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_CDC_2))
uint8_t             usbd_cdc2_acm_send_buf   [USBD_CDC2_SEND_BUF_SIZE]    __attribute__((aligned(4)));
uint8_t             usbd_cdc2_acm_receive_buf[USBD_CDC2_RECEIVE_BUF_SIZE] __attribute__((aligned(4)));
uint8_t             usbd_cdc2_acm_notify_buf [10]                         __attribute__((aligned(4)));
#endif

#if    (defined(RTE_USB_Device_CDC_3))
uint8_t             usbd_cdc3_acm_send_buf   [USBD_CDC3_SEND_BUF_SIZE]    __attribute__((aligned(4)));
uint8_t             usbd_cdc3_acm_receive_buf[USBD_CDC3_RECEIVE_BUF_SIZE] __attribute__((aligned(4)));
uint8_t             usbd_cdc3_acm_notify_buf [10]                         __attribute__((aligned(4)));
#endif

const   uint8_t     usbd_cdc_num  =     USBD_CDC_NUM;
usbd_cdc_data_t     usbd_cdc_data      [USBD_CDC_NUM];
usbd_cdc_t          usbd_cdc           [USBD_CDC_NUM] = { 
#if     defined(RTE_USB_Device_CDC_0)
                                      { 
                                        usbd_cdc0_acm_send_buf, 
                                        usbd_cdc0_acm_receive_buf, 
                                        usbd_cdc0_acm_notify_buf, 
                                       &usbd_cdc_data[0], 
                                        USBD_CDC0_DEV, 
                                        USBD_CDC0_CIF, 
                                        USBD_CDC0_DIF, 
                                        USBD_CDC0_EP_INT_IN, 
                                        USBD_CDC0_EP_BULK_IN, 
                                        USBD_CDC0_EP_BULK_OUT, 
                                        USBD_CDC0_SEND_BUF_SIZE, 
                                        USBD_CDC0_RECEIVE_BUF_SIZE, 
                                      { USBD_CDC0_WMAXPACKETSIZE,  USBD_CDC0_HS_WMAXPACKETSIZE  }, 
                                      { USBD_CDC0_WMAXPACKETSIZE1, USBD_CDC0_HS_WMAXPACKETSIZE1 } 
                                      }
#endif
#if     defined(RTE_USB_Device_CDC_1)
                                    , { 
                                        usbd_cdc1_acm_send_buf, 
                                        usbd_cdc1_acm_receive_buf, 
                                        usbd_cdc1_acm_notify_buf, 
                                       &usbd_cdc_data[0], 
                                        USBD_CDC1_DEV, 
                                        USBD_CDC1_CIF, 
                                        USBD_CDC1_DIF, 
                                        USBD_CDC1_EP_INT_IN, 
                                        USBD_CDC1_EP_BULK_IN, 
                                        USBD_CDC1_EP_BULK_OUT, 
                                        USBD_CDC1_SEND_BUF_SIZE, 
                                        USBD_CDC1_RECEIVE_BUF_SIZE, 
                                      { USBD_CDC1_WMAXPACKETSIZE,  USBD_CDC1_HS_WMAXPACKETSIZE  }, 
                                      { USBD_CDC1_WMAXPACKETSIZE1, USBD_CDC1_HS_WMAXPACKETSIZE1 } 
                                      }
#endif
#if     defined(RTE_USB_Device_CDC_2)
                                    , { 
                                        usbd_cdc2_acm_send_buf, 
                                        usbd_cdc2_acm_receive_buf, 
                                        usbd_cdc2_acm_notify_buf, 
                                       &usbd_cdc_data[0], 
                                        USBD_CDC2_DEV, 
                                        USBD_CDC2_CIF, 
                                        USBD_CDC2_DIF, 
                                        USBD_CDC2_EP_INT_IN, 
                                        USBD_CDC2_EP_BULK_IN, 
                                        USBD_CDC2_EP_BULK_OUT, 
                                        USBD_CDC2_SEND_BUF_SIZE, 
                                        USBD_CDC2_RECEIVE_BUF_SIZE, 
                                      { USBD_CDC2_WMAXPACKETSIZE,  USBD_CDC2_HS_WMAXPACKETSIZE  }, 
                                      { USBD_CDC2_WMAXPACKETSIZE1, USBD_CDC2_HS_WMAXPACKETSIZE1 } 
                                      }
#endif
#if     defined(RTE_USB_Device_CDC_3)
                                    , { 
                                        usbd_cdc3_acm_send_buf, 
                                        usbd_cdc3_acm_receive_buf, 
                                        usbd_cdc3_acm_notify_buf, 
                                       &usbd_cdc_data[0], 
                                        USBD_CDC3_DEV, 
                                        USBD_CDC3_CIF, 
                                        USBD_CDC3_DIF, 
                                        USBD_CDC3_EP_INT_IN, 
                                        USBD_CDC3_EP_BULK_IN, 
                                        USBD_CDC3_EP_BULK_OUT, 
                                        USBD_CDC3_SEND_BUF_SIZE, 
                                        USBD_CDC3_RECEIVE_BUF_SIZE, 
                                      { USBD_CDC3_WMAXPACKETSIZE,  USBD_CDC3_HS_WMAXPACKETSIZE  }, 
                                      { USBD_CDC3_WMAXPACKETSIZE1, USBD_CDC3_HS_WMAXPACKETSIZE1 } 
                                      }
#endif
};
const   usbd_cdc_t * const usbd_cdc_ptr[USBD_CDC_NUM] = { 
#if     defined(RTE_USB_Device_CDC_0)
                                       &usbd_cdc[0]
#endif
#if     defined(RTE_USB_Device_CDC_1)
                                     , &usbd_cdc[1]
#endif
#if     defined(RTE_USB_Device_CDC_2)
                                     , &usbd_cdc[2]
#endif
#if     defined(RTE_USB_Device_CDC_3)
                                     , &usbd_cdc[3]
#endif
};

#endif


/*------------------------------------------------------------------------------
 *      USB Device Event Handler Fuctions
 *----------------------------------------------------------------------------*/

#if    (defined(LAST_DEVICE_COMPILE) && (USBD_DEV_NUM))

const   uint8_t         usbd_ep_max_num = USBD_EP_LAST_NUM + 1;
const   osThreadId     *usbd_ep_thread_id_ptr[USBD_DEV_NUM * (USBD_EP_LAST_NUM + 1)] = {
#if    (USBD_DEV_NUM > 0)
  EP_POINTERS(0)
#endif
#if    (USBD_DEV_NUM > 1)
, EP_POINTERS(1)
#endif
#if    (USBD_DEV_NUM > 2)
, EP_POINTERS(2)
#endif
#if    (USBD_DEV_NUM > 3)
, EP_POINTERS(3)
#endif
};

#if    (defined(RTE_USB_Device_0) || defined(RTE_USB_Device_1) || defined(RTE_USB_Device_2) || defined(RTE_USB_Device_3))
extern  void    USBD_CoreThread          (void const *arg);
osThreadDef(USBD_CoreThread, osPriorityAboveNormal, USBD_DEV_NUM, NULL);
const osThreadDef_t * const usbd_core_thread_def_ptr[USBD_DEV_NUM] = {
#if     defined(RTE_USB_Device_0)
        osThread(USBD_CoreThread)
#endif
#if     defined(RTE_USB_Device_1)
      , osThread(USBD_CoreThread)
#endif
#if     defined(RTE_USB_Device_2)
      , osThread(USBD_CoreThread)
#endif
#if     defined(RTE_USB_Device_3)
      , osThread(USBD_CoreThread)
#endif
};

extern  void    USBD_SignalDeviceEvent   (int8_t device, ARM_USBD_EVENT event);
extern  void    USBD_SignalEndpointEvent (int8_t device, uint8_t ep_addr, ARM_USBD_EP_EVENT event);
#if     defined(RTE_USB_Device_0)
        void    USBD0_SignalDeviceEvent  (ARM_USBD_EVENT event)                       { USBD_SignalDeviceEvent  (0, event);            }
        void    USBD0_SignalEndpointEvent(uint8_t ep_addr, ARM_USBD_EP_EVENT event)   { USBD_SignalEndpointEvent(0, ep_addr, event);   }
#endif
#if     defined(RTE_USB_Device_1)
        void    USBD1_SignalDeviceEvent  (ARM_USBD_EVENT event)                       { USBD_SignalDeviceEvent  (1, event);            }
        void    USBD1_SignalEndpointEvent(uint8_t ep_addr, ARM_USBD_EP_EVENT event)   { USBD_SignalEndpointEvent(1, ep_addr, event);   }
#endif
#if     defined(RTE_USB_Device_2)
        void    USBD2_SignalDeviceEvent  (ARM_USBD_EVENT event)                       { USBD_SignalDeviceEvent  (2, event);            }
        void    USBD2_SignalEndpointEvent(uint8_t ep_addr, ARM_USBD_EP_EVENT event)   { USBD_SignalEndpointEvent(2, ep_addr, event);   }
#endif
#if     defined(RTE_USB_Device_3)
        void    USBD3_SignalDeviceEvent  (ARM_USBD_EVENT event)                       { USBD_SignalDeviceEvent  (3, event);            }
        void    USBD3_SignalEndpointEvent(uint8_t ep_addr, ARM_USBD_EP_EVENT event)   { USBD_SignalEndpointEvent(3, ep_addr, event);   }
#endif

        ARM_USBD_SignalDeviceEvent_t const usbd_cb_device_event_ptr[USBD_DEV_NUM]   = {
#if    (USBD_DEV_NUM > 0)
        USBD0_SignalDeviceEvent
#endif
#if    (USBD_DEV_NUM > 1)
      , USBD1_SignalDeviceEvent
#endif
#if    (USBD_DEV_NUM > 2)
      , USBD2_SignalDeviceEvent
#endif
#if    (USBD_DEV_NUM > 3)
      , USBD3_SignalDeviceEvent 
#endif
};

        ARM_USBD_SignalEndpointEvent_t const usbd_cb_endpoint_event_ptr[USBD_DEV_NUM] = {
#if    (USBD_DEV_NUM > 0)
        USBD0_SignalEndpointEvent
#endif
#if    (USBD_DEV_NUM > 1)
      , USBD1_SignalEndpointEvent
#endif
#if    (USBD_DEV_NUM > 2)
      , USBD2_SignalEndpointEvent
#endif
#if    (USBD_DEV_NUM > 3)
      , USBD3_SignalEndpointEvent 
#endif
};

ARM_DRV_VERSION       usbd_drv_version [USBD_DEV_NUM] = { 0 };
ARM_USBD_CAPABILITIES usbd_capabilities[USBD_DEV_NUM] = { 0 };
#endif

#if   (!defined(RTE_USB_Device_HID_0) && !defined(RTE_USB_Device_HID_1) && !defined(RTE_USB_Device_HID_2) && !defined(RTE_USB_Device_HID_3))
        bool      USBD_Endpoint0_ReqGetDescriptor_HID (int8_t device, uint8_t **pD, uint32_t *len) { return false; }
        bool      USBD_EndPoint0_Setup_HID_ReqToIF    (int8_t device)                              { return false; }
        bool      USBD_EndPoint0_Out_HID_ReqToIF      (int8_t device)                              { return false; }
#else
extern  usbStatus USBD_HID_ClassInitialize            (int8_t instance);
extern  usbStatus USBD_HID_ClassUninitialize          (int8_t instance);
extern  void      USBD_HID_EventReset                 (int8_t instance);
extern  void      USBD_HID_EventSetConfiguration      (int8_t instance);
extern  void      USBD_HID_EventEndpointEnable        (int8_t instance, uint8_t ep_addr);
__weak  void      USBD_HID0_Initialize                (void)                                                                       {};
__weak  void      USBD_HID0_Uninitialize              (void)                                                                       {};
__weak  int32_t   USBD_HID0_GetReport                 (uint8_t rtype, uint8_t  req, uint8_t rid,       uint8_t *buf)               { return -1;    };
__weak  bool      USBD_HID0_SetReport                 (uint8_t rtype, uint8_t  req, uint8_t rid, const uint8_t *buf, uint16_t len) { return false; };
__weak  int8_t    USBD_HID0_GetProtocol               (void)                                                                       { return -1;    };
__weak  bool      USBD_HID0_SetProtocol               (uint8_t protocol)                                                           { return false; };
#if     defined(RTE_USB_Device_HID_1)
__weak  void      USBD_HID1_Initialize                (void)                                                                       {};
__weak  void      USBD_HID1_Uninitialize              (void)                                                                       {};
__weak  int32_t   USBD_HID1_GetReport                 (uint8_t rtype, uint8_t  req, uint8_t rid,       uint8_t *buf)               { return -1;    };
__weak  bool      USBD_HID1_SetReport                 (uint8_t rtype, uint8_t  req, uint8_t rid, const uint8_t *buf, uint16_t len) { return false; };
__weak  int8_t    USBD_HID1_GetProtocol               (void)                                                                       { return -1;    };
__weak  bool      USBD_HID1_SetProtocol               (uint8_t protocol)                                                           { return false; };
#endif
#if     defined(RTE_USB_Device_HID_2)
__weak  void      USBD_HID2_Initialize                (void)                                                                       {};
__weak  void      USBD_HID2_Uninitialize              (void)                                                                       {};
__weak  int32_t   USBD_HID2_GetReport                 (uint8_t rtype, uint8_t  req, uint8_t rid,       uint8_t *buf)               { return -1;    };
__weak  bool      USBD_HID2_SetReport                 (uint8_t rtype, uint8_t  req, uint8_t rid, const uint8_t *buf, uint16_t len) { return false; };
__weak  int8_t    USBD_HID2_GetProtocol               (void)                                                                       { return -1;    };
__weak  bool      USBD_HID2_SetProtocol               (uint8_t protocol)                                                           { return false; };
#endif
#if     defined(RTE_USB_Device_HID_3)
__weak  void      USBD_HID3_Initialize                (void)                                                                       {};
__weak  void      USBD_HID3_Uninitialize              (void)                                                                       {};
__weak  int32_t   USBD_HID3_GetReport                 (uint8_t rtype, uint8_t  req, uint8_t rid,       uint8_t *buf)               { return -1;    };
__weak  bool      USBD_HID3_SetReport                 (uint8_t rtype, uint8_t  req, uint8_t rid, const uint8_t *buf, uint16_t len) { return false; };
__weak  int8_t    USBD_HID3_GetProtocol               (void)                                                                       { return -1;    };
__weak  bool      USBD_HID3_SetProtocol               (uint8_t protocol)                                                           { return false; };
#endif
        void (* const USBD_HID_Initialize[USBD_HID_NUM])  (void)                                                                 = 
      { 
        USBD_HID0_Initialize
#if     defined(RTE_USB_Device_HID_1)
      , USBD_HID1_Initialize
#endif
#if     defined(RTE_USB_Device_HID_2)
      , USBD_HID2_Initialize
#endif
#if     defined(RTE_USB_Device_HID_3)
      , USBD_HID3_Initialize 
#endif
      };
        void (* const USBD_HID_Uninitialize[USBD_HID_NUM])(void)                                                                 = 
      { 
        USBD_HID0_Uninitialize
#if     defined(RTE_USB_Device_HID_1)
      , USBD_HID1_Uninitialize
#endif
#if     defined(RTE_USB_Device_HID_2)
      , USBD_HID2_Uninitialize
#endif
#if     defined(RTE_USB_Device_HID_3)
      , USBD_HID3_Uninitialize 
#endif
      };
        int32_t  (* const USBD_HID_GetReport[USBD_HID_NUM])   (uint8_t rtype, uint8_t  req, uint8_t  rid, uint8_t *buf)              = 
      { 
        USBD_HID0_GetReport
#if     defined(RTE_USB_Device_HID_1)
      , USBD_HID1_GetReport
#endif
#if     defined(RTE_USB_Device_HID_2)
      , USBD_HID2_GetReport
#endif
#if     defined(RTE_USB_Device_HID_3)
      , USBD_HID3_GetReport 
#endif
      };
        bool (* const USBD_HID_SetReport[USBD_HID_NUM])   (uint8_t rtype, uint8_t req, uint8_t  rid, const uint8_t *buf, uint16_t len) = 
      { 
        USBD_HID0_SetReport
#if     defined(RTE_USB_Device_HID_1)
      , USBD_HID1_SetReport
#endif
#if     defined(RTE_USB_Device_HID_2)
      , USBD_HID2_SetReport
#endif
#if     defined(RTE_USB_Device_HID_3)
      , USBD_HID3_SetReport 
#endif
      };
        int8_t (* const USBD_HID_GetProtocol[USBD_HID_NUM]) (void)                                                                 = 
      { 
        USBD_HID0_GetProtocol
#if     defined(RTE_USB_Device_HID_1)
      , USBD_HID1_GetProtocol
#endif
#if     defined(RTE_USB_Device_HID_2)
      , USBD_HID2_GetProtocol
#endif
#if     defined(RTE_USB_Device_HID_3)
      , USBD_HID3_GetProtocol
#endif
      };
        bool (* const USBD_HID_SetProtocol[USBD_HID_NUM]) (uint8_t protocol)                                                     = 
      { 
        USBD_HID0_SetProtocol
#if     defined(RTE_USB_Device_HID_1)
      , USBD_HID1_SetProtocol
#endif
#if     defined(RTE_USB_Device_HID_2)
      , USBD_HID2_SetProtocol
#endif
#if     defined(RTE_USB_Device_HID_3)
      , USBD_HID3_SetProtocol
#endif
      };
#endif

#if   (!defined(RTE_USB_Device_MSC_0) && !defined(RTE_USB_Device_MSC_1) && !defined(RTE_USB_Device_MSC_2) && !defined(RTE_USB_Device_MSC_3))
        void      USBD_Endpoint0_ReqClrFeature_MSC (int8_t device, uint8_t ep_addr)             {               }
        bool      USBD_EndPoint0_Setup_MSC_ReqToIF (int8_t device)                              { return false; }
#else
extern  usbStatus USBD_MSC_ClassInitialize         (int8_t instance);
extern  usbStatus USBD_MSC_ClassUninitialize       (int8_t instance);
extern  void      USBD_MSC_EventReset              (int8_t instance);
extern  void      USBD_MSC_EventEndpointEnable     (int8_t instance, uint8_t ep_addr);
__weak  void      USBD_MSC0_Initialize             (void)                                                 {};
__weak  void      USBD_MSC0_Uninitialize           (void)                                                 {};
__weak  bool      USBD_MSC0_Read                   (uint32_t lba, uint16_t cnt, uint8_t *buf)             { return false; };
__weak  bool      USBD_MSC0_Write                  (uint32_t lba, uint16_t cnt, const uint8_t *buf)       { return false; };
__weak  bool      USBD_MSC0_StartStop              (bool start)                                           { return false; };
#if     defined(RTE_USB_Device_MSC_1)
__weak  void      USBD_MSC1_Initialize             (void)                                                 {};
__weak  void      USBD_MSC1_Uninitialize           (void)                                                 {};
__weak  bool      USBD_MSC1_Read                   (uint32_t lba, uint16_t cnt, uint8_t *buf)             { return false; };
__weak  bool      USBD_MSC1_Write                  (uint32_t lba, uint16_t cnt, const uint8_t *buf)       { return false; };
__weak  bool      USBD_MSC1_StartStop              (bool start)                                           { return false; };
#endif
#if     defined(RTE_USB_Device_MSC_2)
__weak  void      USBD_MSC2_Initialize             (void)                                                 {};
__weak  void      USBD_MSC2_Uninitialize           (void)                                                 {};
__weak  bool      USBD_MSC2_Read                   (uint32_t lba, uint16_t cnt, uint8_t *buf)             { return false; };
__weak  bool      USBD_MSC2_Write                  (uint32_t lba, uint16_t cnt, const uint8_t *buf)       { return false; };
__weak  bool      USBD_MSC2_StartStop              (bool start)                                           { return false; };
#endif
#if     defined(RTE_USB_Device_MSC_3)
__weak  void      USBD_MSC3_Initialize             (void)                                                 {};
__weak  void      USBD_MSC3_Uninitialize           (void)                                                 {};
__weak  bool      USBD_MSC3_Read                   (uint32_t lba, uint16_t cnt, uint8_t *buf)             { return false; };
__weak  bool      USBD_MSC3_Write                  (uint32_t lba, uint16_t cnt, const uint8_t *buf)       { return false; };
__weak  bool      USBD_MSC3_StartStop              (bool start)                                           { return false; };
#endif
        void (* const USBD_MSC_Initialize[USBD_MSC_NUM]) (void)                                                 = 
      { 
        USBD_MSC0_Initialize
#if     defined(RTE_USB_Device_MSC_1)
      , USBD_MSC1_Initialize
#endif
#if     defined(RTE_USB_Device_MSC_2)
      , USBD_MSC2_Initialize
#endif
#if     defined(RTE_USB_Device_MSC_3)
      , USBD_MSC3_Initialize 
#endif
      };
        void (* const USBD_MSC_Uninitialize[USBD_MSC_NUM]) (void)                                                 = 
      { 
        USBD_MSC0_Uninitialize
#if     defined(RTE_USB_Device_MSC_1)
      , USBD_MSC1_Uninitialize
#endif
#if     defined(RTE_USB_Device_MSC_2)
      , USBD_MSC2_Uninitialize
#endif
#if     defined(RTE_USB_Device_MSC_3)
      , USBD_MSC3_Uninitialize 
#endif
      };
        bool (* const USBD_MSC_Read[USBD_MSC_NUM]) (uint32_t lba, uint16_t cnt, uint8_t *buf) = 
      { 
        USBD_MSC0_Read
#if     defined(RTE_USB_Device_MSC_1)
      , USBD_MSC1_Read
#endif
#if     defined(RTE_USB_Device_MSC_2)
      , USBD_MSC2_Read
#endif
#if     defined(RTE_USB_Device_MSC_3)
      , USBD_MSC3_Read
#endif
      };
        bool (* const USBD_MSC_Write[USBD_MSC_NUM]) (uint32_t lba, uint16_t cnt, const uint8_t *buf) = 
      { 
        USBD_MSC0_Write
#if     defined(RTE_USB_Device_MSC_1)
      , USBD_MSC1_Write
#endif
#if     defined(RTE_USB_Device_MSC_2)
      , USBD_MSC2_Write
#endif
#if     defined(RTE_USB_Device_MSC_3)
      , USBD_MSC3_Write
#endif
      };
        bool (* const USBD_MSC_StartStop[USBD_MSC_NUM]) (bool start) = 
      { 
        USBD_MSC0_StartStop
#if     defined(RTE_USB_Device_MSC_1)
      , USBD_MSC1_StartStop
#endif
#if     defined(RTE_USB_Device_MSC_2)
      , USBD_MSC2_StartStop
#endif
#if     defined(RTE_USB_Device_MSC_3)
      , USBD_MSC3_StartStop 
#endif
      };
#endif

#if (!defined(RTE_USB_Device_ADC_0) && !defined(RTE_USB_Device_ADC_1) && !defined(RTE_USB_Device_ADC_2) && !defined(RTE_USB_Device_ADC_3))
        bool      USBD_EndPoint0_Setup_ADC_ReqToIF (int8_t device)                              { return false; }
        bool      USBD_EndPoint0_Setup_ADC_ReqToEP (int8_t device)                              { return false; }
        bool      USBD_EndPoint0_Out_ADC_ReqToIF   (int8_t device)                              { return false; }
        bool      USBD_EndPoint0_Out_ADC_ReqToEP   (int8_t device)                              { return false; }
#else
extern  bool      USBD_EndPoint0_Setup_ADC_ReqToIF (int8_t device);
extern  bool      USBD_EndPoint0_Setup_ADC_ReqToEP (int8_t device);
extern  bool      USBD_EndPoint0_Out_ADC_ReqToIF   (int8_t device);
extern  bool      USBD_EndPoint0_Out_ADC_ReqToEP   (int8_t device);
extern  usbStatus USBD_ADC_ClassInitialize         (int8_t instance);
extern  usbStatus USBD_ADC_ClassUninitialize       (int8_t instance);
extern  void      USBD_ADC_EventReset              (int8_t instance);
extern  void      USBD_ADC_EventEndpointEnable     (int8_t instance, uint8_t ep_addr);
__weak  void      USBD_ADC0_Initialize             (void) {};
__weak  void      USBD_ADC0_Uninitialize           (void) {};
#if     defined(RTE_USB_Device_ADC_1)
__weak  void      USBD_ADC1_Initialize             (void) {};
__weak  void      USBD_ADC1_Uninitialize           (void) {};
#endif
#if     defined(RTE_USB_Device_ADC_2)
__weak  void      USBD_ADC2_Initialize             (void) {};
__weak  void      USBD_ADC2_Uninitialize           (void) {};
#endif
#if     defined(RTE_USB_Device_ADC_3)
__weak  void      USBD_ADC3_Initialize             (void) {};
__weak  void      USBD_ADC3_Uninitialize           (void) {};
#endif
        void (* const USBD_ADC_Initialize[USBD_ADC_NUM]) (void) = 
      { 
        USBD_ADC0_Initialize
#if     defined(RTE_USB_Device_ADC_1)
      , USBD_ADC1_Initialize
#endif
#if     defined(RTE_USB_Device_ADC_2)
      , USBD_ADC2_Initialize
#endif
#if     defined(RTE_USB_Device_ADC_3)
      , USBD_ADC3_Initialize 
#endif
      };
        void (* const USBD_ADC_Uninitialize[USBD_ADC_NUM]) (void) = 
      { 
        USBD_ADC0_Uninitialize
#if     defined(RTE_USB_Device_ADC_1)
      , USBD_ADC1_Uninitialize
#endif
#if     defined(RTE_USB_Device_ADC_2)
      , USBD_ADC2_Uninitialize
#endif
#if     defined(RTE_USB_Device_ADC_3)
      , USBD_ADC3_Uninitialize 
#endif
      };
#endif

#if   (!defined(RTE_USB_Device_CDC_0) && !defined(RTE_USB_Device_CDC_1) && !defined(RTE_USB_Device_CDC_2) && !defined(RTE_USB_Device_CDC_3))
        bool      USBD_EndPoint0_Setup_CDC_ReqToIF (int8_t device)                              { return false; }
        bool      USBD_EndPoint0_Out_CDC_ReqToIF   (int8_t device)                              { return false; }
#else
extern  bool      USBD_EndPoint0_Setup_CDC_ReqToIF (int8_t device);
extern  bool      USBD_EndPoint0_Out_CDC_ReqToIF   (int8_t device);
extern  usbStatus USBD_CDC_ClassInitialize         (int8_t instance);
extern  usbStatus USBD_CDC_ClassUninitialize       (int8_t instance);
extern  void      USBD_CDC_EventReset              (int8_t instance);
extern  void      USBD_CDC_EventEndpointEnable     (int8_t instance, uint8_t ep_addr);
__weak  void      USBD_CDC0_ACM_Initialize         (void)                         { };
__weak  void      USBD_CDC0_ACM_Uninitialize       (void)                         { };
__weak  void      USBD_CDC0_ACM_Reset              (void)                         { };
__weak  bool      USBD_CDC0_ACM_SetLineCoding      (CDC_LINE_CODING *line_coding) { return false; };
__weak  bool      USBD_CDC0_ACM_GetLineCoding      (CDC_LINE_CODING *line_coding) { return false; };
__weak  bool      USBD_CDC0_ACM_SetControlLineState(uint16_t state)               { return false; };
#if     defined(RTE_USB_Device_CDC_1)
__weak  void      USBD_CDC1_ACM_Initialize         (void)                         { };
__weak  void      USBD_CDC1_ACM_Uninitialize       (void)                         { };
__weak  void      USBD_CDC1_ACM_Reset              (void)                         { };
__weak  bool      USBD_CDC1_ACM_SetLineCoding      (CDC_LINE_CODING *line_coding) { return false; };
__weak  bool      USBD_CDC1_ACM_GetLineCoding      (CDC_LINE_CODING *line_coding) { return false; };
__weak  bool      USBD_CDC1_ACM_SetControlLineState(uint16_t state)               { return false; };
#endif
#if     defined(RTE_USB_Device_CDC_2)
__weak  void      USBD_CDC2_ACM_Initialize         (void)                         { };
__weak  void      USBD_CDC2_ACM_Uninitialize       (void)                         { };
__weak  bool      USBD_CDC2_ACM_Reset              (void)                         { return false; };
__weak  bool      USBD_CDC2_ACM_SetLineCoding      (CDC_LINE_CODING *line_coding) { return false; };
__weak  bool      USBD_CDC2_ACM_GetLineCoding      (CDC_LINE_CODING *line_coding) { return false; };
__weak  bool      USBD_CDC2_ACM_SetControlLineState(uint16_t state)               { return false; };
#endif
#if     defined(RTE_USB_Device_CDC_3)
__weak  void      USBD_CDC3_ACM_Initialize         (void)                         { };
__weak  void      USBD_CDC3_ACM_Uninitialize       (void)                         { };
__weak  void      USBD_CDC3_ACM_Reset              (void)                         { };
__weak  bool      USBD_CDC3_ACM_SetLineCoding      (CDC_LINE_CODING *line_coding) { return false; };
__weak  bool      USBD_CDC3_ACM_GetLineCoding      (CDC_LINE_CODING *line_coding) { return false; };
__weak  bool      USBD_CDC3_ACM_SetControlLineState(uint16_t state)               { return false; };
#endif
__weak  void    USBD_CDC_ACM_DataReceived          (int8_t instance, int32_t len)   { };
        void (* const USBD_CDC_ACM_Initialize[USBD_CDC_NUM]) (void) = 
      { 
        USBD_CDC0_ACM_Initialize
#if     defined(RTE_USB_Device_CDC_1)
      , USBD_CDC1_ACM_Initialize
#endif
#if     defined(RTE_USB_Device_CDC_2)
      , USBD_CDC2_ACM_Initialize
#endif
#if     defined(RTE_USB_Device_CDC_3)
      , USBD_CDC3_ACM_Initialize 
#endif
      };
        void (* const USBD_CDC_ACM_Uninitialize[USBD_CDC_NUM]) (void) = 
      { 
        USBD_CDC0_ACM_Uninitialize
#if     defined(RTE_USB_Device_CDC_1)
      , USBD_CDC1_ACM_Uninitialize
#endif
#if     defined(RTE_USB_Device_CDC_2)
      , USBD_CDC2_ACM_Uninitialize
#endif
#if     defined(RTE_USB_Device_CDC_3)
      , USBD_CDC3_ACM_Uninitialize 
#endif
      };
        void (* const USBD_CDC_ACM_Reset[USBD_CDC_NUM]) (void) = 
      { 
        USBD_CDC0_ACM_Reset
#if     defined(RTE_USB_Device_CDC_1)
      , USBD_CDC1_ACM_Reset
#endif
#if     defined(RTE_USB_Device_CDC_2)
      , USBD_CDC2_ACM_Reset
#endif
#if     defined(RTE_USB_Device_CDC_3)
      , USBD_CDC3_ACM_Reset 
#endif
      };
        bool (* const USBD_CDC_ACM_SetLineCoding[USBD_CDC_NUM]) (CDC_LINE_CODING *line_coding) = 
      { 
        USBD_CDC0_ACM_SetLineCoding
#if     defined(RTE_USB_Device_CDC_1)
      , USBD_CDC1_ACM_SetLineCoding
#endif
#if     defined(RTE_USB_Device_CDC_2)
      , USBD_CDC2_ACM_SetLineCoding
#endif
#if     defined(RTE_USB_Device_CDC_3)
      , USBD_CDC3_ACM_SetLineCoding 
#endif
      };
        bool (* const USBD_CDC_ACM_GetLineCoding[USBD_CDC_NUM]) (CDC_LINE_CODING *line_coding) = 
      { 
        USBD_CDC0_ACM_GetLineCoding
#if     defined(RTE_USB_Device_CDC_1)
      , USBD_CDC1_ACM_GetLineCoding
#endif
#if     defined(RTE_USB_Device_CDC_2)
      , USBD_CDC2_ACM_GetLineCoding
#endif
#if     defined(RTE_USB_Device_CDC_3)
      , USBD_CDC3_ACM_GetLineCoding 
#endif
      };
        bool (* const USBD_CDC_ACM_SetControlLineState[USBD_CDC_NUM]) (uint16_t ctrl_bmp) = 
      { 
        USBD_CDC0_ACM_SetControlLineState
#if     defined(RTE_USB_Device_CDC_1)
      , USBD_CDC1_ACM_SetControlLineState
#endif
#if     defined(RTE_USB_Device_CDC_2)
      , USBD_CDC2_ACM_SetControlLineState
#endif
#if     defined(RTE_USB_Device_CDC_3)
      , USBD_CDC3_ACM_SetControlLineState 
#endif
      };
#endif

#if (!defined(RTE_USB_Device_CLS_0) && !defined(RTE_USB_Device_CLS_1) && !defined(RTE_USB_Device_CLS_2) && !defined(RTE_USB_Device_CLS_3))
        bool    USBD_EndPoint0_Setup_CLS_ReqToDEV (int8_t device)                              { return false; }
        bool    USBD_EndPoint0_Setup_CLS_ReqToIF  (int8_t device)                              { return false; }
        bool    USBD_EndPoint0_Setup_CLS_ReqToEP  (int8_t device)                              { return false; }
        bool    USBD_EndPoint0_Out_CLS_ReqToDEV   (int8_t device)                              { return false; }
        bool    USBD_EndPoint0_Out_CLS_ReqToIF    (int8_t device)                              { return false; }
        bool    USBD_EndPoint0_Out_CLS_ReqToEP    (int8_t device)                              { return false; }
#else
extern  bool    USBD0_CLS_RequestToDevice         (bool setup);
extern  bool    USBD0_CLS_RequestToInterface      (bool setup);
extern  bool    USBD0_CLS_RequestToEndpoint       (bool setup);
#if     defined(RTE_USB_Device_CDC_1)
extern  bool    USBD1_CLS_RequestToDevice         (bool setup);
extern  bool    USBD1_CLS_RequestToInterface      (bool setup);
extern  bool    USBD1_CLS_RequestToEndpoint       (bool setup);
#endif
#if     defined(RTE_USB_Device_CDC_2)
extern  bool    USBD2_CLS_RequestToDevice         (bool setup);
extern  bool    USBD2_CLS_RequestToInterface      (bool setup);
extern  bool    USBD2_CLS_RequestToEndpoint       (bool setup);
#endif
#if     defined(RTE_USB_Device_CDC_3)
extern  bool    USBD3_CLS_ReqequestToDevice       (bool setup);
extern  bool    USBD3_CLS_RequestToInterface      (bool setup);
extern  bool    USBD3_CLS_RequestToEndpoint       (bool setup);
#endif
        bool (* const USBD_CLS_RequestToDevice[USBD_CLS_NUM]) (bool setup) = 
      { 
        USBD0_CLS_RequestToDevice
#if     defined(RTE_USB_Device_CLC_1)
      , USBD1_CLS_RequestToDevice
#endif
#if     defined(RTE_USB_Device_CLC_2)
      , USBD2_CLS_RequestToDevice
#endif
#if     defined(RTE_USB_Device_CLC_3)
      , USBD3_CLS_RequestToDevice 
#endif
      };
        bool (* const USBD_CLS_RequestToInterface[USBD_CLS_NUM]) (bool setup) = 
      { 
        USBD0_CLS_RequestToInterface
#if     defined(RTE_USB_Device_CLC_1)
      , USBD1_CLS_RequestToInterface
#endif
#if     defined(RTE_USB_Device_CLC_2)
      , USBD2_CLS_RequestToInterface
#endif
#if     defined(RTE_USB_Device_CLC_3)
      , USBD3_CLS_RequestToInterface 
#endif
      };
        bool (* const USBD_CLS_RequestToEndpoint[USBD_CLS_NUM]) (bool setup) = 
      { 
        USBD0_CLS_RequestToEndpoint
#if     defined(RTE_USB_Device_CLC_1)
      , USBD1_CLS_RequestToEndpoint
#endif
#if     defined(RTE_USB_Device_CLC_2)
      , USBD2_CLS_RequestToEndpoint
#endif
#if     defined(RTE_USB_Device_CLC_3)
      , USBD3_CLS_RequestToEndpoint 
#endif
      };
#endif

__weak usbStatus USBD_ClassInitialize (int8_t device) {
  uint8_t   instance;
  usbStatus status_ret, status;

  status_ret = usbOK;
#if (defined(RTE_USB_Device_HID_0) || defined(RTE_USB_Device_HID_1) || defined(RTE_USB_Device_HID_2) || defined(RTE_USB_Device_HID_3))
  for (instance = 0; instance < usbd_hid_num; instance++) { 
    if (usbd_hid_ptr[instance]->data_ptr != NULL) { 
      if (usbd_hid_ptr[instance]->dev_num == device) { 
        status = USBD_HID_ClassInitialize (instance);
        if (status) status_ret = status;
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_MSC_0) || defined(RTE_USB_Device_MSC_1) || defined(RTE_USB_Device_MSC_2) || defined(RTE_USB_Device_MSC_3))
  for (instance = 0; instance < usbd_msc_num; instance++) { 
    if (usbd_msc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_msc_ptr[instance]->dev_num == device) { 
        status = USBD_MSC_ClassInitialize (instance);
        if (status) status_ret = status;
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_ADC_0) || defined(RTE_USB_Device_ADC_1) || defined(RTE_USB_Device_ADC_2) || defined(RTE_USB_Device_ADC_3))
  for (instance = 0; instance < usbd_adc_num; instance++) { 
    if (usbd_adc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_adc_ptr[instance]->dev_num == device) { 
        status = USBD_ADC_ClassInitialize (instance);
        if (status) status_ret = status;
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_CDC_0) || defined(RTE_USB_Device_CDC_1) || defined(RTE_USB_Device_CDC_2) || defined(RTE_USB_Device_CDC_3))
  for (instance = 0; instance < usbd_cdc_num; instance++) { 
    if (usbd_cdc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_cdc_ptr[instance]->dev_num == device) { 
        status = USBD_CDC_ClassInitialize (instance);
        if (status) status_ret = status;
      }
    }
  }
#endif

  return status_ret;
}

__weak usbStatus USBD_ClassUninitialize (int8_t device) {
  int8_t    instance;
  usbStatus status_ret, status;

  status_ret = usbOK;
#if (defined(RTE_USB_Device_HID_0) || defined(RTE_USB_Device_HID_1) || defined(RTE_USB_Device_HID_2) || defined(RTE_USB_Device_HID_3))
  for (instance = 0; instance < usbd_hid_num; instance++) { 
    if (usbd_hid_ptr[instance]->data_ptr != NULL) { 
      if (usbd_hid_ptr[instance]->dev_num == device) { 
        status = USBD_HID_ClassUninitialize (instance);
        if (status) status_ret = status;
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_MSC_0) || defined(RTE_USB_Device_MSC_1) || defined(RTE_USB_Device_MSC_2) || defined(RTE_USB_Device_MSC_3))
  for (instance = 0; instance < usbd_msc_num; instance++) { 
    if (usbd_msc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_msc_ptr[instance]->dev_num == device) { 
        status = USBD_MSC_ClassUninitialize (instance);
        if (status) status_ret = status;
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_ADC_0) || defined(RTE_USB_Device_ADC_1) || defined(RTE_USB_Device_ADC_2) || defined(RTE_USB_Device_ADC_3))
  for (instance = 0; instance < usbd_adc_num; instance++) { 
    if (usbd_adc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_adc_ptr[instance]->dev_num == device) { 
        status = USBD_ADC_ClassUninitialize (instance);
        if (status) status_ret = status;
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_CDC_0) || defined(RTE_USB_Device_CDC_1) || defined(RTE_USB_Device_CDC_2) || defined(RTE_USB_Device_CDC_3))
  for (instance = 0; instance < usbd_cdc_num; instance++) { 
    if (usbd_cdc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_cdc_ptr[instance]->dev_num == device) { 
        status = USBD_CDC_ClassUninitialize (instance);
        if (status) status_ret = status;
      }
    }
  }
#endif

  return status_ret;
}

__weak void USBD_EventPowerOn                    (int8_t device) {};
__weak void USBD_EventPowerOff                   (int8_t device) {};
__weak void USBD_EventConnect                    (int8_t device) {};
__weak void USBD_EventDisconnect                 (int8_t device) {};
__weak void USBD_EventHighSpeed                  (int8_t device) {};
__weak void USBD_EventSuspend                    (int8_t device) {};
__weak void USBD_EventResume                     (int8_t device) {};
__weak void USBD_EventRemoteWakeup               (int8_t device) {};
__weak void USBD_EventSetInterface               (int8_t device) {};
__weak void USBD_EventSetFeature                 (int8_t device) {};
__weak void USBD_EventClearFeature               (int8_t device) {};
__weak void USBD_EventEnableRemoteWakeup         (int8_t device) {};
__weak void USBD_EventDisableRemoteWakeup        (int8_t device) {};

__weak void USBD_EventReset                      (int8_t device) {
  int8_t instance;

#if (defined(RTE_USB_Device_HID_0) || defined(RTE_USB_Device_HID_1) || defined(RTE_USB_Device_HID_2) || defined(RTE_USB_Device_HID_3))
  for (instance = 0; instance < usbd_hid_num; instance++) { 
    if (usbd_hid_ptr[instance]->data_ptr != NULL) { 
      if (usbd_hid_ptr[instance]->dev_num == device) { 
        USBD_HID_EventReset (instance);
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_MSC_0) || defined(RTE_USB_Device_MSC_1) || defined(RTE_USB_Device_MSC_2) || defined(RTE_USB_Device_MSC_3))
  for (instance = 0; instance < usbd_msc_num; instance++) { 
    if (usbd_msc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_msc_ptr[instance]->dev_num == device) { 
        USBD_MSC_EventReset (instance);
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_ADC_0) || defined(RTE_USB_Device_ADC_1) || defined(RTE_USB_Device_ADC_2) || defined(RTE_USB_Device_ADC_3))
  for (instance = 0; instance < usbd_adc_num; instance++) { 
    if (usbd_adc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_adc_ptr[instance]->dev_num == device) { 
        USBD_ADC_EventReset (instance);
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_CDC_0) || defined(RTE_USB_Device_CDC_1) || defined(RTE_USB_Device_CDC_2) || defined(RTE_USB_Device_CDC_3))
  for (instance = 0; instance < usbd_cdc_num; instance++) { 
    if (usbd_cdc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_cdc_ptr[instance]->dev_num == device) { 
        USBD_CDC_EventReset (instance);
      }
    }
  }
#endif
} 

__weak void USBD_EventSetConfiguration           (int8_t device) {
#if (defined(RTE_USB_Device_HID_0) || defined(RTE_USB_Device_HID_1) || defined(RTE_USB_Device_HID_2) || defined(RTE_USB_Device_HID_3))
  int8_t instance;

  for (instance = 0; instance < usbd_hid_num; instance++) { 
    if (usbd_hid_ptr[instance]->data_ptr != NULL) { 
      if (usbd_hid_ptr[instance]->dev_num == device) { 
        USBD_HID_EventSetConfiguration (instance);
      }
    }
  }
#endif
}

__weak void USBD_EventEndpointEnable             (int8_t device, uint8_t ep_addr) {
  int8_t instance;

#if (defined(RTE_USB_Device_HID_0) || defined(RTE_USB_Device_HID_1) || defined(RTE_USB_Device_HID_2) || defined(RTE_USB_Device_HID_3))
  for (instance = 0; instance < usbd_hid_num; instance++) { 
    if (usbd_hid_ptr[instance]->data_ptr != NULL) { 
      if (usbd_hid_ptr[instance]->dev_num == device) { 
        if (((usbd_hid_ptr[instance]->ep_int_in | 0x80) == ep_addr) ||
            ((usbd_hid_ptr[instance]->ep_int_out      ) == ep_addr)) { 
          USBD_HID_EventEndpointEnable (instance, ep_addr);
        }
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_MSC_0) || defined(RTE_USB_Device_MSC_1) || defined(RTE_USB_Device_MSC_2) || defined(RTE_USB_Device_MSC_3))
  for (instance = 0; instance < usbd_msc_num; instance++) { 
    if (usbd_msc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_msc_ptr[instance]->dev_num == device) { 
        if (((usbd_msc_ptr[instance]->ep_bulk_in | 0x80) == ep_addr) ||
            ((usbd_msc_ptr[instance]->ep_bulk_out      ) == ep_addr)) { 
          USBD_MSC_EventEndpointEnable (instance, ep_addr);
        }
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_ADC_0) || defined(RTE_USB_Device_ADC_1) || defined(RTE_USB_Device_ADC_2) || defined(RTE_USB_Device_ADC_3))
  for (instance = 0; instance < usbd_adc_num; instance++) { 
    if (usbd_adc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_adc_ptr[instance]->dev_num == device) { 
        if (((usbd_hid_ptr[instance]->ep_iso_out       ) == ep_addr)) { 
          USBD_ADC_EventEndpointEnable (instance, ep_addr);
        }
      }
    }
  }
#endif

#if (defined(RTE_USB_Device_CDC_0) || defined(RTE_USB_Device_CDC_1) || defined(RTE_USB_Device_CDC_2) || defined(RTE_USB_Device_CDC_3))
  for (instance = 0; instance < usbd_cdc_num; instance++) { 
    if (usbd_cdc_ptr[instance]->data_ptr != NULL) { 
      if (usbd_cdc_ptr[instance]->dev_num == device) { 
        if (((usbd_cdc_ptr[instance]->ep_int_in  | 0x80) == ep_addr) ||
            ((usbd_cdc_ptr[instance]->ep_bulk_in | 0x80) == ep_addr) ||
            ((usbd_cdc_ptr[instance]->ep_bulk_out      ) == ep_addr)) { 
          USBD_CDC_EventEndpointEnable (instance, ep_addr);
        }
      }
    }
  }
#endif
}

#endif


/*------------------------------------------------------------------------------
 *      USB Device Global Macros for Descriptors
 *----------------------------------------------------------------------------*/

#define WBVAL(x)                       (x & 0xFF),((x >> 8) & 0xFF)
#define B3VAL(x)                       (x & 0xFF),((x >> 8) & 0xFF),((x >> 16) & 0xFF)
#define USB_DEVICE_DESC_SIZE           (sizeof(USB_DEVICE_DESCRIPTOR))
#define USB_DEVICE_QUALI_SIZE          (sizeof(USB_DEVICE_QUALIFIER_DESCRIPTOR))
#define USB_CONFIGURATION_DESC_SIZE    (sizeof(USB_CONFIGURATION_DESCRIPTOR))
#define USB_INTERFACE_ASSOC_DESC_SIZE  (sizeof(USB_INTERFACE_ASSOCIATION_DESCRIPTOR))
#define USB_INTERFACE_DESC_SIZE        (sizeof(USB_INTERFACE_DESCRIPTOR))
#define USB_ENDPOINT_DESC_SIZE         (sizeof(USB_ENDPOINT_DESCRIPTOR))
#define USB_HID_DESC_SIZE              (sizeof(HID_DESCRIPTOR))

#define USBD_EP0(n)                               /* Endpoint 0 */                                            \
  /* Endpoint 0 */                                                                                            \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  0,                                              /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_CONTROL,                      /* bmAttributes */                                          \
  WBVAL(USBD##n##_MAX_PACKET0),                   /* wMaxPacketSize */                                        \
  0                                               /* bInterval */

#define USBD_HIDn_IAD(n,first,num_of_ifs)         /* HID: Interface Association Descriptor */                 \
  USB_INTERFACE_ASSOC_DESC_SIZE,                  /* bLength */                                               \
  USB_INTERFACE_ASSOCIATION_DESCRIPTOR_TYPE,      /* bDescriptorType */                                       \
 (first),                                         /* bFirstInterface */                                       \
 (num_of_ifs),                                    /* bInterfaceCount */                                       \
  USB_DEVICE_CLASS_HUMAN_INTERFACE,               /* bFunctionClass */                                        \
  HID_SUBCLASS_NONE,                              /* bFunctionSubclass */                                     \
  HID_PROTOCOL_NONE,                              /* bFunctionProtocol */                                     \
  USBD_HID##n##_STR,                              /* iFunction */                                             \

#define USBD_HIDn_DESC(n)                                                                                     \
  /* Interface, Alternate Setting 0, HID Class */                                                             \
  USB_INTERFACE_DESC_SIZE,                        /* bLength */                                               \
  USB_INTERFACE_DESCRIPTOR_TYPE,                  /* bDescriptorType */                                       \
  USBD_HID##n##_IF,                               /* bInterfaceNumber */                                      \
  0x00,                                           /* bAlternateSetting */                                     \
  0x01+(USBD_HID##n##_EP_INT_OUT != 0),           /* bNumEndpoints */                                         \
  USB_DEVICE_CLASS_HUMAN_INTERFACE,               /* bInterfaceClass */                                       \
  HID_SUBCLASS_NONE,                              /* bInterfaceSubClass */                                    \
  HID_PROTOCOL_NONE,                              /* bInterfaceProtocol */                                    \
  USBD_HID##n##_STR,                              /* iInterface */                                            \
                                                                                                              \
  /* HID Class Descriptor */                                                                                  \
  USB_HID_DESC_SIZE,                              /* bLength */                                               \
  HID_HID_DESCRIPTOR_TYPE,                        /* bDescriptorType */                                       \
  WBVAL(0x0111), /* 1.11 */                       /* bcdHID */                                                \
  0x00,                                           /* bCountryCode */                                          \
  0x01,                                           /* bNumDescriptors */                                       \
  HID_REPORT_DESCRIPTOR_TYPE,                     /* bDescriptorType */                                       \
  WBVAL(USBD_HID##n##_REPORT_DESC_SIZE),          /* wDescriptorLength */

#define USBD_HIDn_EP(n)                           /* HID Endpoint for Low-speed/Full-speed */                 \
  /* Endpoint, HID Interrupt In */                                                                            \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_HID##n##_EP_INT_IN),       /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_INTERRUPT,                    /* bmAttributes */                                          \
  WBVAL(USBD_HID##n##_WMAXPACKETSIZE),            /* wMaxPacketSize */                                        \
  USBD_HID##n##_BINTERVAL,                        /* bInterval */

#define USBD_HIDn_EP_INOUT(n)                     /* HID Endpoint for Low-speed/Full-speed */                 \
  /* Endpoint, HID Interrupt In */                                                                            \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_HID##n##_EP_INT_IN),       /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_INTERRUPT,                    /* bmAttributes */                                          \
  WBVAL(USBD_HID##n##_WMAXPACKETSIZE),            /* wMaxPacketSize */                                        \
  USBD_HID##n##_BINTERVAL,                        /* bInterval */                                             \
                                                                                                              \
  /* Endpoint, HID Interrupt Out */                                                                           \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_OUT(USBD_HID##n##_EP_INT_OUT),     /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_INTERRUPT,                    /* bmAttributes */                                          \
  WBVAL(USBD_HID##n##_WMAXPACKETSIZE),            /* wMaxPacketSize */                                        \
  USBD_HID##n##_BINTERVAL,                        /* bInterval */

#define USBD_HIDn_EP_HS(n)                        /* HID Endpoint for High-speed */                           \
  /* Endpoint, HID Interrupt In */                                                                            \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_HID##n##_EP_INT_IN),       /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_INTERRUPT,                    /* bmAttributes */                                          \
  WBVAL(USBD_HID##n##_HS_WMAXPACKETSIZE),         /* wMaxPacketSize */                                        \
  USBD_HID##n##_HS_BINTERVAL,                     /* bInterval */

#define USBD_HIDn_EP_INOUT_HS(n)                  /* HID Endpoint for High-speed */                           \
  /* Endpoint, HID Interrupt In */                                                                            \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_HID##n##_EP_INT_IN),       /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_INTERRUPT,                    /* bmAttributes */                                          \
  WBVAL(USBD_HID##n##_HS_WMAXPACKETSIZE),         /* wMaxPacketSize */                                        \
  USBD_HID##n##_HS_BINTERVAL,                     /* bInterval */                                             \
                                                                                                              \
  /* Endpoint, HID Interrupt Out */                                                                           \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_OUT(USBD_HID##n##_EP_INT_OUT),     /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_INTERRUPT,                    /* bmAttributes */                                          \
  WBVAL(USBD_HID##n##_HS_WMAXPACKETSIZE),         /* wMaxPacketSize */                                        \
  USBD_HID##n##_HS_BINTERVAL,                     /* bInterval */

#define USBD_MSCn_IAD(n,first,num_of_ifs)         /* MSC: Interface Association Descriptor */                 \
  USB_INTERFACE_ASSOC_DESC_SIZE,                  /* bLength */                                               \
  USB_INTERFACE_ASSOCIATION_DESCRIPTOR_TYPE,      /* bDescriptorType */                                       \
 (first),                                         /* bFirstInterface */                                       \
 (num_of_ifs),                                    /* bInterfaceCount */                                       \
  USB_DEVICE_CLASS_STORAGE,                       /* bFunctionClass */                                        \
  MSC_SUBCLASS_SCSI,                              /* bFunctionSubclass */                                     \
  MSC_PROTOCOL_BULK_ONLY,                         /* bFunctionProtocol */                                     \
  USBD_MSC##n##_STR,                              /* iFunction */                                             \

#define USBD_MSCn_DESC(n)                                                                                     \
  /* Interface, Alternate Setting 0, MSC Class */                                                             \
  USB_INTERFACE_DESC_SIZE,                        /* bLength */                                               \
  USB_INTERFACE_DESCRIPTOR_TYPE,                  /* bDescriptorType */                                       \
  USBD_MSC##n##_IF,                               /* bInterfaceNumber */                                      \
  0x00,                                           /* bAlternateSetting */                                     \
  0x02,                                           /* bNumEndpoints */                                         \
  USB_DEVICE_CLASS_STORAGE,                       /* bInterfaceClass */                                       \
  MSC_SUBCLASS_SCSI,                              /* bInterfaceSubClass */                                    \
  MSC_PROTOCOL_BULK_ONLY,                         /* bInterfaceProtocol */                                    \
  USBD_MSC##n##_STR,                              /* iInterface */

#define USBD_MSCn_EP(n)                           /* MSC Endpoints for Low-speed/Full-speed */                \
  /* Endpoint, EP Bulk IN */                                                                                  \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_MSC##n##_EP_BULK_IN),      /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_BULK,                         /* bmAttributes */                                          \
  WBVAL(USBD_MSC##n##_WMAXPACKETSIZE),            /* wMaxPacketSize */                                        \
  0x00,                                           /* bInterval: ignore for Bulk transfer */                   \
                                                                                                              \
  /* Endpoint, EP Bulk OUT */                                                                                 \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_OUT(USBD_MSC##n##_EP_BULK_OUT),    /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_BULK,                         /* bmAttributes */                                          \
  WBVAL(USBD_MSC##n##_WMAXPACKETSIZE),            /* wMaxPacketSize */                                        \
  0x00,                                           /* bInterval: ignore for Bulk transfer */

#define USBD_MSCn_EP_HS(n)                        /* MSC Endpoints for High-speed */                          \
  /* Endpoint, EP Bulk IN */                                                                                  \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_MSC##n##_EP_BULK_IN),      /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_BULK,                         /* bmAttributes */                                          \
  WBVAL(USBD_MSC##n##_HS_WMAXPACKETSIZE),         /* wMaxPacketSize */                                        \
  USBD_MSC##n##_HS_BINTERVAL,                     /* bInterval */                                             \
                                                                                                              \
  /* Endpoint, EP Bulk OUT */                                                                                 \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_OUT(USBD_MSC##n##_EP_BULK_OUT),    /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_BULK,                         /* bmAttributes */                                          \
  WBVAL(USBD_MSC##n##_HS_WMAXPACKETSIZE),         /* wMaxPacketSize */                                        \
  USBD_MSC##n##_HS_BINTERVAL,                     /* bInterval */

#define USBD_ADCn_IAD(n,first,num_of_ifs)         /* ADC: Interface Association Descriptor */                 \
  USB_INTERFACE_ASSOC_DESC_SIZE,                  /* bLength */                                               \
  USB_INTERFACE_ASSOCIATION_DESCRIPTOR_TYPE,      /* bDescriptorType */                                       \
 (first),                                         /* bFirstInterface */                                       \
 (num_of_ifs),                                    /* bInterfaceCount */                                       \
  USB_DEVICE_CLASS_AUDIO,                         /* bFunctionClass */                                        \
  AUDIO_SUBCLASS_AUDIOCONTROL,                    /* bFunctionSubclass */                                     \
  AUDIO_PROTOCOL_UNDEFINED,                       /* bFunctionProtocol */                                     \
  USBD_ADC##n##_CIF_STR,                          /* iFunction */                                             \

#define USBD_ADCn_DESC(n)                                                                                     \
  /* Interface, Alternate Setting 0, Audio Control */                                                         \
  USB_INTERFACE_DESC_SIZE,                        /* bLength */                                               \
  USB_INTERFACE_DESCRIPTOR_TYPE,                  /* bDescriptorType */                                       \
  USBD_ADC##n##_CIF,                              /* bInterfaceNumber */                                      \
  0x00,                                           /* bAlternateSetting */                                     \
  0x00,                                           /* bNumEndpoints */                                         \
  USB_DEVICE_CLASS_AUDIO,                         /* bInterfaceClass */                                       \
  AUDIO_SUBCLASS_AUDIOCONTROL,                    /* bInterfaceSubClass */                                    \
  AUDIO_PROTOCOL_UNDEFINED,                       /* bInterfaceProtocol */                                    \
  USBD_ADC##n##_CIF_STR,                          /* iInterface */                                            \
  /* Audio Control Interface */                                                                               \
  AUDIO_CONTROL_INTERFACE_DESC_SZ(1),             /* bLength */                                               \
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,                /* bDescriptorType */                                       \
  AUDIO_CONTROL_HEADER,                           /* bDescriptorSubtype */                                    \
  WBVAL(0x0100), /* 1.00 */                       /* bcdADC */                                                \
  WBVAL(                                          /* wTotalLength */                                          \
    AUDIO_CONTROL_INTERFACE_DESC_SZ(1) +                                                                      \
    AUDIO_INPUT_TERMINAL_DESC_SIZE     +                                                                      \
    AUDIO_FEATURE_UNIT_DESC_SZ(1,1)    +                                                                      \
    AUDIO_OUTPUT_TERMINAL_DESC_SIZE                                                                           \
  ),                                                                                                          \
  0x01,                                           /* bInCollection */                                         \
  0x01,                                           /* baInterfaceNr */                                         \
                                                                                                              \
  /* Audio Input Terminal */                                                                                  \
  AUDIO_INPUT_TERMINAL_DESC_SIZE,                 /* bLength */                                               \
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,                /* bDescriptorType */                                       \
  AUDIO_CONTROL_INPUT_TERMINAL,                   /* bDescriptorSubtype */                                    \
  0x01,                                           /* bTerminalID */                                           \
  WBVAL(AUDIO_TERMINAL_USB_STREAMING),            /* wTerminalType */                                         \
  0x00,                                           /* bAssocTerminal */                                        \
  0x01,                                           /* bNrChannels */                                           \
  WBVAL(AUDIO_CHANNEL_M),                         /* wChannelConfig */                                        \
  0x00,                                           /* iChannelNames */                                         \
  0x00,                                           /* iTerminal */                                             \
                                                                                                              \
  /* Audio Feature Unit */                                                                                    \
  AUDIO_FEATURE_UNIT_DESC_SZ(1,1),                /* bLength */                                               \
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,                /* bDescriptorType */                                       \
  AUDIO_CONTROL_FEATURE_UNIT,                     /* bDescriptorSubtype */                                    \
  0x02,                                           /* bUnitID */                                               \
  0x01,                                           /* bSourceID */                                             \
  0x01,                                           /* bControlSize */                                          \
  AUDIO_CONTROL_MUTE |                                                                                        \
  AUDIO_CONTROL_VOLUME,                           /* bmaControls(0) */                                        \
  0x00,                                           /* bmaControls(1) */                                        \
  0x00,                                           /* iTerminal */                                             \
                                                                                                              \
  /* Audio Output Terminal */                                                                                 \
  AUDIO_OUTPUT_TERMINAL_DESC_SIZE,                /* bLength */                                               \
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,                /* bDescriptorType */                                       \
  AUDIO_CONTROL_OUTPUT_TERMINAL,                  /* bDescriptorSubtype */                                    \
  0x03,                                           /* bTerminalID */                                           \
  WBVAL(AUDIO_TERMINAL_SPEAKER),                  /* wTerminalType */                                         \
  0x00,                                           /* bAssocTerminal */                                        \
  0x02,                                           /* bSourceID */                                             \
  0x00,                                           /* iTerminal */                                             \
                                                                                                              \
  /* Interface, Alternate Setting 0, Audio Streaming - Zero Bandwith */                                       \
  USB_INTERFACE_DESC_SIZE,                        /* bLength */                                               \
  USB_INTERFACE_DESCRIPTOR_TYPE,                  /* bDescriptorType */                                       \
  USBD_ADC##n##_SIF,                              /* bInterfaceNumber */                                      \
  0x00,                                           /* bAlternateSetting */                                     \
  0x00,                                           /* bNumEndpoints */                                         \
  USB_DEVICE_CLASS_AUDIO,                         /* bInterfaceClass */                                       \
  AUDIO_SUBCLASS_AUDIOSTREAMING,                  /* bInterfaceSubClass */                                    \
  AUDIO_PROTOCOL_UNDEFINED,                       /* bInterfaceProtocol */                                    \
  USBD_ADC##n##_SIF1_STR,                         /* iInterface */                                            \
                                                                                                              \
  /* Interface, Alternate Setting 1, Audio Streaming - Operational */                                         \
  USB_INTERFACE_DESC_SIZE,                        /* bLength */                                               \
  USB_INTERFACE_DESCRIPTOR_TYPE,                  /* bDescriptorType */                                       \
  USBD_ADC##n##_SIF,                              /* bInterfaceNumber */                                      \
  0x01,                                           /* bAlternateSetting */                                     \
  0x01,                                           /* bNumEndpoints */                                         \
  USB_DEVICE_CLASS_AUDIO,                         /* bInterfaceClass */                                       \
  AUDIO_SUBCLASS_AUDIOSTREAMING,                  /* bInterfaceSubClass */                                    \
  AUDIO_PROTOCOL_UNDEFINED,                       /* bInterfaceProtocol */                                    \
  USBD_ADC##n##_SIF2_STR,                         /* iInterface */                                            \
                                                                                                              \
  /* Audio Streaming Interface */                                                                             \
  AUDIO_STREAMING_INTERFACE_DESC_SIZE,            /* bLength */                                               \
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,                /* bDescriptorType */                                       \
  AUDIO_STREAMING_GENERAL,                        /* bDescriptorSubtype */                                    \
  0x01,                                           /* bTerminalLink */                                         \
  0x01,                                           /* bDelay */                                                \
  WBVAL(AUDIO_FORMAT_PCM),                        /* wFormatTag */                                            \
                                                                                                              \
  /* Audio Type I Format */                                                                                   \
  AUDIO_FORMAT_TYPE_I_DESC_SZ(1),                 /* bLength */                                               \
  AUDIO_INTERFACE_DESCRIPTOR_TYPE,                /* bDescriptorType */                                       \
  AUDIO_STREAMING_FORMAT_TYPE,                    /* bDescriptorSubtype */                                    \
  AUDIO_FORMAT_TYPE_I,                            /* bFormatType */                                           \
  0x01,                                           /* bNrChannels */                                           \
  USBD_ADC##n##_BSUBFRAMESIZE,                    /* bSubFrameSize */                                         \
  USBD_ADC##n##_BBITRESOLUTION,                   /* bBitResolution */                                        \
  0x01,                                           /* bSamFreqType */                                          \
  B3VAL(USBD_ADC##n##_TSAM_FREQ),                  /* tSamFreq */

#define USBD_ADCn_EP(n)                           /* ADC Endpoints for Low-speed/Full-speed */                \
  /* Endpoint, EP ISO OUT - Standard Descriptor */                                                            \
  AUDIO_STANDARD_ENDPOINT_DESC_SIZE,              /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_OUT(USBD_ADC##n##_EP_ISO_OUT),     /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_ISOCHRONOUS,                  /* bmAttributes */                                          \
  WBVAL(USBD_ADC##n##_WMAXPACKETSIZE),            /* wMaxPacketSize */                                        \
  USBD_ADC##n##_BINTERVAL,                        /* bInterval */                                             \
  0x00,                                           /* bRefresh */                                              \
  0x00,                                           /* bSynchAddress */                                         \
                                                                                                              \
  /* Endpoint - Audio Streaming */                                                                            \
  AUDIO_STREAMING_ENDPOINT_DESC_SIZE,             /* bLength */                                               \
  AUDIO_ENDPOINT_DESCRIPTOR_TYPE,                 /* bDescriptorType */                                       \
  AUDIO_ENDPOINT_GENERAL,                         /* bDescriptor */                                           \
  0x00,                                           /* bmAttributes */                                          \
  0x00,                                           /* bLockDelayUnits */                                       \
  WBVAL(0x0000),                                  /* wLockDelay */

#define USBD_ADCn_EP_HS(n)                        /* ADC Endpoints for High-speed */                          \
  /* Endpoint, EP ISO OUT - Standard Descriptor */                                                            \
  AUDIO_STANDARD_ENDPOINT_DESC_SIZE,              /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_OUT(USBD_ADC##n##_EP_ISO_OUT),     /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_ISOCHRONOUS,                  /* bmAttributes */                                          \
  WBVAL(USBD_ADC##n##_HS_WMAXPACKETSIZE),         /* wMaxPacketSize */                                        \
  USBD_ADC##n##_BINTERVAL,                        /* bInterval */                                             \
  0x00,                                           /* bRefresh */                                              \
  0x00,                                           /* bSynchAddress */                                         \
                                                                                                              \
  /* Endpoint - Audio Streaming */                                                                            \
  AUDIO_STREAMING_ENDPOINT_DESC_SIZE,             /* bLength */                                               \
  AUDIO_ENDPOINT_DESCRIPTOR_TYPE,                 /* bDescriptorType */                                       \
  AUDIO_ENDPOINT_GENERAL,                         /* bDescriptor */                                           \
  0x00,                                           /* bmAttributes */                                          \
  0x00,                                           /* bLockDelayUnits */                                       \
  WBVAL(0x0000),                                  /* wLockDelay */

#define USBD_CDCn_IAD(n,first,num_of_ifs)         /* CDC: Interface Association Descriptor */                 \
  USB_INTERFACE_ASSOC_DESC_SIZE,                  /* bLength */                                               \
  USB_INTERFACE_ASSOCIATION_DESCRIPTOR_TYPE,      /* bDescriptorType */                                       \
 (first),                                         /* bFirstInterface */                                       \
 (num_of_ifs),                                    /* bInterfaceCount */                                       \
  CDC_COMMUNICATION_INTERFACE_CLASS,              /* bFunctionClass    (Communication Class) */               \
  CDC_ABSTRACT_CONTROL_MODEL,                     /* bFunctionSubclass (Abstract Control Model) */            \
  0x01,                                           /* bFunctionProtocol (V.25ter, Common AT commands) */       \
  USBD_CDC##n##_CIF_STR,                          /* iFunction */                                             \

#define USBD_CDCn_DESC_IF0(n)                                                                                 \
  /* Interface, Alternate Setting 0, CDC Class */                                                             \
  USB_INTERFACE_DESC_SIZE,                        /* bLength */                                               \
  USB_INTERFACE_DESCRIPTOR_TYPE,                  /* bDescriptorType */                                       \
  USBD_CDC##n##_CIF,                              /* bInterfaceNumber: Number of Interface */                 \
  0x00,                                           /* bAlternateSetting: Alternate setting */                  \
  0x01,                                           /* bNumEndpoints: One endpoint used */                      \
  CDC_COMMUNICATION_INTERFACE_CLASS,              /* bInterfaceClass: Communication Interface Class */        \
  CDC_ABSTRACT_CONTROL_MODEL,                     /* bInterfaceSubClass: Abstract Control Model */            \
  0x00,                                           /* bInterfaceProtocol: no protocol used */                  \
  USBD_CDC##n##_CIF_STR,                          /* iInterface: */                                           \
                                                                                                              \
  /* Header Functional Descriptor */                                                                          \
  0x05,                                           /* bLength: Endpoint Descriptor size */                     \
  CDC_CS_INTERFACE,                               /* bDescriptorType: CS_INTERFACE */                         \
  CDC_HEADER,                                     /* bDescriptorSubtype: Header Func Desc */                  \
  WBVAL(CDC_V1_10), /* 1.10 */                    /* bcdCDC */                                                \
  /* Call Management Functional Descriptor */                                                                 \
  0x05,                                           /* bFunctionLength */                                       \
  CDC_CS_INTERFACE,                               /* bDescriptorType: CS_INTERFACE */                         \
  CDC_CALL_MANAGEMENT,                            /* bDescriptorSubtype: Call Management Func Desc */         \
  0x01,                                           /* bmCapabilities: device handles call management */        \
  0x01,                                           /* bDataInterface: CDC data IF ID */                        \
  /* Abstract Control Management Functional Descriptor */                                                     \
  0x04,                                           /* bFunctionLength */                                       \
  CDC_CS_INTERFACE,                               /* bDescriptorType: CS_INTERFACE */                         \
  CDC_ABSTRACT_CONTROL_MANAGEMENT,                /* bDescriptorSubtype: Abstract Control Management desc */  \
  0x02,                                           /* bmCapabilities: SET_LINE_CODING,                         \
                                                                     GET_LINE_CODING,                         \
                                                                     SET_CONTROL_LINE_STATE supported */      \
  /* Union Functional Descriptor */                                                                           \
  0x05,                                           /* bFunctionLength */                                       \
  CDC_CS_INTERFACE,                               /* bDescriptorType: CS_INTERFACE */                         \
  CDC_UNION,                                      /* bDescriptorSubtype: Union func desc */                   \
  USBD_CDC##n##_CIF,                              /* bMasterInterface: Comm class interface is master */      \
  USBD_CDC##n##_DIF,                              /* bSlaveInterface0: Data class interface is slave 0 */

#define USBD_CDCn_EP_IF0(n)                       /* CDC Endpoints for Interface 0 for Low-speed/Full-speed */\
  /* Endpoint, EP Interrupt IN */                 /* event notification (optional) */                         \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_CDC##n##_EP_INT_IN),       /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_INTERRUPT,                    /* bmAttributes */                                          \
  WBVAL(USBD_CDC##n##_WMAXPACKETSIZE),            /* wMaxPacketSize */                                        \
  USBD_CDC##n##_BINTERVAL,                        /* bInterval */

#define USBD_CDCn_EP_IF0_HS(n)                    /* CDC Endpoints for Interface 0 for High-speed */          \
  /* Endpoint, EP Interrupt IN */                 /* event notification (optional) */                         \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_CDC##n##_EP_INT_IN),       /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_INTERRUPT,                    /* bmAttributes */                                          \
  WBVAL(USBD_CDC##n##_HS_WMAXPACKETSIZE),         /* wMaxPacketSize */                                        \
  USBD_CDC##n##_HS_BINTERVAL,                     /* bInterval */

#define USBD_CDCn_DESC_IF1(n)                                                                                 \
  /* Interface, Alternate Setting 0, Data class interface descriptor*/                                        \
  USB_INTERFACE_DESC_SIZE,                        /* bLength */                                               \
  USB_INTERFACE_DESCRIPTOR_TYPE,                  /* bDescriptorType */                                       \
  USBD_CDC##n##_DIF,                              /* bInterfaceNumber: Number of Interface */                 \
  0x00,                                           /* bAlternateSetting: no alternate setting */               \
  0x02,                                           /* bNumEndpoints: two endpoints used */                     \
  CDC_DATA_INTERFACE_CLASS,                       /* bInterfaceClass: Data Interface Class */                 \
  0x00,                                           /* bInterfaceSubClass: no subclass available */             \
  0x00,                                           /* bInterfaceProtocol: no protocol used */                  \
  USBD_CDC##n##_DIF_STR,                          /* iInterface */

#define USBD_CDCn_EP_IF1(n)                       /* CDC Endpoints for Interface 1 for Low-speed/Full-speed */\
  /* Endpoint, EP Bulk OUT */                                                                                 \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_OUT(USBD_CDC##n##_EP_BULK_OUT),    /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_BULK,                         /* bmAttributes */                                          \
  WBVAL(USBD_CDC##n##_WMAXPACKETSIZE1),           /* wMaxPacketSize */                                        \
  0x00,                                           /* bInterval: ignore for Bulk transfer */                   \
                                                                                                              \
  /* Endpoint, EP Bulk IN */                                                                                  \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_CDC##n##_EP_BULK_IN),      /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_BULK,                         /* bmAttributes */                                          \
  WBVAL(USBD_CDC##n##_WMAXPACKETSIZE1),           /* wMaxPacketSize */                                        \
  0x00,                                           /* bInterval: ignore for Bulk transfer */

#define USBD_CDCn_EP_IF1_HS(n)                    /* CDC Endpoints for Interface 1 for High-speed */          \
  /* Endpoint, EP Bulk OUT */                                                                                 \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_OUT(USBD_CDC##n##_EP_BULK_OUT),    /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_BULK,                         /* bmAttributes */                                          \
  WBVAL(USBD_CDC##n##_HS_WMAXPACKETSIZE1),        /* wMaxPacketSize */                                        \
  USBD_CDC##n##_HS_BINTERVAL1,                    /* bInterval */                                             \
                                                                                                              \
  /* Endpoint, EP Bulk IN */                                                                                  \
  USB_ENDPOINT_DESC_SIZE,                         /* bLength */                                               \
  USB_ENDPOINT_DESCRIPTOR_TYPE,                   /* bDescriptorType */                                       \
  USB_ENDPOINT_IN(USBD_CDC##n##_EP_BULK_IN),      /* bEndpointAddress */                                      \
  USB_ENDPOINT_TYPE_BULK,                         /* bmAttributes */                                          \
  WBVAL(USBD_CDC##n##_HS_WMAXPACKETSIZE1),        /* wMaxPacketSize */                                        \
  USBD_CDC##n##_HS_BINTERVAL1,                    /* bInterval */

/* USB Device Create String Descriptor */
#define USBD_STR_DEF(n)            \
  struct {                         \
    uint8_t  len;                  \
    uint8_t  type;                 \
    uint16_t str[sizeof(##n)/2-1]; \
  } desc##n                      

#define USBD_STR_VAL(n)                 \
 { sizeof(##n), USB_STRING_DESCRIPTOR_TYPE, ##n }  


/*------------------------------------------------------------------------------
 *      USB HID 0..3 Device Descriptors
 *----------------------------------------------------------------------------*/

#if     defined(RTE_USB_Device_HID_0)
#define USBD_HID0_DESC_NUM              1
#else
#define USBD_HID0_DESC_NUM              0
#endif
#if     defined(RTE_USB_Device_HID_1)
#define USBD_HID1_DESC_NUM              1
#else
#define USBD_HID1_DESC_NUM              0
#endif
#if     defined(RTE_USB_Device_HID_2)
#define USBD_HID2_DESC_NUM              1
#else
#define USBD_HID2_DESC_NUM              0
#endif
#if     defined(RTE_USB_Device_HID_3)
#define USBD_HID3_DESC_NUM              1
#else
#define USBD_HID3_DESC_NUM              0
#endif

#define USBD_HID_DESC_NUM              (USBD_HID0_DESC_NUM + USBD_HID1_DESC_NUM + USBD_HID2_DESC_NUM + USBD_HID3_DESC_NUM)

#if    (defined(RTE_USB_Device_HID_0) || defined(RTE_USB_Device_HID_1) || defined(RTE_USB_Device_HID_2) || defined(RTE_USB_Device_HID_3))
#if    (defined(RTE_USB_Device_HID_0)      &&  \
      ((defined(USBD0_PORT) && USBD0_HID0) ||  \
       (defined(USBD1_PORT) && USBD1_HID0) ||  \
       (defined(USBD2_PORT) && USBD2_HID0) ||  \
       (defined(USBD3_PORT) && USBD3_HID0)))
/* USB Device 0 HID 0 Report Descriptor */
__weak \
const uint8_t usbd_hid0_report_descriptor[] = {
  HID_UsagePageVendor( 0x00                         ),
  HID_Usage          ( 0x01                         ),
  HID_Collection     ( HID_Application              ),
    HID_LogicalMin   ( 0                            ), /* value range: 0 - 0xFF */
    HID_LogicalMaxS  ( 0xFF                         ),
    HID_ReportSize   ( 8                            ), /* 8 bits */
#if (USBD_HID0_IN_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID0_IN_REPORT_MAX_SZ   ),
#else
    HID_ReportCount  ( USBD_HID0_IN_REPORT_MAX_SZ   ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Input        ( HID_Data | HID_Variable | HID_Absolute ),
#if (USBD_HID0_OUT_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID0_OUT_REPORT_MAX_SZ  ),
#else
    HID_ReportCount  ( USBD_HID0_OUT_REPORT_MAX_SZ  ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Output       ( HID_Data | HID_Variable | HID_Absolute ),
#if (USBD_HID0_FEAT_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID0_FEAT_REPORT_MAX_SZ ),
#else
    HID_ReportCount  ( USBD_HID0_FEAT_REPORT_MAX_SZ ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Feature      ( HID_Data | HID_Variable | HID_Absolute ),
  HID_EndCollection,
};

#define USBD_HID0_REPORT_DESC_SIZE     (sizeof(usbd_hid0_report_descriptor))

usbd_hid_desc_t usbd_hid0_desc = {     (uint8_t *)usbd_hid0_report_descriptor, USBD_HID0_REPORT_DESC_SIZE, USBD_HID0_DESC_OFS };
#endif

#if    (defined(RTE_USB_Device_HID_1)      &&  \
      ((defined(USBD0_PORT) && USBD0_HID1) ||  \
       (defined(USBD1_PORT) && USBD1_HID1) ||  \
       (defined(USBD2_PORT) && USBD2_HID1) ||  \
       (defined(USBD3_PORT) && USBD3_HID1)))
/* USB Device HID 1 Report Descriptor */
__weak \
const uint8_t usbd_hid1_report_descriptor[] = {
  HID_UsagePageVendor( 0x00                         ),
  HID_Usage          ( 0x01                         ),
  HID_Collection     ( HID_Application              ),
    HID_LogicalMin   ( 0                            ), /* value range: 0 - 0xFF */
    HID_LogicalMaxS  ( 0xFF                         ),
    HID_ReportSize   ( 8                            ), /* 8 bits */
#if (USBD_HID1_IN_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID1_IN_REPORT_MAX_SZ   ),
#else
    HID_ReportCount  ( USBD_HID1_IN_REPORT_MAX_SZ   ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Input        ( HID_Data | HID_Variable | HID_Absolute ),
#if (USBD_HID1_OUT_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID1_OUT_REPORT_MAX_SZ  ),
#else
    HID_ReportCount  ( USBD_HID1_OUT_REPORT_MAX_SZ  ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Output       ( HID_Data | HID_Variable | HID_Absolute ),
#if (USBD_HID1_FEAT_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID1_FEAT_REPORT_MAX_SZ ),
#else
    HID_ReportCount  ( USBD_HID1_FEAT_REPORT_MAX_SZ ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Feature      ( HID_Data | HID_Variable | HID_Absolute ),
  HID_EndCollection,
};

#define USBD_HID1_REPORT_DESC_SIZE     (sizeof(usbd_hid1_report_descriptor))

usbd_hid_desc_t usbd_hid1_desc = {     (uint8_t *)usbd_hid1_report_descriptor, USBD_HID1_REPORT_DESC_SIZE, USBD_HID1_DESC_OFS };
#endif

#if    (defined(RTE_USB_Device_HID_2)      &&  \
      ((defined(USBD0_PORT) && USBD0_HID2) ||  \
       (defined(USBD1_PORT) && USBD1_HID2) ||  \
       (defined(USBD2_PORT) && USBD2_HID2) ||  \
       (defined(USBD3_PORT) && USBD3_HID2)))
/* USB Device HID 2 Report Descriptor */
__weak \
const uint8_t usbd_hid2_report_descriptor[] = {
  HID_UsagePageVendor( 0x00                         ),
  HID_Usage          ( 0x01                         ),
  HID_Collection     ( HID_Application              ),
    HID_LogicalMin   ( 0                            ), /* value range: 0 - 0xFF */
    HID_LogicalMaxS  ( 0xFF                         ),
    HID_ReportSize   ( 8                            ), /* 8 bits */
#if (USBD_HID2_IN_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID2_IN_REPORT_MAX_SZ   ),
#else
    HID_ReportCount  ( USBD_HID2_IN_REPORT_MAX_SZ   ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Input        ( HID_Data | HID_Variable | HID_Absolute ),
#if (USBD_HID2_OUT_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID2_OUT_REPORT_MAX_SZ  ),
#else
    HID_ReportCount  ( USBD_HID2_OUT_REPORT_MAX_SZ  ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Output       ( HID_Data | HID_Variable | HID_Absolute ),
#if (USBD_HID2_FEAT_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID2_FEAT_REPORT_MAX_SZ ),
#else
    HID_ReportCount  ( USBD_HID2_FEAT_REPORT_MAX_SZ ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Feature      ( HID_Data | HID_Variable | HID_Absolute ),
  HID_EndCollection,
};

#define USBD_HID2_REPORT_DESC_SIZE     (sizeof(usbd_hid2_report_descriptor))

usbd_hid_desc_t usbd_hid2_desc = {     (uint8_t *)usbd_hid2_report_descriptor, USBD_HID2_REPORT_DESC_SIZE, USBD_HID2_DESC_OFS };
#endif

#if    (defined(RTE_USB_Device_HID_3)      &&  \
      ((defined(USBD0_PORT) && USBD0_HID3) ||  \
       (defined(USBD1_PORT) && USBD1_HID3) ||  \
       (defined(USBD2_PORT) && USBD2_HID3) ||  \
       (defined(USBD3_PORT) && USBD3_HID3)))
/* USB Device HID 3 Report Descriptor */
__weak \
const uint8_t usbd_hid3_report_descriptor[] = {
  HID_UsagePageVendor( 0x00                         ),
  HID_Usage          ( 0x01                         ),
  HID_Collection     ( HID_Application              ),
    HID_LogicalMin   ( 0                            ), /* value range: 0 - 0xFF */
    HID_LogicalMaxS  ( 0xFF                         ),
    HID_ReportSize   ( 8                            ), /* 8 bits */
#if (USBD_HID3_IN_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID3_IN_REPORT_MAX_SZ   ),
#else
    HID_ReportCount  ( USBD_HID3_IN_REPORT_MAX_SZ   ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Input        ( HID_Data | HID_Variable | HID_Absolute ),
#if (USBD_HID3_OUT_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID3_OUT_REPORT_MAX_SZ  ),
#else
    HID_ReportCount  ( USBD_HID3_OUT_REPORT_MAX_SZ  ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Output       ( HID_Data | HID_Variable | HID_Absolute ),
#if (USBD_HID3_FEAT_REPORT_MAX_SZ > 255)
    HID_ReportCountS ( USBD_HID3_FEAT_REPORT_MAX_SZ ),
#else
    HID_ReportCount  ( USBD_HID3_FEAT_REPORT_MAX_SZ ),
#endif
    HID_Usage        ( 0x01                         ),
    HID_Feature      ( HID_Data | HID_Variable | HID_Absolute ),
  HID_EndCollection,
};

#define USBD_HID3_REPORT_DESC_SIZE     (sizeof(usbd_hid3_report_descriptor))

usbd_hid_desc_t usbd_hid3_desc = {     (uint8_t *)usbd_hid3_report_descriptor, USBD_HID3_REPORT_DESC_SIZE, USBD_HID3_DESC_OFS };
#endif
#endif

#if    (defined(LAST_DEVICE_COMPILE) && (USBD_HID_NUM))

#if     defined(RTE_USB_Device_HID_0)
extern  usbd_hid_desc_t usbd_hid0_desc;
#endif
#if     defined(RTE_USB_Device_HID_1)
extern  usbd_hid_desc_t usbd_hid1_desc;
#endif
#if     defined(RTE_USB_Device_HID_2)
extern  usbd_hid_desc_t usbd_hid2_desc;
#endif
#if     defined(RTE_USB_Device_HID_3)
extern  usbd_hid_desc_t usbd_hid3_desc;
#endif

usbd_hid_desc_t * const usbd_hid_desc_ptr[USBD_HID_DESC_NUM]  = { 
#if     defined(RTE_USB_Device_HID_0)
                                       &usbd_hid0_desc
#endif
#if     defined(RTE_USB_Device_HID_1)
                                     , &usbd_hid1_desc
#endif
#if     defined(RTE_USB_Device_HID_2)
                                     , &usbd_hid2_desc
#endif
#if     defined(RTE_USB_Device_HID_3)
                                     , &usbd_hid3_desc
#endif
};
#endif


/*------------------------------------------------------------------------------
 *      USB Device 0 Descriptors
 *----------------------------------------------------------------------------*/

#if    (defined(USBD0_PORT) && !defined(USBD0_DESC_DEF))
#define USBD0_DESC_DEF                  1

/* USB Device 0 Endpoint 0 Descriptor */
__weak \
const uint8_t usbd0_ep0_descriptor[] = { USBD_EP0(0) };

/* USB Device 0 Standard Descriptor */
__weak \
const uint8_t usbd0_device_descriptor[] = {
  USB_DEVICE_DESC_SIZE,                 /* bLength */
  USB_DEVICE_DESCRIPTOR_TYPE,           /* bDescriptorType */
  WBVAL(USBD0_BDC_USB),                 /* bcdUSB */
#if (USBD0_USE_IAD)
  USB_DEVICE_CLASS_MISCELLANEOUS,       /* bDeviceClass */
  0x02,                                 /* bDeviceSubClass */
  0x01,                                 /* bDeviceProtocol */
#else
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
#endif
  USBD0_MAX_PACKET0,                    /* bMaxPacketSize0 */
  WBVAL(USBD0_DEV_DESC_IDVENDOR),       /* idVendor */
  WBVAL(USBD0_DEV_DESC_IDPRODUCT),      /* idProduct */
  WBVAL(USBD0_DEV_DESC_BCDDEVICE),      /* bcdDevice */
  0x01,                                 /* iManufacturer */
  0x02,                                 /* iProduct */
  0x03*USBD0_STR_DESC_SER_EN,           /* iSerialNumber */
  0x01                                  /* bNumConfigurations: one possible configuration*/
};

/* USB Device 0 Configuration Descriptor (for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd0_config_descriptor_fs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD0_WTOTALLENGTH),            /* wTotalLength */
  USBD0_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD0_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD0_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD0_ADC0)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP                         (0)
#endif
#if (USBD0_ADC1)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP                         (1)
#endif
#if (USBD0_ADC2)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP                         (2)
#endif
#if (USBD0_ADC3)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP                         (3)
#endif

#if (USBD0_CDC0)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0                     (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1                     (0)
#endif
#if (USBD0_CDC1)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0                     (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1                     (1)
#endif
#if (USBD0_CDC2)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0                     (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1                     (2)
#endif
#if (USBD0_CDC3)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0                     (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1                     (3)
#endif

#if (USBD0_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD0_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (0)
#else
  USBD_HIDn_EP                         (0)
#endif
#endif
#if (USBD0_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD0_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (1)
#else
  USBD_HIDn_EP                         (1)
#endif
#endif
#if (USBD0_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD0_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (2)
#else
  USBD_HIDn_EP                         (2)
#endif
#endif
#if (USBD0_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD0_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (3)
#else
  USBD_HIDn_EP                         (3)
#endif
#endif

#if (USBD0_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP                         (0)
#endif
#if (USBD0_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP                         (1)
#endif
#if (USBD0_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP                         (2)
#endif
#if (USBD0_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP                         (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#if (!USBD0_HS)                         /* If High-speed not enabled, declare dummy descriptors for High-speed */
__weak \
const uint8_t usbd0_device_qualifier_fs[]              = { 0 };
__weak \
const uint8_t usbd0_device_qualifier_hs[]              = { 0 };
__weak \
const uint8_t usbd0_config_descriptor_hs[]             = { 0 };
__weak \
const uint8_t usbd0_other_speed_config_descriptor_fs[] = { 0 };
__weak \
const uint8_t usbd0_other_speed_config_descriptor_hs[] = { 0 };

#else
/* USB Device 0 Qualifier Descriptor (in Full Speed for High Speed) */
__weak \
const uint8_t usbd0_device_qualifier_fs[] = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD0_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 0 Qualifier Descriptor (in High Speed for Full Speed) */
__weak \
const uint8_t usbd0_device_qualifier_hs[] = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD0_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 0 Configuration Descriptor (for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd0_config_descriptor_hs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD0_WTOTALLENGTH),            /* wTotalLength */
  USBD0_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD0_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD0_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD0_ADC0)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP_HS                      (0)
#endif
#if (USBD0_ADC1)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP_HS                      (1)
#endif
#if (USBD0_ADC2)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP_HS                      (2)
#endif
#if (USBD0_ADC3)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP_HS                      (3)
#endif

#if (USBD0_CDC0)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0_HS                  (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1_HS                  (0)
#endif
#if (USBD0_CDC1)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0_HS                  (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1_HS                  (1)
#endif
#if (USBD0_CDC2)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0_HS                  (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1_HS                  (2)
#endif
#if (USBD0_CDC3)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0_HS                  (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1_HS                  (3)
#endif

#if (USBD0_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD0_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (0)
#else
  USBD_HIDn_EP_HS                      (0)
#endif
#endif
#if (USBD0_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD0_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (1)
#else
  USBD_HIDn_EP_HS                      (1)
#endif
#endif
#if (USBD0_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD0_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (2)
#else
  USBD_HIDn_EP_HS                      (2)
#endif
#endif
#if (USBD0_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD0_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (3)
#else
  USBD_HIDn_EP_HS                      (3)
#endif
#endif

#if (USBD0_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP_HS                      (0)
#endif
#if (USBD0_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP_HS                      (1)
#endif
#if (USBD0_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP_HS                      (2)
#endif
#if (USBD0_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP_HS                      (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 0 Other Speed Configuration Descriptor (in Full Speed for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd0_other_speed_config_descriptor_fs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD0_WTOTALLENGTH),            /* wTotalLength */
  USBD0_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD0_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD0_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD0_ADC0)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP_HS                      (0)
#endif
#if (USBD0_ADC1)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP_HS                      (1)
#endif
#if (USBD0_ADC2)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP_HS                      (2)
#endif
#if (USBD0_ADC3)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP_HS                      (3)
#endif

#if (USBD0_CDC0)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0_HS                  (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1_HS                  (0)
#endif
#if (USBD0_CDC1)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0_HS                  (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1_HS                  (1)
#endif
#if (USBD0_CDC2)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0_HS                  (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1_HS                  (2)
#endif
#if (USBD0_CDC3)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (0,3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0_HS                  (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1_HS                  (3)
#endif

#if (USBD0_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD0_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (0)
#else
  USBD_HIDn_EP_HS                      (0)
#endif
#endif
#if (USBD0_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD0_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (1)
#else
  USBD_HIDn_EP_HS                      (1)
#endif
#endif
#if (USBD0_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD0_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (2)
#else
  USBD_HIDn_EP_HS                      (2)
#endif
#endif
#if (USBD0_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD0_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (3)
#else
  USBD_HIDn_EP_HS                      (3)
#endif
#endif

#if (USBD0_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP_HS                      (0)
#endif
#if (USBD0_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP_HS                      (1)
#endif
#if (USBD0_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP_HS                      (2)
#endif
#if (USBD0_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP_HS                      (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 0 Other Speed Configuration Descriptor (in High Speed for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd0_other_speed_config_descriptor_hs[] = {
/* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD0_WTOTALLENGTH),            /* wTotalLength */
  USBD0_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD0_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD0_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD0_ADC0)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP                         (0)
#endif
#if (USBD0_ADC1)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP                         (1)
#endif
#if (USBD0_ADC2)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP                         (2)
#endif
#if (USBD0_ADC3)
#if (USBD0_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP                         (3)
#endif

#if (USBD0_CDC0)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0                     (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1                     (0)
#endif
#if (USBD0_CDC1)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0                     (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1                     (1)
#endif
#if (USBD0_CDC2)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0                     (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1                     (2)
#endif
#if (USBD0_CDC3)
#if (USBD0_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0                     (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1                     (3)
#endif

#if (USBD0_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD0_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (0)
#else
  USBD_HIDn_EP                         (0)
#endif
#endif
#if (USBD0_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD0_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (1)
#else
  USBD_HIDn_EP                         (1)
#endif
#endif
#if (USBD0_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD0_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (2)
#else
  USBD_HIDn_EP                         (2)
#endif
#endif
#if (USBD0_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD0_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (3)
#else
  USBD_HIDn_EP                         (3)
#endif
#endif

#if (USBD0_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP                         (0)
#endif
#if (USBD0_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP                         (1)
#endif
#if (USBD0_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP                         (2)
#endif
#if (USBD0_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP                         (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#endif

__weak \
const struct {
  struct {
    uint8_t  len;
    uint8_t  type;
    uint16_t langid;
  } desc_langid;                      
  USBD_STR_DEF(USBD0_STR_DESC_MAN);
  USBD_STR_DEF(USBD0_STR_DESC_PROD);
#if  (USBD0_STR_DESC_SER_EN)
  USBD_STR_DEF(USBD0_STR_DESC_SER);
#endif
#if (USBD0_ADC0)
  USBD_STR_DEF(USBD_ADC0_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC0_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC0_SIF2_STR_DESC);
#endif
#if (USBD0_ADC1)
  USBD_STR_DEF(USBD_ADC1_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC1_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC1_SIF2_STR_DESC);
#endif
#if (USBD0_ADC2)
  USBD_STR_DEF(USBD_ADC2_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC2_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC2_SIF2_STR_DESC);
#endif
#if (USBD0_ADC3)
  USBD_STR_DEF(USBD_ADC3_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC3_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC3_SIF2_STR_DESC);
#endif

#if (USBD0_CDC0)
  USBD_STR_DEF(USBD_CDC0_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC0_DIF_STR_DESC);
#endif
#if (USBD0_CDC1)
  USBD_STR_DEF(USBD_CDC1_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC1_DIF_STR_DESC);
#endif
#if (USBD0_CDC2)
  USBD_STR_DEF(USBD_CDC2_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC2_DIF_STR_DESC);
#endif
#if (USBD0_CDC3)
  USBD_STR_DEF(USBD_CDC3_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC3_DIF_STR_DESC);
#endif

#if (USBD0_HID0)
  USBD_STR_DEF(USBD_HID0_STR_DESC);
#endif
#if (USBD0_HID1)
  USBD_STR_DEF(USBD_HID1_STR_DESC);
#endif
#if (USBD0_HID2)
  USBD_STR_DEF(USBD_HID2_STR_DESC);
#endif
#if (USBD0_HID3)
  USBD_STR_DEF(USBD_HID3_STR_DESC);
#endif

#if (USBD0_MSC0)
  USBD_STR_DEF(USBD_MSC0_STR_DESC);
#endif
#if (USBD0_MSC1)
  USBD_STR_DEF(USBD_MSC1_STR_DESC);
#endif
#if (USBD0_MSC2)
  USBD_STR_DEF(USBD_MSC2_STR_DESC);
#endif
#if (USBD0_MSC3)
  USBD_STR_DEF(USBD_MSC3_STR_DESC);
#endif

} usbd0_string_descriptor
  = 
{
  { 4, USB_STRING_DESCRIPTOR_TYPE, USBD0_STR_DESC_LANGID },
  USBD_STR_VAL(USBD0_STR_DESC_MAN),
  USBD_STR_VAL(USBD0_STR_DESC_PROD),
#if (USBD0_STR_DESC_SER_EN)
  USBD_STR_VAL(USBD0_STR_DESC_SER),
#endif
#if (USBD0_ADC0)
  USBD_STR_VAL(USBD_ADC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC0_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC0_SIF2_STR_DESC),
#endif
#if (USBD0_ADC1)
  USBD_STR_VAL(USBD_ADC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC1_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC1_SIF2_STR_DESC),
#endif
#if (USBD0_ADC2)
  USBD_STR_VAL(USBD_ADC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC2_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC2_SIF2_STR_DESC),
#endif
#if (USBD0_ADC3)
  USBD_STR_VAL(USBD_ADC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC3_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC3_SIF2_STR_DESC),
#endif

#if (USBD0_CDC0)
  USBD_STR_VAL(USBD_CDC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC0_DIF_STR_DESC),
#endif
#if (USBD0_CDC1)
  USBD_STR_VAL(USBD_CDC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC1_DIF_STR_DESC),
#endif
#if (USBD0_CDC2)
  USBD_STR_VAL(USBD_CDC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC2_DIF_STR_DESC),
#endif
#if (USBD0_CDC3)
  USBD_STR_VAL(USBD_CDC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC3_DIF_STR_DESC),
#endif

#if (USBD0_HID0)
  USBD_STR_VAL(USBD_HID0_STR_DESC),
#endif
#if (USBD0_HID1)
  USBD_STR_VAL(USBD_HID1_STR_DESC),
#endif
#if (USBD0_HID2)
  USBD_STR_VAL(USBD_HID2_STR_DESC),
#endif
#if (USBD0_HID3)
  USBD_STR_VAL(USBD_HID3_STR_DESC),
#endif

#if (USBD0_MSC0)
  USBD_STR_VAL(USBD_MSC0_STR_DESC),
#endif
#if (USBD0_MSC1)
  USBD_STR_VAL(USBD_MSC1_STR_DESC),
#endif
#if (USBD0_MSC2)
  USBD_STR_VAL(USBD_MSC2_STR_DESC),
#endif
#if (USBD0_MSC3)
  USBD_STR_VAL(USBD_MSC3_STR_DESC),
#endif
};

usbd_desc_t usbd0_desc = {
  (uint8_t *)usbd0_ep0_descriptor,
  (uint8_t *)usbd0_device_descriptor,
  (uint8_t *)usbd0_device_qualifier_fs,
  (uint8_t *)usbd0_device_qualifier_hs,
  (uint8_t *)usbd0_config_descriptor_fs,
  (uint8_t *)usbd0_config_descriptor_hs,
  (uint8_t *)usbd0_other_speed_config_descriptor_fs,
  (uint8_t *)usbd0_other_speed_config_descriptor_hs,
  (uint8_t *)&usbd0_string_descriptor
};

#endif  /* USBD0_PORT */


/*------------------------------------------------------------------------------
 *      USB Device 1 Descriptors
 *----------------------------------------------------------------------------*/

#if    (defined(USBD1_PORT) && !defined(USBD1_DESC_DEF))
#define USBD1_DESC_DEF                  1

/* USB Device 1 Endpoint 0 Descriptor */
__weak \
const uint8_t usbd1_ep0_descriptor[] = { USBD_EP0(1) };

/* USB Device 1 Standard Descriptor */
__weak \
const uint8_t usbd1_device_descriptor[] = {
  USB_DEVICE_DESC_SIZE,                 /* bLength */
  USB_DEVICE_DESCRIPTOR_TYPE,           /* bDescriptorType */
  WBVAL(USBD1_BDC_USB),                 /* bcdUSB */
#if (USBD1_USE_IAD)
  USB_DEVICE_CLASS_MISCELLANEOUS,       /* bDeviceClass */
  0x02,                                 /* bDeviceSubClass */
  0x01,                                 /* bDeviceProtocol */
#else
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
#endif
  USBD1_MAX_PACKET0,                    /* bMaxPacketSize0 */
  WBVAL(USBD1_DEV_DESC_IDVENDOR),       /* idVendor */
  WBVAL(USBD1_DEV_DESC_IDPRODUCT),      /* idProduct */
  WBVAL(USBD1_DEV_DESC_BCDDEVICE),      /* bcdDevice */
  0x01,                                 /* iManufacturer */
  0x02,                                 /* iProduct */
  0x03*USBD1_STR_DESC_SER_EN,           /* iSerialNumber */
  0x01                                  /* bNumConfigurations: one possible configuration*/
};

/* USB Device 1 Configuration Descriptor (for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd1_config_descriptor_fs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD1_WTOTALLENGTH),            /* wTotalLength */
  USBD1_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD1_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD1_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD1_ADC0)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP                         (0)
#endif
#if (USBD1_ADC1)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP                         (1)
#endif
#if (USBD1_ADC2)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP                         (2)
#endif
#if (USBD1_ADC3)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP                         (3)
#endif

#if (USBD1_CDC0)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0                     (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1                     (0)
#endif
#if (USBD1_CDC1)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0                     (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1                     (1)
#endif
#if (USBD1_CDC2)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0                     (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1                     (2)
#endif
#if (USBD1_CDC3)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0                     (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1                     (3)
#endif

#if (USBD1_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD1_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (0)
#else
  USBD_HIDn_EP                         (0)
#endif
#endif
#if (USBD1_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD1_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (1)
#else
  USBD_HIDn_EP                         (1)
#endif
#endif
#if (USBD1_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD1_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (2)
#else
  USBD_HIDn_EP                         (2)
#endif
#endif
#if (USBD1_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD1_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (3)
#else
  USBD_HIDn_EP                         (3)
#endif
#endif

#if (USBD1_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP                         (0)
#endif
#if (USBD1_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP                         (1)
#endif
#if (USBD1_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP                         (2)
#endif
#if (USBD1_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP                         (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#if (!USBD1_HS)                         /* If High-speed not enabled, declare dummy descriptors for High-speed */
__weak \
const uint8_t usbd1_device_qualifier_fs[]              = { 0 };
__weak \
const uint8_t usbd1_device_qualifier_hs[]              = { 0 };
__weak \
const uint8_t usbd1_config_descriptor_hs[]             = { 0 };
__weak \
const uint8_t usbd1_other_speed_config_descriptor_fs[] = { 0 };
__weak \
const uint8_t usbd1_other_speed_config_descriptor_hs[] = { 0 };

#else
/* USB Device 1 Qualifier Descriptor (in Full Speed for High Speed) */
__weak \
const uint8_t usbd1_device_qualifier_fs[] = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD1_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 1 Qualifier Descriptor (in High Speed for Full Speed) */
__weak \
const uint8_t usbd1_device_qualifier_hs[] = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD1_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 1 Configuration Descriptor (for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd1_config_descriptor_hs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD1_WTOTALLENGTH),            /* wTotalLength */
  USBD1_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD1_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD1_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD1_ADC0)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP_HS                      (0)
#endif
#if (USBD1_ADC1)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP_HS                      (1)
#endif
#if (USBD1_ADC2)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP_HS                      (2)
#endif
#if (USBD1_ADC3)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP_HS                      (3)
#endif

#if (USBD1_CDC0)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0_HS                  (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1_HS                  (0)
#endif
#if (USBD1_CDC1)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0_HS                  (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1_HS                  (1)
#endif
#if (USBD1_CDC2)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0_HS                  (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1_HS                  (2)
#endif
#if (USBD1_CDC3)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0_HS                  (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1_HS                  (3)
#endif

#if (USBD1_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD1_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (0)
#else
  USBD_HIDn_EP_HS                      (0)
#endif
#endif
#if (USBD1_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD1_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (1)
#else
  USBD_HIDn_EP_HS                      (1)
#endif
#endif
#if (USBD1_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD1_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (2)
#else
  USBD_HIDn_EP_HS                      (2)
#endif
#endif
#if (USBD1_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD1_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (3)
#else
  USBD_HIDn_EP_HS                      (3)
#endif
#endif

#if (USBD1_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP_HS                      (0)
#endif
#if (USBD1_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP_HS                      (1)
#endif
#if (USBD1_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP_HS                      (2)
#endif
#if (USBD1_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP_HS                      (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 1 Other Speed Configuration Descriptor (in Full Speed for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd1_other_speed_config_descriptor_fs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD1_WTOTALLENGTH),            /* wTotalLength */
  USBD1_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD1_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD1_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD1_ADC0)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP_HS                      (0)
#endif
#if (USBD1_ADC1)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP_HS                      (1)
#endif
#if (USBD1_ADC2)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP_HS                      (2)
#endif
#if (USBD1_ADC3)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP_HS                      (3)
#endif

#if (USBD1_CDC0)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0_HS                  (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1_HS                  (0)
#endif
#if (USBD1_CDC1)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0_HS                  (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1_HS                  (1)
#endif
#if (USBD1_CDC2)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0_HS                  (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1_HS                  (2)
#endif
#if (USBD1_CDC3)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (0,3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0_HS                  (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1_HS                  (3)
#endif

#if (USBD1_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD1_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (0)
#else
  USBD_HIDn_EP_HS                      (0)
#endif
#endif
#if (USBD1_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD1_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (1)
#else
  USBD_HIDn_EP_HS                      (1)
#endif
#endif
#if (USBD1_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD1_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (2)
#else
  USBD_HIDn_EP_HS                      (2)
#endif
#endif
#if (USBD1_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD1_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (3)
#else
  USBD_HIDn_EP_HS                      (3)
#endif
#endif

#if (USBD1_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP_HS                      (0)
#endif
#if (USBD1_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP_HS                      (1)
#endif
#if (USBD1_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP_HS                      (2)
#endif
#if (USBD1_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP_HS                      (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 1 Other Speed Configuration Descriptor (in High Speed for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd1_other_speed_config_descriptor_hs[] = {
/* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD1_WTOTALLENGTH),            /* wTotalLength */
  USBD1_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD1_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD1_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD1_ADC0)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP                         (0)
#endif
#if (USBD1_ADC1)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP                         (1)
#endif
#if (USBD1_ADC2)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP                         (2)
#endif
#if (USBD1_ADC3)
#if (USBD1_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP                         (3)
#endif

#if (USBD1_CDC0)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0                     (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1                     (0)
#endif
#if (USBD1_CDC1)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0                     (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1                     (1)
#endif
#if (USBD1_CDC2)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0                     (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1                     (2)
#endif
#if (USBD1_CDC3)
#if (USBD1_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0                     (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1                     (3)
#endif

#if (USBD1_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD1_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (0)
#else
  USBD_HIDn_EP                         (0)
#endif
#endif
#if (USBD1_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD1_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (1)
#else
  USBD_HIDn_EP                         (1)
#endif
#endif
#if (USBD1_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD1_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (2)
#else
  USBD_HIDn_EP                         (2)
#endif
#endif
#if (USBD1_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD1_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (3)
#else
  USBD_HIDn_EP                         (3)
#endif
#endif

#if (USBD1_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP                         (0)
#endif
#if (USBD1_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP                         (1)
#endif
#if (USBD1_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP                         (2)
#endif
#if (USBD1_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP                         (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#endif

__weak \
const struct {
  struct {
    uint8_t  len;
    uint8_t  type;
    uint16_t langid;
  } desc_langid;                      
  USBD_STR_DEF(USBD1_STR_DESC_MAN);
  USBD_STR_DEF(USBD1_STR_DESC_PROD);
#if  (USBD1_STR_DESC_SER_EN)
  USBD_STR_DEF(USBD1_STR_DESC_SER);
#endif
#if (USBD1_ADC0)
  USBD_STR_DEF(USBD_ADC0_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC0_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC0_SIF2_STR_DESC);
#endif
#if (USBD1_ADC1)
  USBD_STR_DEF(USBD_ADC1_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC1_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC1_SIF2_STR_DESC);
#endif
#if (USBD1_ADC2)
  USBD_STR_DEF(USBD_ADC2_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC2_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC2_SIF2_STR_DESC);
#endif
#if (USBD1_ADC3)
  USBD_STR_DEF(USBD_ADC3_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC3_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC3_SIF2_STR_DESC);
#endif

#if (USBD1_CDC0)
  USBD_STR_DEF(USBD_CDC0_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC0_DIF_STR_DESC);
#endif
#if (USBD1_CDC1)
  USBD_STR_DEF(USBD_CDC1_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC1_DIF_STR_DESC);
#endif
#if (USBD1_CDC2)
  USBD_STR_DEF(USBD_CDC2_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC2_DIF_STR_DESC);
#endif
#if (USBD1_CDC3)
  USBD_STR_DEF(USBD_CDC3_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC3_DIF_STR_DESC);
#endif

#if (USBD1_HID0)
  USBD_STR_DEF(USBD_HID0_STR_DESC);
#endif
#if (USBD1_HID1)
  USBD_STR_DEF(USBD_HID1_STR_DESC);
#endif
#if (USBD1_HID2)
  USBD_STR_DEF(USBD_HID2_STR_DESC);
#endif
#if (USBD1_HID3)
  USBD_STR_DEF(USBD_HID3_STR_DESC);
#endif

#if (USBD1_MSC0)
  USBD_STR_DEF(USBD_MSC0_STR_DESC);
#endif
#if (USBD1_MSC1)
  USBD_STR_DEF(USBD_MSC1_STR_DESC);
#endif
#if (USBD1_MSC2)
  USBD_STR_DEF(USBD_MSC2_STR_DESC);
#endif
#if (USBD1_MSC3)
  USBD_STR_DEF(USBD_MSC3_STR_DESC);
#endif

} usbd1_string_descriptor
  = 
{
  { 4, USB_STRING_DESCRIPTOR_TYPE, USBD1_STR_DESC_LANGID },
  USBD_STR_VAL(USBD1_STR_DESC_MAN),
  USBD_STR_VAL(USBD1_STR_DESC_PROD),
#if (USBD1_STR_DESC_SER_EN)
  USBD_STR_VAL(USBD1_STR_DESC_SER),
#endif
#if (USBD1_ADC0)
  USBD_STR_VAL(USBD_ADC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC0_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC0_SIF2_STR_DESC),
#endif
#if (USBD1_ADC1)
  USBD_STR_VAL(USBD_ADC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC1_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC1_SIF2_STR_DESC),
#endif
#if (USBD1_ADC2)
  USBD_STR_VAL(USBD_ADC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC2_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC2_SIF2_STR_DESC),
#endif
#if (USBD1_ADC3)
  USBD_STR_VAL(USBD_ADC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC3_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC3_SIF2_STR_DESC),
#endif

#if (USBD1_CDC0)
  USBD_STR_VAL(USBD_CDC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC0_DIF_STR_DESC),
#endif
#if (USBD1_CDC1)
  USBD_STR_VAL(USBD_CDC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC1_DIF_STR_DESC),
#endif
#if (USBD1_CDC2)
  USBD_STR_VAL(USBD_CDC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC2_DIF_STR_DESC),
#endif
#if (USBD1_CDC3)
  USBD_STR_VAL(USBD_CDC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC3_DIF_STR_DESC),
#endif

#if (USBD1_HID0)
  USBD_STR_VAL(USBD_HID0_STR_DESC),
#endif
#if (USBD1_HID1)
  USBD_STR_VAL(USBD_HID1_STR_DESC),
#endif
#if (USBD1_HID2)
  USBD_STR_VAL(USBD_HID2_STR_DESC),
#endif
#if (USBD1_HID3)
  USBD_STR_VAL(USBD_HID3_STR_DESC),
#endif

#if (USBD1_MSC0)
  USBD_STR_VAL(USBD_MSC0_STR_DESC),
#endif
#if (USBD1_MSC1)
  USBD_STR_VAL(USBD_MSC1_STR_DESC),
#endif
#if (USBD1_MSC2)
  USBD_STR_VAL(USBD_MSC2_STR_DESC),
#endif
#if (USBD1_MSC3)
  USBD_STR_VAL(USBD_MSC3_STR_DESC),
#endif
};

usbd_desc_t usbd1_desc = {
  (uint8_t *)usbd1_ep0_descriptor,
  (uint8_t *)usbd1_device_descriptor,
  (uint8_t *)usbd1_device_qualifier_fs,
  (uint8_t *)usbd1_device_qualifier_hs,
  (uint8_t *)usbd1_config_descriptor_fs,
  (uint8_t *)usbd1_config_descriptor_hs,
  (uint8_t *)usbd1_other_speed_config_descriptor_fs,
  (uint8_t *)usbd1_other_speed_config_descriptor_hs,
  (uint8_t *)&usbd1_string_descriptor
};

#endif  /* USBD1_PORT */


/*------------------------------------------------------------------------------
 *      USB Device 2 Descriptors
 *----------------------------------------------------------------------------*/

#if    (defined(USBD2_PORT) && !defined(USBD2_DESC_DEF))
#define USBD2_DESC_DEF                  1

/* USB Device 2 Endpoint 0 Descriptor */
__weak \
const uint8_t usbd2_ep0_descriptor[] = { USBD_EP0(2) };

/* USB Device 2 Standard Descriptor */
__weak \
const uint8_t usbd2_device_descriptor[] = {
  USB_DEVICE_DESC_SIZE,                 /* bLength */
  USB_DEVICE_DESCRIPTOR_TYPE,           /* bDescriptorType */
  WBVAL(USBD2_BDC_USB),                 /* bcdUSB */
#if (USBD2_USE_IAD)
  USB_DEVICE_CLASS_MISCELLANEOUS,       /* bDeviceClass */
  0x02,                                 /* bDeviceSubClass */
  0x01,                                 /* bDeviceProtocol */
#else
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
#endif
  USBD2_MAX_PACKET0,                    /* bMaxPacketSize0 */
  WBVAL(USBD2_DEV_DESC_IDVENDOR),       /* idVendor */
  WBVAL(USBD2_DEV_DESC_IDPRODUCT),      /* idProduct */
  WBVAL(USBD2_DEV_DESC_BCDDEVICE),      /* bcdDevice */
  0x01,                                 /* iManufacturer */
  0x02,                                 /* iProduct */
  0x03*USBD2_STR_DESC_SER_EN,           /* iSerialNumber */
  0x01                                  /* bNumConfigurations: one possible configuration*/
};

/* USB Device 2 Configuration Descriptor (for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd2_config_descriptor_fs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD2_WTOTALLENGTH),            /* wTotalLength */
  USBD2_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD2_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD2_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD2_ADC0)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP                         (0)
#endif
#if (USBD2_ADC1)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP                         (1)
#endif
#if (USBD2_ADC2)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP                         (2)
#endif
#if (USBD2_ADC3)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP                         (3)
#endif

#if (USBD2_CDC0)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0                     (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1                     (0)
#endif
#if (USBD2_CDC1)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0                     (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1                     (1)
#endif
#if (USBD2_CDC2)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0                     (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1                     (2)
#endif
#if (USBD2_CDC3)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0                     (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1                     (3)
#endif

#if (USBD2_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD2_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (0)
#else
  USBD_HIDn_EP                         (0)
#endif
#endif
#if (USBD2_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD2_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (1)
#else
  USBD_HIDn_EP                         (1)
#endif
#endif
#if (USBD2_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD2_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (2)
#else
  USBD_HIDn_EP                         (2)
#endif
#endif
#if (USBD2_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD2_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (3)
#else
  USBD_HIDn_EP                         (3)
#endif
#endif

#if (USBD2_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP                         (0)
#endif
#if (USBD2_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP                         (1)
#endif
#if (USBD2_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP                         (2)
#endif
#if (USBD2_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP                         (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#if (!USBD2_HS)                         /* If High-speed not enabled, declare dummy descriptors for High-speed */
__weak \
const uint8_t usbd2_device_qualifier_fs[]              = { 0 };
__weak \
const uint8_t usbd2_device_qualifier_hs[]              = { 0 };
__weak \
const uint8_t usbd2_config_descriptor_hs[]             = { 0 };
__weak \
const uint8_t usbd2_other_speed_config_descriptor_fs[] = { 0 };
__weak \
const uint8_t usbd2_other_speed_config_descriptor_hs[] = { 0 };

#else
/* USB Device 2 Qualifier Descriptor (in Full Speed for High Speed) */
__weak \
const uint8_t usbd2_device_qualifier_fs[] = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD2_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 2 Qualifier Descriptor (in High Speed for Full Speed) */
__weak \
const uint8_t usbd2_device_qualifier_hs[] = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD2_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 2 Configuration Descriptor (for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd2_config_descriptor_hs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD2_WTOTALLENGTH),            /* wTotalLength */
  USBD2_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD2_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD2_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD2_ADC0)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP_HS                      (0)
#endif
#if (USBD2_ADC1)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP_HS                      (1)
#endif
#if (USBD2_ADC2)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP_HS                      (2)
#endif
#if (USBD2_ADC3)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP_HS                      (3)
#endif

#if (USBD2_CDC0)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0_HS                  (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1_HS                  (0)
#endif
#if (USBD2_CDC1)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0_HS                  (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1_HS                  (1)
#endif
#if (USBD2_CDC2)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0_HS                  (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1_HS                  (2)
#endif
#if (USBD2_CDC3)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0_HS                  (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1_HS                  (3)
#endif

#if (USBD2_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD2_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (0)
#else
  USBD_HIDn_EP_HS                      (0)
#endif
#endif
#if (USBD2_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD2_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (1)
#else
  USBD_HIDn_EP_HS                      (1)
#endif
#endif
#if (USBD2_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD2_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (2)
#else
  USBD_HIDn_EP_HS                      (2)
#endif
#endif
#if (USBD2_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD2_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (3)
#else
  USBD_HIDn_EP_HS                      (3)
#endif
#endif

#if (USBD2_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP_HS                      (0)
#endif
#if (USBD2_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP_HS                      (1)
#endif
#if (USBD2_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP_HS                      (2)
#endif
#if (USBD2_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP_HS                      (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 2 Other Speed Configuration Descriptor (in Full Speed for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd2_other_speed_config_descriptor_fs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD2_WTOTALLENGTH),            /* wTotalLength */
  USBD2_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD2_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD2_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD2_ADC0)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP_HS                      (0)
#endif
#if (USBD2_ADC1)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP_HS                      (1)
#endif
#if (USBD2_ADC2)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP_HS                      (2)
#endif
#if (USBD2_ADC3)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP_HS                      (3)
#endif

#if (USBD2_CDC0)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0_HS                  (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1_HS                  (0)
#endif
#if (USBD2_CDC1)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0_HS                  (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1_HS                  (1)
#endif
#if (USBD2_CDC2)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0_HS                  (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1_HS                  (2)
#endif
#if (USBD2_CDC3)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (0,3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0_HS                  (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1_HS                  (3)
#endif

#if (USBD2_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD2_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (0)
#else
  USBD_HIDn_EP_HS                      (0)
#endif
#endif
#if (USBD2_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD2_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (1)
#else
  USBD_HIDn_EP_HS                      (1)
#endif
#endif
#if (USBD2_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD2_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (2)
#else
  USBD_HIDn_EP_HS                      (2)
#endif
#endif
#if (USBD2_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD2_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (3)
#else
  USBD_HIDn_EP_HS                      (3)
#endif
#endif

#if (USBD2_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP_HS                      (0)
#endif
#if (USBD2_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP_HS                      (1)
#endif
#if (USBD2_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP_HS                      (2)
#endif
#if (USBD2_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP_HS                      (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 2 Other Speed Configuration Descriptor (in High Speed for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd2_other_speed_config_descriptor_hs[] = {
/* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD2_WTOTALLENGTH),            /* wTotalLength */
  USBD2_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD2_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD2_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD2_ADC0)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP                         (0)
#endif
#if (USBD2_ADC1)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP                         (1)
#endif
#if (USBD2_ADC2)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP                         (2)
#endif
#if (USBD2_ADC3)
#if (USBD2_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP                         (3)
#endif

#if (USBD2_CDC0)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0                     (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1                     (0)
#endif
#if (USBD2_CDC1)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0                     (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1                     (1)
#endif
#if (USBD2_CDC2)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0                     (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1                     (2)
#endif
#if (USBD2_CDC3)
#if (USBD2_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0                     (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1                     (3)
#endif

#if (USBD2_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD2_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (0)
#else
  USBD_HIDn_EP                         (0)
#endif
#endif
#if (USBD2_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD2_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (1)
#else
  USBD_HIDn_EP                         (1)
#endif
#endif
#if (USBD2_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD2_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (2)
#else
  USBD_HIDn_EP                         (2)
#endif
#endif
#if (USBD2_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD2_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (3)
#else
  USBD_HIDn_EP                         (3)
#endif
#endif

#if (USBD2_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP                         (0)
#endif
#if (USBD2_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP                         (1)
#endif
#if (USBD2_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP                         (2)
#endif
#if (USBD2_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP                         (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#endif

__weak \
const struct {
  struct {
    uint8_t  len;
    uint8_t  type;
    uint16_t langid;
  } desc_langid;                      
  USBD_STR_DEF(USBD2_STR_DESC_MAN);
  USBD_STR_DEF(USBD2_STR_DESC_PROD);
#if  (USBD2_STR_DESC_SER_EN)
  USBD_STR_DEF(USBD2_STR_DESC_SER);
#endif
#if (USBD2_ADC0)
  USBD_STR_DEF(USBD_ADC0_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC0_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC0_SIF2_STR_DESC);
#endif
#if (USBD2_ADC1)
  USBD_STR_DEF(USBD_ADC1_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC1_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC1_SIF2_STR_DESC);
#endif
#if (USBD2_ADC2)
  USBD_STR_DEF(USBD_ADC2_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC2_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC2_SIF2_STR_DESC);
#endif
#if (USBD2_ADC3)
  USBD_STR_DEF(USBD_ADC3_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC3_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC3_SIF2_STR_DESC);
#endif

#if (USBD2_CDC0)
  USBD_STR_DEF(USBD_CDC0_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC0_DIF_STR_DESC);
#endif
#if (USBD2_CDC1)
  USBD_STR_DEF(USBD_CDC1_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC1_DIF_STR_DESC);
#endif
#if (USBD2_CDC2)
  USBD_STR_DEF(USBD_CDC2_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC2_DIF_STR_DESC);
#endif
#if (USBD2_CDC3)
  USBD_STR_DEF(USBD_CDC3_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC3_DIF_STR_DESC);
#endif

#if (USBD2_HID0)
  USBD_STR_DEF(USBD_HID0_STR_DESC);
#endif
#if (USBD2_HID1)
  USBD_STR_DEF(USBD_HID1_STR_DESC);
#endif
#if (USBD2_HID2)
  USBD_STR_DEF(USBD_HID2_STR_DESC);
#endif
#if (USBD2_HID3)
  USBD_STR_DEF(USBD_HID3_STR_DESC);
#endif

#if (USBD2_MSC0)
  USBD_STR_DEF(USBD_MSC0_STR_DESC);
#endif
#if (USBD2_MSC1)
  USBD_STR_DEF(USBD_MSC1_STR_DESC);
#endif
#if (USBD2_MSC2)
  USBD_STR_DEF(USBD_MSC2_STR_DESC);
#endif
#if (USBD2_MSC3)
  USBD_STR_DEF(USBD_MSC3_STR_DESC);
#endif

} usbd2_string_descriptor
  = 
{
  { 4, USB_STRING_DESCRIPTOR_TYPE, USBD2_STR_DESC_LANGID },
  USBD_STR_VAL(USBD2_STR_DESC_MAN),
  USBD_STR_VAL(USBD2_STR_DESC_PROD),
#if (USBD2_STR_DESC_SER_EN)
  USBD_STR_VAL(USBD2_STR_DESC_SER),
#endif
#if (USBD2_ADC0)
  USBD_STR_VAL(USBD_ADC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC0_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC0_SIF2_STR_DESC),
#endif
#if (USBD2_ADC1)
  USBD_STR_VAL(USBD_ADC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC1_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC1_SIF2_STR_DESC),
#endif
#if (USBD2_ADC2)
  USBD_STR_VAL(USBD_ADC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC2_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC2_SIF2_STR_DESC),
#endif
#if (USBD2_ADC3)
  USBD_STR_VAL(USBD_ADC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC3_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC3_SIF2_STR_DESC),
#endif

#if (USBD2_CDC0)
  USBD_STR_VAL(USBD_CDC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC0_DIF_STR_DESC),
#endif
#if (USBD2_CDC1)
  USBD_STR_VAL(USBD_CDC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC1_DIF_STR_DESC),
#endif
#if (USBD2_CDC2)
  USBD_STR_VAL(USBD_CDC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC2_DIF_STR_DESC),
#endif
#if (USBD2_CDC3)
  USBD_STR_VAL(USBD_CDC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC3_DIF_STR_DESC),
#endif

#if (USBD2_HID0)
  USBD_STR_VAL(USBD_HID0_STR_DESC),
#endif
#if (USBD2_HID1)
  USBD_STR_VAL(USBD_HID1_STR_DESC),
#endif
#if (USBD2_HID2)
  USBD_STR_VAL(USBD_HID2_STR_DESC),
#endif
#if (USBD2_HID3)
  USBD_STR_VAL(USBD_HID3_STR_DESC),
#endif

#if (USBD2_MSC0)
  USBD_STR_VAL(USBD_MSC0_STR_DESC),
#endif
#if (USBD2_MSC1)
  USBD_STR_VAL(USBD_MSC1_STR_DESC),
#endif
#if (USBD2_MSC2)
  USBD_STR_VAL(USBD_MSC2_STR_DESC),
#endif
#if (USBD2_MSC3)
  USBD_STR_VAL(USBD_MSC3_STR_DESC),
#endif
};

usbd_desc_t usbd2_desc = {
  (uint8_t *)usbd2_ep0_descriptor,
  (uint8_t *)usbd2_device_descriptor,
  (uint8_t *)usbd2_device_qualifier_fs,
  (uint8_t *)usbd2_device_qualifier_hs,
  (uint8_t *)usbd2_config_descriptor_fs,
  (uint8_t *)usbd2_config_descriptor_hs,
  (uint8_t *)usbd2_other_speed_config_descriptor_fs,
  (uint8_t *)usbd2_other_speed_config_descriptor_hs,
  (uint8_t *)&usbd2_string_descriptor
};

#endif  /* USBD2_PORT */


/*------------------------------------------------------------------------------
 *      USB Device 0 Descriptors
 *----------------------------------------------------------------------------*/

#if    (defined(USBD3_PORT) && !defined(USBD3_DESC_DEF))
#define USBD3_DESC_DEF                  1

/* USB Device 3 Endpoint 0 Descriptor */
__weak \
const uint8_t usbd3_ep0_descriptor[] = { USBD_EP0(3) };

/* USB Device 3 Standard Descriptor */
__weak \
const uint8_t usbd3_device_descriptor[] = {
  USB_DEVICE_DESC_SIZE,                 /* bLength */
  USB_DEVICE_DESCRIPTOR_TYPE,           /* bDescriptorType */
  WBVAL(USBD3_BDC_USB),                 /* bcdUSB */
#if (USBD3_USE_IAD)
  USB_DEVICE_CLASS_MISCELLANEOUS,       /* bDeviceClass */
  0x02,                                 /* bDeviceSubClass */
  0x01,                                 /* bDeviceProtocol */
#else
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
#endif
  USBD3_MAX_PACKET0,                    /* bMaxPacketSize0 */
  WBVAL(USBD3_DEV_DESC_IDVENDOR),       /* idVendor */
  WBVAL(USBD3_DEV_DESC_IDPRODUCT),      /* idProduct */
  WBVAL(USBD3_DEV_DESC_BCDDEVICE),      /* bcdDevice */
  0x01,                                 /* iManufacturer */
  0x02,                                 /* iProduct */
  0x03*USBD3_STR_DESC_SER_EN,           /* iSerialNumber */
  0x01                                  /* bNumConfigurations: one possible configuration*/
};

/* USB Device 3 Configuration Descriptor (for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd3_config_descriptor_fs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD3_WTOTALLENGTH),            /* wTotalLength */
  USBD3_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD3_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD3_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD3_ADC0)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP                         (0)
#endif
#if (USBD3_ADC1)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP                         (1)
#endif
#if (USBD3_ADC2)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP                         (2)
#endif
#if (USBD3_ADC3)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP                         (3)
#endif

#if (USBD3_CDC0)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0                     (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1                     (0)
#endif
#if (USBD3_CDC1)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0                     (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1                     (1)
#endif
#if (USBD3_CDC2)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0                     (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1                     (2)
#endif
#if (USBD3_CDC3)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0                     (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1                     (3)
#endif

#if (USBD3_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD3_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (0)
#else
  USBD_HIDn_EP                         (0)
#endif
#endif
#if (USBD3_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD3_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (1)
#else
  USBD_HIDn_EP                         (1)
#endif
#endif
#if (USBD3_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD3_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (2)
#else
  USBD_HIDn_EP                         (2)
#endif
#endif
#if (USBD3_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD3_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (3)
#else
  USBD_HIDn_EP                         (3)
#endif
#endif

#if (USBD3_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP                         (0)
#endif
#if (USBD3_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP                         (1)
#endif
#if (USBD3_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP                         (2)
#endif
#if (USBD3_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP                         (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#if (!USBD3_HS)                         /* If High-speed not enabled, declare dummy descriptors for High-speed */
__weak \
const uint8_t usbd3_device_qualifier_fs[]              = { 0 };
__weak \
const uint8_t usbd3_device_qualifier_hs[]              = { 0 };
__weak \
const uint8_t usbd3_config_descriptor_hs[]             = { 0 };
__weak \
const uint8_t usbd3_other_speed_config_descriptor_fs[] = { 0 };
__weak \
const uint8_t usbd3_other_speed_config_descriptor_hs[] = { 0 };

#else
/* USB Device 3 Qualifier Descriptor (in Full Speed for High Speed) */
__weak \
const uint8_t usbd3_device_qualifier_fs[] = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD3_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 3 Qualifier Descriptor (in High Speed for Full Speed) */
__weak \
const uint8_t usbd3_device_qualifier_hs[] = {
  USB_DEVICE_QUALI_SIZE,                /* bLength */
  USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(0x0200), /* 2.00 */             /* bcdUSB */
  0x00,                                 /* bDeviceClass */
  0x00,                                 /* bDeviceSubClass */
  0x00,                                 /* bDeviceProtocol */
  USBD3_MAX_PACKET0,                    /* bMaxPacketSize0 */
  0x01,                                 /* bNumConfigurations */
  0x00                                  /* bReserved */
};

/* USB Device 3 Configuration Descriptor (for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd3_config_descriptor_hs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_CONFIGURATION_DESCRIPTOR_TYPE,    /* bDescriptorType */
  WBVAL(USBD3_WTOTALLENGTH),            /* wTotalLength */
  USBD3_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD3_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD3_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD3_ADC0)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP_HS                      (0)
#endif
#if (USBD3_ADC1)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP_HS                      (1)
#endif
#if (USBD3_ADC2)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP_HS                      (2)
#endif
#if (USBD3_ADC3)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP_HS                      (3)
#endif

#if (USBD3_CDC0)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0_HS                  (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1_HS                  (0)
#endif
#if (USBD3_CDC1)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0_HS                  (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1_HS                  (1)
#endif
#if (USBD3_CDC2)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0_HS                  (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1_HS                  (2)
#endif
#if (USBD3_CDC3)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0_HS                  (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1_HS                  (3)
#endif

#if (USBD3_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD3_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (0)
#else
  USBD_HIDn_EP_HS                      (0)
#endif
#endif
#if (USBD3_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD3_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (1)
#else
  USBD_HIDn_EP_HS                      (1)
#endif
#endif
#if (USBD3_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD3_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (2)
#else
  USBD_HIDn_EP_HS                      (2)
#endif
#endif
#if (USBD3_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD3_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (3)
#else
  USBD_HIDn_EP_HS                      (3)
#endif
#endif

#if (USBD3_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP_HS                      (0)
#endif
#if (USBD3_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP_HS                      (1)
#endif
#if (USBD3_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP_HS                      (2)
#endif
#if (USBD3_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP_HS                      (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 3 Other Speed Configuration Descriptor (in Full Speed for High Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd3_other_speed_config_descriptor_fs[] = {
  /* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD3_WTOTALLENGTH),            /* wTotalLength */
  USBD3_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD3_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD3_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD3_ADC0)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP_HS                      (0)
#endif
#if (USBD3_ADC1)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP_HS                      (1)
#endif
#if (USBD3_ADC2)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP_HS                      (2)
#endif
#if (USBD3_ADC3)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP_HS                      (3)
#endif

#if (USBD3_CDC0)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0_HS                  (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1_HS                  (0)
#endif
#if (USBD3_CDC1)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0_HS                  (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1_HS                  (1)
#endif
#if (USBD3_CDC2)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0_HS                  (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1_HS                  (2)
#endif
#if (USBD3_CDC3)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (0,3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0_HS                  (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1_HS                  (3)
#endif

#if (USBD3_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD3_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (0)
#else
  USBD_HIDn_EP_HS                      (0)
#endif
#endif
#if (USBD3_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD3_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (1)
#else
  USBD_HIDn_EP_HS                      (1)
#endif
#endif
#if (USBD3_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD3_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (2)
#else
  USBD_HIDn_EP_HS                      (2)
#endif
#endif
#if (USBD3_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD3_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT_HS                (3)
#else
  USBD_HIDn_EP_HS                      (3)
#endif
#endif

#if (USBD3_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP_HS                      (0)
#endif
#if (USBD3_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP_HS                      (1)
#endif
#if (USBD3_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP_HS                      (2)
#endif
#if (USBD3_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP_HS                      (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

/* USB Device 3 Other Speed Configuration Descriptor (in High Speed for Full Speed) */
/*   All Descriptors (Configuration, Interface, Endpoint, Class, Vendor) */
__weak \
const uint8_t usbd3_other_speed_config_descriptor_hs[] = {
/* Configuration 1 */
  USB_CONFIGURATION_DESC_SIZE,          /* bLength */
  USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE, /* bDescriptorType */
  WBVAL(USBD3_WTOTALLENGTH),            /* wTotalLength */
  USBD3_IF_NUM,                         /* bNumInterfaces */
  0x01,                                 /* bConfigurationValue: 0x01 is used to select this configuration */
  0x00,                                 /* iConfiguration: no string to describe this configuration */
  USBD3_CFG_DESC_BMATTRIBUTES,          /* bmAttributes */
  USBD3_CFG_DESC_BMAXPOWER,             /* bMaxPower, device power consumption */

#if (USBD3_ADC0)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (0,USBD_ADC0_CIF,2)
#endif
  USBD_ADCn_DESC                       (0)
  USBD_ADCn_EP                         (0)
#endif
#if (USBD3_ADC1)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (1,USBD_ADC1_CIF,2)
#endif
  USBD_ADCn_DESC                       (1)
  USBD_ADCn_EP                         (1)
#endif
#if (USBD3_ADC2)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (2,USBD_ADC2_CIF,2)
#endif
  USBD_ADCn_DESC                       (2)
  USBD_ADCn_EP                         (2)
#endif
#if (USBD3_ADC3)
#if (USBD3_USE_IAD)
  USBD_ADCn_IAD                        (3,USBD_ADC3_CIF,2)
#endif
  USBD_ADCn_DESC                       (3)
  USBD_ADCn_EP                         (3)
#endif

#if (USBD3_CDC0)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (0,USBD_CDC0_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (0)
  USBD_CDCn_EP_IF0                     (0)
  USBD_CDCn_DESC_IF1                   (0)
  USBD_CDCn_EP_IF1                     (0)
#endif
#if (USBD3_CDC1)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (1,USBD_CDC1_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (1)
  USBD_CDCn_EP_IF0                     (1)
  USBD_CDCn_DESC_IF1                   (1)
  USBD_CDCn_EP_IF1                     (1)
#endif
#if (USBD3_CDC2)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (2,USBD_CDC2_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (2)
  USBD_CDCn_EP_IF0                     (2)
  USBD_CDCn_DESC_IF1                   (2)
  USBD_CDCn_EP_IF1                     (2)
#endif
#if (USBD3_CDC3)
#if (USBD3_USE_IAD)
  USBD_CDCn_IAD                        (3,USBD_CDC3_CIF,2)
#endif
  USBD_CDCn_DESC_IF0                   (3)
  USBD_CDCn_EP_IF0                     (3)
  USBD_CDCn_DESC_IF1                   (3)
  USBD_CDCn_EP_IF1                     (3)
#endif

#if (USBD3_HID0)
  USBD_HIDn_DESC                       (0)
#if (USBD3_HID0_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (0)
#else
  USBD_HIDn_EP                         (0)
#endif
#endif
#if (USBD3_HID1)
  USBD_HIDn_DESC                       (1)
#if (USBD3_HID1_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (1)
#else
  USBD_HIDn_EP                         (1)
#endif
#endif
#if (USBD3_HID2)
  USBD_HIDn_DESC                       (2)
#if (USBD3_HID2_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (2)
#else
  USBD_HIDn_EP                         (2)
#endif
#endif
#if (USBD3_HID3)
  USBD_HIDn_DESC                       (3)
#if (USBD3_HID3_EP_INT_OUT != 0)
  USBD_HIDn_EP_INOUT                   (3)
#else
  USBD_HIDn_EP                         (3)
#endif
#endif

#if (USBD3_MSC0)
  USBD_MSCn_DESC                       (0)
  USBD_MSCn_EP                         (0)
#endif
#if (USBD3_MSC1)
  USBD_MSCn_DESC                       (1)
  USBD_MSCn_EP                         (1)
#endif
#if (USBD3_MSC2)
  USBD_MSCn_DESC                       (2)
  USBD_MSCn_EP                         (2)
#endif
#if (USBD3_MSC3)
  USBD_MSCn_DESC                       (3)
  USBD_MSCn_EP                         (3)
#endif

/* Terminator */
  0                                     /* bLength */
};

#endif

__weak \
const struct {
  struct {
    uint8_t  len;
    uint8_t  type;
    uint16_t langid;
  } desc_langid;                      
  USBD_STR_DEF(USBD3_STR_DESC_MAN);
  USBD_STR_DEF(USBD3_STR_DESC_PROD);
#if  (USBD3_STR_DESC_SER_EN)
  USBD_STR_DEF(USBD3_STR_DESC_SER);
#endif
#if (USBD3_ADC0)
  USBD_STR_DEF(USBD_ADC0_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC0_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC0_SIF2_STR_DESC);
#endif
#if (USBD3_ADC1)
  USBD_STR_DEF(USBD_ADC1_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC1_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC1_SIF2_STR_DESC);
#endif
#if (USBD3_ADC2)
  USBD_STR_DEF(USBD_ADC2_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC2_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC2_SIF2_STR_DESC);
#endif
#if (USBD3_ADC3)
  USBD_STR_DEF(USBD_ADC3_CIF_STR_DESC);
  USBD_STR_DEF(USBD_ADC3_SIF1_STR_DESC);
  USBD_STR_DEF(USBD_ADC3_SIF2_STR_DESC);
#endif

#if (USBD3_CDC0)
  USBD_STR_DEF(USBD_CDC0_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC0_DIF_STR_DESC);
#endif
#if (USBD3_CDC1)
  USBD_STR_DEF(USBD_CDC1_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC1_DIF_STR_DESC);
#endif
#if (USBD3_CDC2)
  USBD_STR_DEF(USBD_CDC2_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC2_DIF_STR_DESC);
#endif
#if (USBD3_CDC3)
  USBD_STR_DEF(USBD_CDC3_CIF_STR_DESC);
  USBD_STR_DEF(USBD_CDC3_DIF_STR_DESC);
#endif

#if (USBD3_HID0)
  USBD_STR_DEF(USBD_HID0_STR_DESC);
#endif
#if (USBD3_HID1)
  USBD_STR_DEF(USBD_HID1_STR_DESC);
#endif
#if (USBD3_HID2)
  USBD_STR_DEF(USBD_HID2_STR_DESC);
#endif
#if (USBD3_HID3)
  USBD_STR_DEF(USBD_HID3_STR_DESC);
#endif

#if (USBD3_MSC0)
  USBD_STR_DEF(USBD_MSC0_STR_DESC);
#endif
#if (USBD3_MSC1)
  USBD_STR_DEF(USBD_MSC1_STR_DESC);
#endif
#if (USBD3_MSC2)
  USBD_STR_DEF(USBD_MSC2_STR_DESC);
#endif
#if (USBD3_MSC3)
  USBD_STR_DEF(USBD_MSC3_STR_DESC);
#endif

} usbd3_string_descriptor
  = 
{
  { 4, USB_STRING_DESCRIPTOR_TYPE, USBD3_STR_DESC_LANGID },
  USBD_STR_VAL(USBD3_STR_DESC_MAN),
  USBD_STR_VAL(USBD3_STR_DESC_PROD),
#if (USBD3_STR_DESC_SER_EN)
  USBD_STR_VAL(USBD3_STR_DESC_SER),
#endif
#if (USBD3_ADC0)
  USBD_STR_VAL(USBD_ADC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC0_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC0_SIF2_STR_DESC),
#endif
#if (USBD3_ADC1)
  USBD_STR_VAL(USBD_ADC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC1_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC1_SIF2_STR_DESC),
#endif
#if (USBD3_ADC2)
  USBD_STR_VAL(USBD_ADC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC2_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC2_SIF2_STR_DESC),
#endif
#if (USBD3_ADC3)
  USBD_STR_VAL(USBD_ADC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_ADC3_SIF1_STR_DESC),
  USBD_STR_VAL(USBD_ADC3_SIF2_STR_DESC),
#endif

#if (USBD3_CDC0)
  USBD_STR_VAL(USBD_CDC0_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC0_DIF_STR_DESC),
#endif
#if (USBD3_CDC1)
  USBD_STR_VAL(USBD_CDC1_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC1_DIF_STR_DESC),
#endif
#if (USBD3_CDC2)
  USBD_STR_VAL(USBD_CDC2_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC2_DIF_STR_DESC),
#endif
#if (USBD3_CDC3)
  USBD_STR_VAL(USBD_CDC3_CIF_STR_DESC),
  USBD_STR_VAL(USBD_CDC3_DIF_STR_DESC),
#endif

#if (USBD3_HID0)
  USBD_STR_VAL(USBD_HID0_STR_DESC),
#endif
#if (USBD3_HID1)
  USBD_STR_VAL(USBD_HID1_STR_DESC),
#endif
#if (USBD3_HID2)
  USBD_STR_VAL(USBD_HID2_STR_DESC),
#endif
#if (USBD3_HID3)
  USBD_STR_VAL(USBD_HID3_STR_DESC),
#endif

#if (USBD3_MSC0)
  USBD_STR_VAL(USBD_MSC0_STR_DESC),
#endif
#if (USBD3_MSC1)
  USBD_STR_VAL(USBD_MSC1_STR_DESC),
#endif
#if (USBD3_MSC2)
  USBD_STR_VAL(USBD_MSC2_STR_DESC),
#endif
#if (USBD3_MSC3)
  USBD_STR_VAL(USBD_MSC3_STR_DESC),
#endif
};

usbd_desc_t usbd3_desc = {
  (uint8_t *)usbd3_ep0_descriptor,
  (uint8_t *)usbd3_device_descriptor,
  (uint8_t *)usbd3_device_qualifier_fs,
  (uint8_t *)usbd3_device_qualifier_hs,
  (uint8_t *)usbd3_config_descriptor_fs,
  (uint8_t *)usbd3_config_descriptor_hs,
  (uint8_t *)usbd3_other_speed_config_descriptor_fs,
  (uint8_t *)usbd3_other_speed_config_descriptor_hs,
  (uint8_t *)&usbd3_string_descriptor
};

#endif  /* USBD3_PORT */


/*------------------------------------------------------------------------------
 *      USB Device 0..3 Pointers to Descriptors
 *----------------------------------------------------------------------------*/

#if    (defined(LAST_DEVICE_COMPILE) && (USBD_DEV_NUM))

#if     defined(RTE_USB_Device_0)
extern  usbd_desc_t     usbd0_desc;
#endif
#if     defined(RTE_USB_Device_1)
extern  usbd_desc_t     usbd1_desc;
#endif
#if     defined(RTE_USB_Device_2)
extern  usbd_desc_t     usbd2_desc;
#endif
#if     defined(RTE_USB_Device_3)
extern  usbd_desc_t     usbd3_desc;
#endif

usbd_desc_t     *usbd_desc_ptr         [USBD_DEV_NUM]  = { 
#if     defined(RTE_USB_Device_0)
                                       &usbd0_desc
#endif
#if     defined(RTE_USB_Device_1)
                                     , &usbd1_desc
#endif
#if     defined(RTE_USB_Device_2)
                                     , &usbd2_desc
#endif
#if     defined(RTE_USB_Device_3)
                                     , &usbd3_desc
#endif
};

#endif

#endif  /* __NO_USB_LIB_C */
